<?php
class wpecus_downloads_core extends WPeCus_community_core  {
    
}

/** 
 * Downloads section of WP e-Customers plugin by WebTechGlobal.co.uk
 *
 * Core functions for the downloaads section of the plugin. If a function does not
 * apply to the other download function files then put it in here.
 *
 * @package WP e-Customers
 */
 
/**
* Registers custom post type downloads
*
* @package WP e-Customers
* @since 0.0.1 
*/
class wpecus_downloads  {
    
    function wpecus_downloads() {
        add_action('init',array($this,'wpecus_downloads_post_type'));
    }
     
    function wpecus_downloads_post_type() {
        $labels = array(
            'name' => 'Downloads',
            'singular_name' => 'Download',
            'add_new' => 'Add New',
            'all_items' => 'All Downloads',
            'add_new_item' => 'Add New Download',
            'edit_item' => 'Edit Download',
            'new_item' => 'New Download',
            'view_item' => 'View Download',
            'search_items' => 'Search Downloads',
            'not_found' =>  'No Downloads Found',
            'not_found_in_trash' => 'No Downloads found in trash',
            'parent_item_colon' => 'Parent Download:',
            'menu_name' => 'Downloads'
        );
        $args = array(
            'labels' => $labels,
            'description' => "Free and premium digital downloads managed by WP e-Customer plugin",
            'public' => true,
            'exclude_from_search' => false,
            'publicly_queryable' => true,
            'show_ui' => true, 
            'show_in_nav_menus' => true, 
            'show_in_menu' => true,
            'show_in_admin_bar' => true,
            'menu_position' => 25,
            'menu_icon' => null,
            'capability_type' => 'post',
            'hierarchical' => true,
            'supports' => array('title','editor','author','thumbnail','trackbacks','custom-fields','comments','revisions','page-attributes','post-formats'),
            'has_archive' => true,
            'rewrite' => true,
            'query_var' => 'downloads',
            'query_var' => true,
            'can_export' => true
        ); 
        register_post_type('downloads',$args);
    }
    
}
$wpecus_downloads = new wpecus_downloads();

/**
* Adds meta boxes for downloads post type
* 
* Called using add_action()
* 
* @package WP e-Customers
* @since 0.0.1  
*/
function wpecus_add_meta_boxes_downloads() {
    add_meta_box('downloads-meta-latestversion',esc_html__( 'Latest Version', 'example' ),'wpecus_meta_box_downloads_latestversion','downloads','side','default');
    add_meta_box('downloads-meta-stableversion',esc_html__( 'Stable Version', 'example' ),'wpecus_meta_box_downloads_stableversion','downloads','side','default');
    add_meta_box('downloads-meta-mainpremiumfile',esc_html__( 'Main Premium File', 'example' ),'wpecus_meta_box_downloads_mainpremiumfile','downloads','normal','default');
    add_meta_box('downloads-meta-mainfreefiles',esc_html__( 'Main Free File', 'example' ),'wpecus_meta_box_downloads_mainfreefile','downloads','normal','default');    
    add_meta_box('downloads-meta-files',esc_html__( 'Additional Premium Files', 'example' ),'wpecus_meta_box_downloads_files','downloads','normal','default');
    add_meta_box('downloads-meta-freefiles',esc_html__( 'Additional Free Files', 'example' ),'wpecus_meta_box_downloads_freefiles','downloads','normal','default');
}   
add_action( 'add_meta_boxes', 'wpecus_add_meta_boxes_downloads' );

/**
* latest version meta box for downloads post type
* 
* Called in wpecus_add_meta_boxes_downloads()
* 
* @package WP e-Customers
* @since 0.0.1  
*/
function wpecus_meta_box_downloads_latestversion( $object, $box ) { ?>
    <?php wp_nonce_field( basename( __FILE__ ), 'wpecus_downloadsnonce' ); ?>
    <p> 
        <input class="widefat" type="text" name="wpecus_downloads_latestversion" id="latestversion-id" value="<?php echo esc_attr( get_post_meta( $object->ID, '_wpecusdownloads_latestversion', true ) ); ?>" size="30" />
    </p><?php 
}

/**
* stable version meta box for downloads post type
* 
* Called in wpecus_add_meta_boxes_downloads()
* 
* @package WP e-Customers
* @since 0.0.1  
*/
function wpecus_meta_box_downloads_stableversion( $object, $box ) { ?>
    <?php wp_nonce_field( basename( __FILE__ ), 'wpecus_downloadsnonce' ); ?>
    <p> 
        <input class="widefat" type="text" name="wpecus_downloads_stableversion" id="stableversion-id" value="<?php echo esc_attr( get_post_meta( $object->ID, '_wpecusdownloads_stableversion', true ) ); ?>" size="30" />
    </p><?php 
}

/**
* Main Premium File meta box for downloads post type.
* 
* Called in wpecus_add_meta_boxes_downloads()
* 
* @package WP e-Customers
* @since 0.0.1  
*/
function wpecus_meta_box_downloads_mainpremiumfile( $object, $box ) { ?>
    <?php wp_nonce_field( basename( __FILE__ ), 'wpecus_downloadsnonce' ); ?>
    
    <strong>Path</strong>
    <label for="mainpremiumfilepath-id"><input class="widefat" type="text" name="wpecus_downloads_mainpremiumfilepath" id="mainpremiumfilepath-id" value="<?php echo esc_attr( get_post_meta( $object->ID, '_wpecusdownloads_mainpremiumfilepath', true ) ); ?>" size="3" /></label>
    
    <strong>Version</strong>
    <label for="mainpremiumfileversion-id"><input class="widefat" type="text" name="wpecus_downloads_mainpremiumfileversion" id="mainpremiumfileversion-id" value="<?php echo esc_attr( get_post_meta( $object->ID, '_wpecusdownloads_mainpremiumfileversion', true ) ); ?>" size="5" /></label>

    <?php 
} 

/**
* Main Free File meta box for downloads post type.
* 
* Use to offer a free edition, possibly from another source i.e. Wordpress.org
* 
* Called in wpecus_add_meta_boxes_downloads()
* 
* @package WP e-Customers
* @since 0.0.1  
*/
function wpecus_meta_box_downloads_mainfreefile( $object, $box ) { ?>
    <?php wp_nonce_field( basename( __FILE__ ), 'wpecus_downloadsnonce' ); ?>
    
    <strong>Path</strong>
    <label for="mainfreefilepath-id"><input class="widefat" type="text" name="wpecus_downloads_mainfreefilepath" id="mainfreefilepath-id" value="<?php echo esc_attr( get_post_meta( $object->ID, '_wpecusdownloads_mainfreefilepath', true ) ); ?>" size="3" /></label>
    
    <strong>Version</strong>
    <label for="mainfreefileversion-id"><input class="widefat" type="text" name="wpecus_downloads_mainfreefileversion" id="mainfreefileversion-id" value="<?php echo esc_attr( get_post_meta( $object->ID, '_wpecusdownloads_mainfreefileversion', true ) ); ?>" size="5" /></label>

    <?php 
} 

/**
* Premium Files meta box for downloads post type. Add multiple file paths with version, access control.
* 
* Use to offer a history of file versions or addons
* 
* Called in wpecus_add_meta_boxes_downloads()
* 
* @package WP e-Customers
* @since 0.0.1  
*/
function wpecus_meta_box_downloads_files( $object, $box ) { ?>
    <?php wp_nonce_field( basename( __FILE__ ), 'wpecus_downloadsnonce' ); ?>
    <p>Premium files are never offered for free, even if the Cost Type is changed from paid to free.</p>
    <table>
        <tr>
            <td>Name</td>
            <td>File Path</td>
            <td>Version</td>
        </tr>
        
        <?php 
        for($i=1;$i<=3;$i++){?>
            <tr>
                <td><input class="widefat" type="text" name="wpecus_downloads_filetitle_<?php echo $i;?>" id="filetitle<?php echo $i;?>-id" value="<?php echo esc_attr( get_post_meta( $object->ID, '_wpecusdownloads_filetitle_'.$i, true ) ); ?>" size="40" /></td>
                <td><input class="widefat" type="text" name="wpecus_downloads_filepath_<?php echo $i;?>" id="filepath<?php echo $i;?>-id" value="<?php echo esc_attr( get_post_meta( $object->ID, '_wpecusdownloads_filepath_'.$i, true ) ); ?>" size="80" /></td>
                <td><input class="widefat" type="text" name="wpecus_downloads_fileversion_<?php echo $i;?>" id="fileversion<?php echo $i;?>-id" value="<?php echo esc_attr( get_post_meta( $object->ID, '_wpecusdownloads_fileversion_'.$i, true ) ); ?>" size="5" /></td>
            </tr>
        <?php 
        }?>
        
    </table>
    
    <?php 
} 

/**
* Free Files meta box for downloads post type. Add multiple free file paths with version, access control.
* 
* Use to offer a history of file versions or free addons
* 
* Called in wpecus_add_meta_boxes_downloads()
* 
* @package WP e-Customers
* @since 0.0.1  
*/
function wpecus_meta_box_downloads_freefiles( $object, $box ) { ?>
    <?php wp_nonce_field( basename( __FILE__ ), 'wpecus_downloadsnonce' ); ?>
    
    <table>
    
        <tr>
            <td>Name</td>
            <td>File Path</td>
            <td>Version</td>
        </tr>
            
        <?php 
        for($i=1;$i<=3;$i++){?>
            <tr>
                <td><input class="widefat" type="text" name="wpecus_downloads_freefiletitle_<?php echo $i;?>" id="freefiletitle<?php echo $i;?>-id" value="<?php echo esc_attr( get_post_meta( $object->ID, '_wpecusdownloads_freefiletitle_'.$i, true ) ); ?>" size="40" /></td>
                <td><input class="widefat" type="text" name="wpecus_downloads_freefilepath_<?php echo $i;?>" id="freefilepath<?php echo $i;?>-id" value="<?php echo esc_attr( get_post_meta( $object->ID, '_wpecusdownloads_freefilepath_'.$i, true ) ); ?>" size="80" /></td>
                <td><input class="widefat" type="text" name="wpecus_downloads_freefileversion_<?php echo $i;?>" id="freefileversion<?php echo $i;?>-id" value="<?php echo esc_attr( get_post_meta( $object->ID, '_wpecusdownloads_freefileversion_'.$i, true ) ); ?>" size="5" /></td>
            </tr>
        <?php 
        }?>
           
    </table>
    
    <?php 
}  

/**
* Save flag meta box's
* 
* @param mixed $post_id
* @param mixed $post
*/
function wpecus_save_meta_boxes_downloads( $post_id, $post ) {
    $flagmeta_array = array(
        'latestversion','stableversion',
        'mainpremiumfilepath','mainpremiumfileversion',
        'mainfreefilepath','mainfreefileversion',
        'filetitle1','filepath1','fileversion1',
        'filetitle2','filepath2','fileversion2',
        'filetitle3','filepath3','fileversion3',
        'freefiletitle_1','freefilepath_1','freefileversion1',
        'freefiletitle2','freefilepath_2','freefileversion2',
        'freefiletitle3','freefilepath_3','freefileversion3',
    );
    
    /* Verify the nonce before proceeding. */
    if ( !isset( $_POST['wpecus_downloadsnonce'] ) || !wp_verify_nonce( $_POST['wpecus_downloadsnonce'], basename( __FILE__ ) ) )    
        return $post_id;
        
    // check autosave
    if (defined('DOING_AUTOSAVE') && DOING_AUTOSAVE) {
        return $post_id;
    }
 
    // check permissions
    if ( (key_exists('post_type', $post)) && ('downloads' == $post->post_type) ) {
        if (!current_user_can('edit_page', $post_id)) {
            return $post_id;
        }
    } elseif (!current_user_can('edit_post', $post_id)) {
        return $post_id;
    }        

    /* Get the post type object. */
    $post_type = get_post_type_object( $post->post_type );

    /* Check if the current user has permission to edit the post. */
    if ( !current_user_can( $post_type->cap->edit_post, $post_id ) ){
        return $post_id;
    }
 
    // loop through our terms and meta functions
    foreach($flagmeta_array as $key => $term){  
        $new_meta_value = '';
         
        /* Get the meta key. */
        $meta_key = '_wpecusdownloads_' . $term;

        if(isset($_POST['wpecus_downloads_'.$term])){
            $new_meta_value = $_POST['wpecus_downloads_'.$term];    
        }
        
        /* Get the meta value of the custom field key. */
        $meta_value = get_post_meta( $post_id, $meta_key, true );

        if ( $new_meta_value && '' == $meta_value ){
            add_post_meta( $post_id, $meta_key, $new_meta_value, true );# new meta value was added and there was no previous value
        }elseif ( $new_meta_value && $new_meta_value != $meta_value ){
            update_post_meta( $post_id, $meta_key, $new_meta_value );# new meta value does not match the old value, update it
        }elseif ( '' == $new_meta_value && $meta_value ){
            delete_post_meta( $post_id, $meta_key, $meta_value );# no new meta value but an old value exists, delete it
        }
    }
}

add_action( 'save_post', 'wpecus_save_meta_boxes_downloads',10,2 ); 

/**
* Queries the wpecusdownloads table for an existing row with giving user_id and download_id
*
* @package WP e-Customers
* @since 0.0.1
*
* @return the query result from $wpdb->get_row() 
*/
function wtgvb_downloads_get_downloads_row($user_ID,$download_ID) {
    global $wpdb;
    return $wpdb->get_row("
        SELECT * 
        FROM $wpdb->wpecusdownloads 
        WHERE user_id = $user_ID AND download_id = $download_ID
    ");    
}    

/**
* Inserts new row to the wpecusdownloads table to give a user permission to download a premium product

* @package WP e-Customers
* @since 0.0.1
*
* @return $wpdb->query() 
*/
function wtgvb_downloads_paidpermission_insert($user_ID,$download_ID) {
    global $wpdb;

    return $wpdb->query("   
        INSERT INTO
            ".$wpdb->prefix."wpecusdownloads(`user_id`,`download_id`,`paidpermission`,`eventdate`)
        VALUES
            (".$user_ID.",".$download_ID.",1,'".wpecus_date()."')
    ");     
}

/**
* Inserts new row to the wpecusdownloads table to give a user permission to download a free product

* @package WP e-Customers
* @since 0.0.1
*
* @return $wpdb->query() 
*/
function wtgvb_downloads_freepermission_insert($user_ID,$download_ID) {
    global $wpdb;
    return $wpdb->query("   
        INSERT INTO
            ".$wpdb->prefix."wpecusdownloads(`user_id`,`download_id`,`freepermission`,`eventdate`)
        VALUES
            (".$user_ID.",".$download_ID.",1,'".wpecus_date()."')
    ");     
}

/**
* Updates row to give user permission to download a premium product
*
* @package WP e-Customers
* @since 0.0.1
*
* @return $wpdb->query() 
*/
function wtgvb_downloads_grantpermission_update_paid($user_ID,$download_ID) {
    global $wpdb;
    return $wpdb->query("   
        UPDATE ".$wpdb->prefix."wpecusdownloads
        SET paidpermission = 1, eventdate = '".wpecus_date()."'
        WHERE user_id = ".$user_ID." AND download_id = ".$download_ID."
    ");
}

/**
* Updates row to give user permission to download a free product
*
* @package WP e-Customers
* @since 0.0.1
*
* @return $wpdb->query() 
*/
function wtgvb_downloads_grantpermission_update_free($user_ID,$download_ID) {
    global $wpdb;
    return $wpdb->query("   
        UPDATE ".$wpdb->prefix."wpecusdownloads
        SET freepermission = 1, eventdate = '".wpecus_date()."'
        WHERE user_id = ".$user_ID." AND download_id = ".$download_ID."
    ");
}

/**
* Updates row to remove users permission to download premium product
*
* @package WP e-Customers
* @since 0.0.1
*
* @return $wpdb->query() 
*/
function wtgvb_downloads_removepermission_paid($user_ID,$download_ID) {
    global $wpdb;
    return $wpdb->query("   
        UPDATE ".$wpdb->prefix."wpecusdownloads
        SET paidpermission = 0, eventdate = '".wpecus_date()."'
        WHERE user_id = ".$user_ID." AND download_id = ".$download_ID."
    ");
}

/**
* Updates row to remove users permission to download free product
*
* @package WP e-Customers
* @since 0.0.1
*
* @return $wpdb->query() 
*/
function wtgvb_downloads_removepermission_free($user_ID,$download_ID) {
    global $wpdb;
    return $wpdb->query("   
        UPDATE ".$wpdb->prefix."wpecusdownloads
        SET freepermission = 0, eventdate = '".wpecus_date()."'
        WHERE user_id = ".$user_ID." AND download_id = ".$download_ID."
    ");
}

/**
* Queries the wpecusdownloads table, getting all ID for premium files that the user has permission for. 
*
* @package WP e-Customers
* @since 0.0.1
*
* @return $wpdb->get_results() 
*/
function wpecus_downloads_get_paid($user_ID) {
    global $wpdb;
    return $wpdb->get_results("
        SELECT * 
        FROM $wpdb->wpecusdownloads 
        WHERE user_id = $user_ID AND paidpermission = 1
    ");
}

/**
* Queries the wpecusdownloads table, getting all ID for free files that the user has permission for. 
*
* @package WP e-Customers
* @since 0.0.1
*
* @return $wpdb->get_results() 
*/
function wpecus_downloads_get_free($user_ID) {
    global $wpdb;
    return $wpdb->get_results("
        SELECT * 
        FROM $wpdb->wpecusdownloads 
        WHERE user_id = $user_ID AND freepermission = 1
    ");
}

/**
* Gets a post for custom post type "downloads", outputs notice if failure else returns WP array.
*
* @package WP e-Customers
* @since 0.0.1
* 
* @uses get_post()
* @param mixed $download_ID
* @return WP_Post
*/
function wpecus_downloads_get_post($download_ID){
    $post_array = get_post($download_ID);    
    if(!$post_array){
        wpecus_notice_postresult('error','Download ID Does Not Exist','You have entered '.$download_ID.' as a download ID but that ID does not belong to any
        post in this blog. Please try again or report this issue if you feel the ID is correct.');
        return false;
    }
    
    return $post_array;      
}

/**
* Returns the cost type for giving download post and outputs notice if no _wpecusdownloads_costtype.
* Use this function where costtype is required. 
* 
* @package WP e-Customers
* @since 0.0.1
* 
* @param mixed $download_ID
*/
function wpecus_downloads_get_costtype_required($download_ID){
    $costtype = get_post_meta( $download_ID, '_wpecusdownloads_costtype', true );
    if(!$costtype){
        global $wpdb;
        wpecus_notice_postresult('error','Problem Found','The download being assigned to the user does not have a cost type. That is the meta
        value which indicates free,paid or freemium. Please update the download post with a cost type so that it can be stored in the '.$wpdb->prefix.'wpecusdownloads table.');
        return false;    
    }
    return $costtype;
}

/**
* Returns the cost type for giving download post.
* 1. Use wpecus_downloads_get_costtype_required() if exit or return will be done on failure and notice required.
* 2. Meta key: _wpecusdownloads_costtype
* 
* @package WP e-Customers
* @since 0.0.1
* 
* @param get_post_meta()
*/
function wpecus_downloads_get_costtype($download_ID){
    return get_post_meta( $download_ID, '_wpecusdownloads_costtype', true );
}

/**
* Gets post meta value for _wpecusdownloads_mainpremiumfilepath
* 
* @package WP e-Customers
* @since 0.0.1           
* 
* @param mixed $post_ID
*/
function wpecus_downloads_getpaidpath($post_ID){
    return get_post_meta($post_ID,'_wpecusdownloads_mainpremiumfilepath');
}

/**
* Gets post meta value for _wpecusdownloads_mainfreefilepath
* 
* @package WP e-Customers
* @since 0.0.1           
* 
* @param mixed $post_ID
*/
function wpecus_downloads_getfreepath($post_ID){
    return get_post_meta($post_ID,'_wpecusdownloads_mainfreefilepath');
}
?>