<?php
/** 
 * $_POST and $_GET processing for Downloads section of WP e-Customers plugin by WebTechGlobal.co.uk
 *
 * A list of all downloads related processing functions. All functions are in wpecus_downloads_processing_2.php.
 * This file is called alongside all processing files like it but once the required processing function is found
 * no further functions are called. It is a very straight forward and easy to manage system.
 *
 * @package WP e-Customers
 */

class WPeCus_downloads_POSTGET_processing {
    
} 
 
if($continue_processing){
    $continue_processing = wpecus_users_action_grantdownloadpermission(); 
    $continue_processing = wpecus_users_action_removedownloadpermission();  
}


/** 
 * $_POST and $_GET processing for Downloads section of WP e-Customers plugin by WebTechGlobal.co.uk
 *
 * Functions for processing submitted forms, URL actions and download related automation.
 * This file is always included but wpecus_downloads_processing_1.php is called first. That file is called by another
 * function which checks nonce. Then file 1 goes through all processing functions until it finds the correct one. This
 * allows a very simple submission processing system in terms of development.
 *
 * @package WP e-Customers
 */
 

/**
* Allow giving user to download a premium package
* 
* Uses the wpecusdownloads table paidpermission column. Can be used to giving permission for free downloads where restriction applied
* 
* @package WP e-Customers
* @since 0.0.1 
*/
function wpecus_users_action_grantdownloadpermission() {
    if(isset($_POST['wpecus_hidden_panel_name']) && $_POST['wpecus_hidden_panel_name'] == 'grantdownloadpermission'){
        global $wpdb;    
        
        // ensure user ID is valid and user exists
        $user_ID = wtgvb_clean_numeric($_POST['wpecus_userid']);
        
        if(!is_numeric($user_ID)){  
            wpecus_notice_postresult('error','Invalid ID Entered','Wordpress user ID are numeric. Please ensure you enter numbers only.');
            return false;
        }
             
        $user_array = get_user_by('id',$user_ID);
        if(!$user_array){
            wpecus_notice_postresult('error','User ID Does Not Exist','You have entered '.$user_ID.' as a user ID but that ID does not belong to any
            user in this blog. Please try again.');
            return false;
        }
        
        // ensure a download package with the giving ID exists
        $download_ID = wtgvb_clean_numeric($_POST['wpecus_downloadid']);
        
        if(!is_numeric($download_ID)){  
            wpecus_notice_postresult('error','Invalid ID Entered','Wordpress post ID are numeric. Please ensure you enter the post number for the download post type.');
            return false;
        }
  
        $post_array = wpecus_downloads_get_post($download_ID);      
        if(!$post_array){
            return false;
        }        
        
        // we need the meta value for costtype
        $costtype = wpecus_downloads_get_costtype_required($download_ID);
        if(!$costtype){
            return false;    
        }
        
        // check for existing row
        $result = wtgvb_downloads_get_downloads_row($user_ID,$download_ID);

        if($result == false){
            
            if($_POST['wpecus_costtype'] == 'paid'){
                $result = wtgvb_downloads_paidpermission_insert($user_ID,$download_ID); 
                wpecus_notice_postresult('success','Premium Permission Granted','The user has been issued paid permission to download the premium files for the product.');
            }else{
                $result = wtgvb_downloads_freepermission_insert($user_ID,$download_ID);
                wpecus_notice_postresult('success','Free Permission Granted','The user has been issued permission to download the free files for the product.');                
            }
   
        }else{
            
            if($_POST['wpecus_costtype'] == 'paid'){
                $result = wtgvb_downloads_grantpermission_update_paid($user_ID,$download_ID);
                wpecus_notice_postresult('success','Paid Permission Updated','The users permission to download the premium files were updated.');                    
            }else{
                $result = wtgvb_downloads_grantpermission_update_free($user_ID,$download_ID);
                wpecus_notice_postresult('success','Free Permission Updated','Permission to download the free files for the product have been updated.');                
            }
        }
        
        return false;
    }else{
        return true;
    }
} 

/**
* Prevent a user downloading a specific package
*
* @package WP e-Customers
* @since 0.0.1 
*/
function wpecus_users_action_removedownloadpermission() {     
    if(isset($_POST['wpecus_hidden_panel_name']) && $_POST['wpecus_hidden_panel_name'] == 'removedownloadpermission'){
        global $wpdb;     
        
        // ensure user ID is valid and user exists
        $user_ID = wtgvb_clean_numeric($_POST['wpecus_userid']);
        
        if(!is_numeric($user_ID)){  
            wpecus_notice_postresult('error','Invalid ID Entered','Wordpress user ID are numeric. Please ensure you enter numbers only.');
            return false;
        }
             
        $user_array = get_user_by('id',$user_ID);
        if(!$user_array){
            wpecus_notice_postresult('error','User ID Does Not Exist','You have entered '.$user_ID.' as a user ID but that ID does not belong to any
            user in this blog. Please try again.');
            return false;
        }
        
        // ensure download packaage ID valid
        $download_ID = wtgvb_clean_numeric($_POST['wpecus_downloadid']);
        
        if(!is_numeric($download_ID)){  
            wpecus_notice_postresult('error','Invalid ID Entered','Wordpress post ID are numeric. Please ensure you enter the post number for the download post type.');
            return false;
        }        
        
        if($_POST['wpecus_costtype'] == 'paid'){
            $result = wtgvb_downloads_removepermission_paid($user_ID,$download_ID);
            wpecus_notice_postresult('success','Paid Permission Removed','The users permission to download the premium files were withdrawn.');                    
        }else{
            $result = wtgvb_downloads_removepermission_free($user_ID,$download_ID);
            wpecus_notice_postresult('success','Free Permission Removed','Permission to download the free files for the product have been withdrawn.');                
        }
                    
        if($result){
            wpecus_notice_postresult('success','Package Permission Updated','The users permission to download the files has been revoked.');    
        }else{
            wpecus_notice_postresult('error','Please Try Again','Permissions for the user could not be updated, please try again and report this issue if it continues.');    
        }

        return false;
    }else{
        return true;
    }
} 
?>