<?php
/**
* Main users section class
* 
* @package WP e-Customers
* @since 0.0.1
* @version 0.0.1
*/
class WPeCus_users_core extends WPeCus_tickets_core {

    /**
    * initiate a new Wordpress by adding WP e-Customers meta values
    * 1. this could involve getting from phpBB and setting in phpBB
    * 
    * @param mixed $user_ID
    * @return void 
    */
    public function initiateuser($user_ID) {  
        update_user_meta( $user_ID, 'wpecus_mainauthorization','white' );  
        update_user_meta( $user_ID, 'wpecus_users_new','white' );
    }
        
    /**
    * Suspend one or more user accounts by WP user ID
    *
    * Changes user meta wpecus_mainauthorization to "yellow". We are using colors for this to help
    * avoid accidental authorization with incorrect numeric or boolean values.
    *
    * @parameter $user_ID
    */
    public function suspend($user_ID) {
        return update_user_meta( $user_ID, 'wpecus_mainauthorization','yellow' );
    }

    /**
    * Bans user from accessing WebTechGlobal free services and products
    *
    * Set user meta wpecus_mainauthorization to red.
    *
    * @parameter $user_ID
    */
    public function ban($user_ID) {
        return update_user_meta( $user_ID, 'wpecus_mainauthorization','red' );
    }

    /**
    * Authorizes user account to use WebTechGlobal free services and products.
    *
    * Set user meta wpecus_mainauthorization to green.
    *
    * @parameter $user_ID
    * @package WP e-Customers
    * @since 0.0.1
    */
    public function authorize($user_ID) {
        return update_user_meta( $user_ID, 'wpecus_mainauthorization','green' );
    } 
         
    /**
    * checks if email address domain (with top level) has been banned and returns true if it has.
    * 1. ensure valid email address before calling this function
    * 
    * @param mixed $user_email
    */
    public function is_email_domain_banned($user_email) {     
        list( $email_local_part, $email_domain ) = explode( '@', $user_email );
        
        $banned_domains = WPeCustomers::get_banned_domains();
        if(!is_array($banned_domains)){
            $banned_domains = explode(',',$banned_domains);
        }
        if(in_array($email_domain,$banned_domains)){
            return true;
        }
        return false;
    }              
}

/**
* Suspend one or more user accounts by WP user ID
*
* Changes user meta wpecus_mainauthorization to "yellow". We are using colors for this to help
* avoid accidental authorization with incorrect numeric or boolean values.
*
* @parameter $user_ID
* @package WP e-Customers
* @since 0.0.1 
*/
function wpecus_users_suspend($user_ID) {
    update_user_meta( $user_ID, 'wpecus_mainauthorization','yellow' );
}

/**
* Bans user from accessing WebTechGlobal free services and products
*
* Set user meta wpecus_mainauthorization to red.
*
* @parameter $user_ID
* @package WP e-Customers
* @since 0.0.1 
*/
function wpecus_users_ban($user_ID) {
    update_user_meta( $user_ID, 'wpecus_mainauthorization','red' );
}

/**
* Authorizes user account to use WebTechGlobal free services and products.
*
* Set user meta wpecus_mainauthorization to green.
*
* @parameter $user_ID
* @package WP e-Customers
* @since 0.0.1
*/
function wpecus_users_authorize($user_ID) {
    update_user_meta( $user_ID, 'wpecus_mainauthorization','green' );
}  

/**
* Combines multiple commonly used user related values into one array.
*
* Some values may be from Wordpress core itself, some from 3rd party plugins and the rest from any section of this plugin.
* This function is called in the main file. It does not replace Wordpress $current_user but is often used alongside it.
*
* @uses $current_user which is set in the plugins own screen header function by get_currentuserinfo(). It means this function
* must be called within screen files.
* 
* @package WP e-Customers
* @since 0.0.1
* @version 0.0.1
*
* @return array 
*/
function wpecus_users_get_current_user() {
    global $current_user,$wpecus_current_user;
    $wpecus_current_user = array();
    $wpecus_current_user['ID'] = $current_user->ID;
    $wpecus_current_user['security']['mainauthorization'] = get_user_meta( $current_user->ID, 'wpecus_mainauthorization', true );// white,green,yellow,red
    $wpecus_current_user['security']['activationanswer'] = get_user_meta( $current_user->ID, 'wpecus_activationanswer', true );   
    return $wpecus_current_user;
}  

/**
* Ouputs recommended action link (styled as button), for initializing activity scores for users who do not have them.
*
* Queries all users that do not have wpecus_activityscore meta value and if any found the action will be recommended.
*
* @package WP e-Customers
* @since 0.0.1 
*/
function wpecus_recommendedaction_initializeactivityscores(){
    // check activity scores and recommend initialization action if users do not have a value     
    global $wpdb;

    $myrows = $wpdb->get_results( 
    "
        SELECT * FROM ".$wpdb->prefix."users 
        LEFT JOIN  ".$wpdb->prefix."usermeta  ON  ".$wpdb->prefix."users.ID = ".$wpdb->prefix."usermeta.user_id 
        AND ".$wpdb->prefix."usermeta.meta_key = 'wpecus_activityscore' 
        WHERE  (  ".$wpdb->prefix."usermeta.meta_value IS NULL )
    " );
    
    $total = count($myrows);
    
    if($total > 0){?>
        <a href="<?php echo wp_nonce_url( admin_url() . 'admin.php?page=' . $p . $wpecus_tab, 'initializeactivityscore' );?>" title="Initialize activity score for users who do not have a score yet" class="button">Initialize Activity Scores</a><?php     
    }  
}

/**
* Prevents access to screen if no authorization giving. Determines if user has been authorized to use WebTechGlobal and requests user to 
* submit form to request authorization if not.
*
* Automatic authorization on submitting question and answer form by calling wpecus_WP_security_question(). 
*
* @uses wpecus_WP_security_question()
* @package WP e-Customers
* @since 0.0.1 
*/
function wpecus_users_request_authorization() {
    global $wpecus_current_user;
    
    if($wpecus_current_user['ID'] != 1){          
        if(!isset($wpecus_current_user['security']['mainauthorization']) || $wpecus_current_user['security']['mainauthorization'] == '' || $wpecus_current_user['security']['mainauthorization'] == 'white'){
            echo '<h1>Welcome To WebTechGlobal<h1>';
            echo '<h2>Please activate your account to begin using our services.</h2>';
            wpecus_WP_security_question();
            exit;
        }elseif($wpecus_current_user['security']['mainauthorization'] == 'yellow'){
            // suspended
            echo 'Your account has been suspended. This may be temporary and you may contact <a href="mailto:support@webtechglobal.co.uk">support@webtechglobal.co.uk</a> for more information.';
            exit;
        }elseif($wpecus_current_user['security']['mainauthorization'] == 'green'){
            // authorized     
        }elseif($wpecus_current_user['security']['mainauthorization'] == 'red'){
            // banned
            echo 'Your account has been banned. This is only happens if we have investigated your account and found activities to be 
            to be a security or legal risk. Your account cannot be re-activated.';
            exit;
        }
    }
}

/**
* Begins security procedure which involves user selecting answer for question.
*
* Correct answer is stored temporarily in users meta while they answer.
* The number of trys is recorded in user meta.
* If 10 trys are attempted the account is suspended and marked as a spammer for deletion later. 
*
* @package WP e-Customers
* @since 0.0.1 
*/
function wpecus_WP_security_question(){
    global $wpecus_current_user;
    
    // check attempts history, prevent spam submission
    $usersstoredquestion = get_user_meta($wpecus_current_user['ID'],'wpecus_activationquestion',true);
    if(!is_array($usersstoredquestion)){$usersstoredquestion = array();}

    if(isset($usersstoredquestion['attempts']) && is_numeric($usersstoredquestion['attempts']) && $usersstoredquestion['attempts'] > 5){
        echo '<p>Sorry we have tried more than five attempts to authorize your account. Security requires us to prevent
        further submission. Please contact <a href="support@webtechglobal.co.uk">support@webtechglobal.co.uk</a>.';
        exit;
    }

    // display question and answers
    $questions_array = array();
    
    // pick question type
    // other possible types we can add and improve on 'colors','words','animals','anatomy'
    $types_array = array('mathematics');
    $type = array_rand($types_array,1);
    
    // pick question and answer
    switch ($type) {
        case 'mathematics':
            
            $questions_array[0]['q'] = 'How many letters are there in the word Wordpress?';
            $questions_array[0]['a'] = 9;
            $questions_array[0]['p'] = array(5,8,16,18,19);
            $questions_array[1]['q'] = 'How many letters are there in the word Qwerty?';
            $questions_array[1]['a'] = 5;
            $questions_array[1]['p'] = array(7,4,10);
            $questions_array[2]['q'] = 'Count the number of letters in "WebTechGlobal" and select the answer below...';
            $questions_array[2]['a'] = 13;
            $questions_array[2]['p'] = array(1,14,20,23);
            $questions_array[3]['q'] = 'How many numeric characters are there in our plugin named WP e-Customers?';
            $questions_array[3]['a'] = 1;
            $questions_array[3]['p'] = array(2,3,4,5,6);
            $questions_array[4]['q'] = 'What is 7 + 7?';
            $questions_array[4]['a'] = 14;
            $questions_array[4]['p'] = array(13,17,22);
            $questions_array[5]['q'] = 'What is 2 + 5?';
            $questions_array[5]['a'] = 7;
            $questions_array[5]['p'] = array(5,8,10,12);
                         
            break;
        case 'colors':
            
            $questions_array[0]['q'] = '';
            $questions_array[0]['a'] = '';
            $questions_array[0]['p'] = array();
            $questions_array[1]['q'] = '';
            $questions_array[1]['a'] = '';
            $questions_array[1]['p'] = array();
            $questions_array[2]['q'] = '';
            $questions_array[2]['a'] = '';
            $questions_array[2]['p'] = array();
            $questions_array[3]['q'] = '';
            $questions_array[3]['a'] = '';
            $questions_array[3]['p'] = array();
            
            break;
        case 'words':
        
            $questions_array[0]['q'] = '';
            $questions_array[0]['a'] = '';
            $questions_array[0]['p'] = array();
            $questions_array[1]['q'] = '';
            $questions_array[1]['a'] = '';
            $questions_array[1]['p'] = array();
            $questions_array[2]['q'] = '';
            $questions_array[2]['a'] = '';
            $questions_array[2]['p'] = array();
            $questions_array[3]['q'] = '';
            $questions_array[3]['a'] = '';
            $questions_array[3]['p'] = array();
            
            break;       
        case 'animals':
        
            $questions_array[0]['q'] = '';
            $questions_array[0]['a'] = '';
            $questions_array[0]['p'] = array();
            $questions_array[1]['q'] = '';
            $questions_array[1]['a'] = '';
            $questions_array[1]['p'] = array();
            $questions_array[2]['q'] = '';
            $questions_array[2]['a'] = '';
            $questions_array[2]['p'] = array();
            $questions_array[3]['q'] = '';
            $questions_array[3]['a'] = '';
            $questions_array[3]['p'] = array();
            
            break;       
        case 'anatomy':
        
            $questions_array[0]['q'] = '';
            $questions_array[0]['a'] = '';
            $questions_array[0]['p'] = array();
            $questions_array[1]['q'] = '';
            $questions_array[1]['a'] = '';
            $questions_array[1]['p'] = array();
            $questions_array[2]['q'] = '';
            $questions_array[2]['a'] = '';
            $questions_array[2]['p'] = array();
            $questions_array[3]['q'] = '';
            $questions_array[3]['a'] = '';
            $questions_array[3]['p'] = array();
            
            break;        
    }
    
    $question_key = array_rand($questions_array,1);
    
    // store correct answer in meta for comparison during processing
    $usersstoredquestion['type'] = $type;
    $usersstoredquestion['questionkey'] = $question_key;
    $usersstoredquestion['correctanswer'] = $questions_array[$question_key]['a'];
    update_user_meta($wpecus_current_user['ID'],'wpecus_activationquestion',$usersstoredquestion);
        
    echo '<p>To make sure you are not a bot, we require you to answer the security question below. If the question is not answered, your account will be
    suspended pending manual activation which you must request manually also. If you do not request authorization your account data will be removed from
    our database automatically.</p>';
    
    echo '<h3>'.$questions_array[$question_key]['q'].'</h3><br>';?>

    <?php   
    $correct_answer_placement = rand(1,3);
    $correct_answer_placed = false;
    $question_output_counter = 0;    
    foreach($questions_array[$question_key]['p'] as $key => $possible_answer){?>
    
        <a href="<?php echo wp_nonce_url( admin_url() . 'admin.php?page=' . $_GET['page'] . '&wpecusprocess=newuserauthanswer&nonceaction=option'.$possible_answer.'&answer=' . $possible_answer, 'option'.$possible_answer );?>" title="Select <?php echo $possible_answer;?> as your answer to the security question" class="button"><?php echo $possible_answer;?></a> - <?php 
        ++$question_output_counter;
        
        if($question_output_counter > $correct_answer_placement && $correct_answer_placed == false){?>
            <a href="<?php echo wp_nonce_url( admin_url() . 'admin.php?page=' . $_GET['page'] . '&wpecusprocess=newuserauthanswer&nonceaction=option'.$questions_array[$question_key]['a'].'&answer=' . $questions_array[$question_key]['a'], 'option'.$questions_array[$question_key]['a'] );?>" title="Select <?php echo $questions_array[$question_key]['a'];?> as your answer to the security question" class="button"><?php echo $questions_array[$question_key]['a'];?></a> - <?php  
            $correct_answer_placed = true;  
        }      
    }
    
    ?>
    <a href="<?php echo wp_nonce_url( admin_url() . 'admin.php?page=' . $_GET['page'] . '&wpecusprocess=newuserauthanswer&nonceaction=option0&answer=0', 'option0' );?>" title="Select 0 as your answer to the security question" class="button">0</a> 
    <?php          
} 
?>