<?php
/** 
 * Paid edition file for WP e-Customers plugin by WebTechGlobal.co.uk
 *
 * @package WP e-Customers
 * 
 * @author Ryan Bayne | ryan@webtechglobal.co.uk
 */
 
/**
* Determines if an event is due and processes what we refer to as an action (event action) it if true.
* 1. Only call this function once $wpecus_is_free has been checked to avoid over processing
* 2. Early in the function we do every possible check to find a reason not to process
* 3. This function checks all required values exist, else it sets them then returns as this is considered an event action
* 4. This function itself is considered part of the event, we cycle through event types
* 
* Trace
* $wpecus_schedule_array['history']['trace'] is used to indicate how far the this script went before a return.
* This is a simple way to quickly determine where we are arriving.
* 
* Returns
* There are many points of return. The reason is to track any faults where specific values are not being set
* or they are being unset somewhere. It causes the schedule to stop working because the return happens each time
* the value in the schedule array needs correcting. But this also allows us to pick up on the fact that this
* function only gets so far.
* 
* @link http://www.webtechglobal.co.uk/hacking/wpecus_event_check-function-explained
* 
* @return boolean false if no due events else returns true to indicate event was due and full function ran
*/
function wpecus_event_check(){
           
    // do not continue if Wordpress is DOING_AJAX
    if(wpecus_is_processing()){return;}
    
    // free edition does not have automation, this file should not exist in free installation but this is required during development and testing in free mode                                    
    global $wpecus_is_free;
    if($wpecus_is_free){return;/* if your going to hack this, you will need to write some functions also*/}
    
    // do not continue if certain forms are being submitted i.e. uninstallation or re-installation
    if(isset($_POST['wpecus_hidden_panel_name']) && $_POST['wpecus_hidden_panel_name'] == 'partialuninstall'){   
        return;
    } 
    
    wpecus_log_schedule('The schedule is being checked. There should be further log entries explaining the outcome.','schedule being checked',1,'scheduledeventcheck',__LINE__,__FILE__,__FUNCTION__);
    
    // now do checks that we will store the return reason for to allow us to quickly determine why it goes no further                     
    //  get and ensure we have the schedule array
    //  we do not initialize the schedule array as the user may be in the processing of deleting it
    //  do not use wpecus_event_refused as we do not want to set the array
    global $wpecus_schedule_array;
    if(!isset($wpecus_schedule_array) || !is_array($wpecus_schedule_array)){ 
        wpecus_log_schedule('Scheduled events cannot be peformed due to the schedule array of stored settings not existing.','schedule settings missing',1,'scheduledeventcheck',__LINE__,__FILE__,__FUNCTION__);
        return false;
    }
                                   
    // check when last event was run - avoid running two events within 1 minute of each other
    // I've set it here because this function could grow over time and we dont want to go through all the checks PER VISIT or even within a few seconds of each other.
    if(isset($wpecus_schedule_array['history']['lasteventtime'])){
        
        // increase lasteventtime by 60 seconds
        $soonest = $wpecus_schedule_array['history']['lasteventtime'] + 60;//hack info page http://www.webtechglobal.co.uk/hacking/increase-automatic-events-delay-time
        
        if($soonest > time()){
            
            wpecus_event_refused('Has not been 60 seconds since list event'); 
            wpecus_log_schedule('No changed made as it has not been 60 seconds since the last event.','enforcing schedule event delay',1,'scheduledeventcheck',__LINE__,__FILE__,__FUNCTION__);

            return;
        } 
        
    }else{
        
        // set lasteventtime value for the first time
        $wpecus_schedule_array['history']['lasteventtime'] = time();
        $wpecus_schedule_array['history']['lastreturnreason'] = 'The last even time event was set for the first time, no further processing was done.';
        wpecus_update_option_schedule_array($wpecus_schedule_array);

        wpecus_log_schedule('The plugin initialized the timer for enforcing a delay between events. This action is treated as an event itself and no further
        changes are made during this schedule check.','initialized schedule delay timer',1,'scheduledeventcheck',__LINE__,__FILE__,__FUNCTION__);

        return;        
    }                             
                                          
    // is last event type value set? if not set default as dataupdate, this means postcreation is the next event
    if(!isset($wpecus_schedule_array['history']['lasteventtype'])){
        
        $wpecus_schedule_array['history']['lasteventtype'] = 'dataupdate';
        $wpecus_schedule_array['history']['lastreturnreason'] = 'The last event type value was set for the first time';
        wpecus_update_option_schedule_array($wpecus_schedule_array);

        wpecus_log_schedule('The plugin initialized last even type value, this tells the plugin what event was last performed and it is used to
        determine what event comes next.','initialized schedule last event value',1,'scheduledeventcheck',__LINE__,__FILE__,__FUNCTION__);
        
        return;
    }

    // does the "day_lastreset"" time value exist, if not we set it now then return
    if(!isset($wpecus_schedule_array['history']['day_lastreset'])){
        
        $wpecus_schedule_array['history']['day_lastreset'] = time();
        $wpecus_schedule_array['history']['lastreturnreason'] = 'The last daily reset time was set for the first time';
        wpecus_update_option_schedule_array($wpecus_schedule_array);
        
        wpecus_log_schedule('Day timer was set in schedule system. This is the 24 hour timer used to track daily events. It was set, no further action was taking 
        and should only happen once.','24 hour timer set',1,'scheduledeventcheck',__LINE__,__FILE__,__FUNCTION__);
                
        return;
    } 
                                                     
    // does the "hour_lastreset"" time value exist, if not we set it now then return
    if(!isset($wpecus_schedule_array['history']['hour_lastreset'])){
        
        $wpecus_schedule_array['history']['hour_lastreset'] = time();
        $wpecus_schedule_array['history']['lastreturnreason'] = 'The hourly reset time was set for the first time';
        wpecus_update_option_schedule_array($wpecus_schedule_array);
        
        wpecus_log_schedule('Hourly timer was set in schedule system. The time has been set for hourly countdown. No further action was 
        taking. This should only happen once.','one hour timer set',1,'scheduledeventcheck',__LINE__,__FILE__,__FUNCTION__);        
        
        return;
    }    

    // does the hourcounter value exist, if not we set it now then return (this is to initialize the variable)
    if(!isset($wpecus_schedule_array['history']['hourcounter'])){
        
        $wpecus_schedule_array['history']['hourcounter'] = 0;
        $wpecus_schedule_array['history']['lastreturnreason'] = 'The hourly events counter was set for the first time';
        wpecus_update_option_schedule_array($wpecus_schedule_array);
    
        wpecus_log_schedule('Number of events per hour has been set for the first time, this change is treated as an event.','hourly events counter set',1,'scheduledeventcheck',__LINE__,__FILE__,__FUNCTION__);     
           
        return;
    }     
                                 
    // does the daycounter value exist, if not we set it now then return (this is to initialize the variable)
    if(!isset($wpecus_schedule_array['history']['daycounter'])){
        
        $wpecus_schedule_array['history']['daycounter'] = 0;
        $wpecus_schedule_array['history']['lastreturnreason'] = 'The daily events counter was set for the first time';
        wpecus_update_option_schedule_array($wpecus_schedule_array);

        
        wpecus_log_schedule('The daily events counter was not set. No further action was taking. 
        This measure should only happen once.','daily events counter set',1,'scheduledeventcheck',__LINE__,__FILE__,__FUNCTION__);     
                   
        return;
    } 

    // has hourly target counter been reset for this hour - if not, reset now then return (this is an event)
    // does not actually start at the beginning of an hour, it is a 60 min allowance not hour to hour
    $hour_reset_time = $wpecus_schedule_array['history']['hour_lastreset'] + 3600;
    if(time() > $hour_reset_time ){
        
        // reset hour_lastreset value and the hourlycounter
        $wpecus_schedule_array['history']['hour_lastreset'] = time();
        $wpecus_schedule_array['history']['hourcounter'] = 0;
        $wpecus_schedule_array['history']['lastreturnreason'] = 'Hourly counter was reset for another 60 minute period';
        wpecus_update_option_schedule_array($wpecus_schedule_array);
        
        wpecus_log_schedule('Hourly counter has been reset, no further action is taking 
        during this event. This should only happen once every hour.','hourly counter reset',1,'scheduledeventcheck',__LINE__,__FILE__,__FUNCTION__);
                
        return;
    }  

    // have all target counters been reset for today - if not we will reset now and end event check (in otherwords this was the event)
    $day_reset_time = $wpecus_schedule_array['history']['day_lastreset'] + 86400;
    if(time() > $day_reset_time ){
        
        $wpecus_schedule_array['history']['hour_lastreset'] = time();
        $wpecus_schedule_array['history']['day_lastreset'] = time();
        $wpecus_schedule_array['history']['hourcounter'] = 0;
        $wpecus_schedule_array['history']['daycounter'] = 0;
        $wpecus_schedule_array['history']['lastreturnreason'] = 'Daily and hourly events counter reset for a new 24 hours period';
        wpecus_update_option_schedule_array($wpecus_schedule_array); 

        wpecus_log_schedule('24 hours had passed and the daily counter had to be reset. No further action 
        is taking during these events and this should only happen once a day.','daily counter reset',1,'scheduledeventcheck',__LINE__,__FILE__,__FUNCTION__);   
                    
        return;
    }

    // ensure event processing allowed today
    $day = strtolower(date('l'));
    if(!isset($wpecus_schedule_array['days'][$day])){
        wpecus_event_refused('Event processing is has not been permitted for today');

        wpecus_log_schedule('Event processing is not permitted for today. Please check schedule settings 
        to change this.','schedule not permitted today',1,'scheduledeventcheck',__LINE__,__FILE__,__FUNCTION__);
                
        return;    
    } 

    // ensure event processing allow this hour   
    $hour = strtolower( date('G') );
    if(!isset($wpecus_schedule_array['hours'][$hour])){
        wpecus_event_refused('Event processing is has not been permitted for the hour');
        
        wpecus_log_schedule('Processsing is not permitted for the current hour. Please check 
        schedule settings to change this.','hour not permitted',1,'scheduledeventcheck',__LINE__,__FILE__,__FUNCTION__);
                
        return;    
    }

    // ensure hourly limit value has been set
    if(!isset($wpecus_schedule_array['limits']['hour'])){
        
        $wpecus_schedule_array['limits']['hour'] = 1;
        $wpecus_schedule_array['history']['lastreturnreason'] = 'Hourly limit was set for the first time';
        wpecus_update_option_schedule_array($wpecus_schedule_array);
        
        wpecus_log_schedule('The hourly limit value had not been set yet. You can change the limit 
        but the default has been set to one. No further action is taking during this event and this should only happen once.','no hourly limit set',1,'scheduledeventcheck',__LINE__,__FILE__,__FUNCTION__);
                
        return;
    }     
                
    // ensure daily limit value has been set
    if(!isset($wpecus_schedule_array['limits']['day'])){
        
        $wpecus_schedule_array['limits']['day'] = 1;
        $wpecus_schedule_array['history']['lastreturnreason'] = 'Daily limit was set for the first time';
        wpecus_update_option_schedule_array($wpecus_schedule_array);

        wpecus_log_schedule('The daily limit value had not been set yet. It has now been set as one 
        which allows only one post to be created or updated etc. This action should only happen once.','no daily limit set',1,'scheduledeventcheck',__LINE__,__FILE__,__FUNCTION__);
                   
        return;
    }

    // if this hours target has been met return
    if($wpecus_schedule_array['history']['hourcounter'] >= $wpecus_schedule_array['limits']['hour']){
        wpecus_event_refused('The hours event limit/target has been met');
        
        wpecus_log_schedule('The events target for the current hour has been met so no further 
        processing is permitted.','hourly target met',1,'scheduledeventcheck',__LINE__,__FILE__,__FUNCTION__);
                    
        return;        
    }
     
    // if this days target has been met return
    if($wpecus_schedule_array['history']['daycounter'] >= $wpecus_schedule_array['limits']['day']){
        wpecus_event_refused('The days event limit/target has been met');
    
        wpecus_log_schedule('The daily events target has been met for the current 24 hour 
        period (see daily timer counter). No events will be processed until the daily timer reaches 24 hours and is reset.','daily target met',1,'scheduledeventcheck',__LINE__,__FILE__,__FUNCTION__);
                
        return;       
    }
          
    // now decide what type of even should be run, going to return the event type to this function so
    // that we can run tests and debug from here alone. We then pass the value to another function
    $run_event_type = wpecus_event_decide();
    wpecus_log_schedule('Schedule: Next Event Type Returned','The schedule system decided that the next event type 
    is "'.$run_event_type.'".','schedule',__LINE__,__FILE__,__FUNCTION__);
            
    wpecus_log_schedule('The schedule system decided that the next event type is "'.$run_event_type.'".','next event type determined',1,'scheduledeventcheck',__LINE__,__FILE__,__FUNCTION__);
        
    // update $wpecus_schedule_array with decided event type to advance the cycle and increase hourly plus daily counter
    $wpecus_schedule_array['history']['lasteventtype'] = $run_event_type;
    $wpecus_schedule_array['history']['lasteventtime'] = time(); 
    $wpecus_schedule_array['history']['hourcounter'] = $wpecus_schedule_array['history']['hourcounter'] + 1; 
    $wpecus_schedule_array['history']['daycounter'] = $wpecus_schedule_array['history']['daycounter'] + 1;
    wpecus_update_option_schedule_array($wpecus_schedule_array);
    
    // now run the event. We use this function to call the appropriate functions to perform an action.
    // this will return a general outcome (although that might not always be practical so)
    $event_action_outcome = wpecus_event_action($run_event_type); 
    return $event_action_outcome;   
}

/**
* Establishes which event should be run then return it.
* Must call ths function within a function that checks that uses wpecus_DOING_AJAX() first 
* 
* 1. If you add a new event type, you must also update wpecus_tab1_pagecreation.php (Schedule), specifically Events Status panel
* 2. Update wpecus_event_action when adding a new event type
* 3. Update the Event Types panel and add option for new event types
* 4. Update wpecus_form_save_eventtypes
* 
* @link http://www.webtechglobal.co.uk/hacking/event-types
*/
function wpecus_event_decide(){
    global $wpecus_schedule_array;
    
    // first we provide the option to override this process, quicker and allows user to force plugin to focus on specific event type
    $override_event = wpecus_event_decide_focus();// returns false if no override settings in place    
    if($override_event){

        wpecus_log_schedule('The plugins ability to override the next due event type has been applied and then next event forced is '.$override_event.'.',
        'next event type override',1,'scheduledeventcheck',__LINE__,__FILE__,__FUNCTION__);
                        
        return $override_event;
    }    
    
    #######################################################################################
    #                                                                                     #
    #     PROCESSING BEGINS HERE - ALL SHORT CHECKS ARE TO BE PLACED BEFORE THIS POINT    #
    #                                                                                     #
    #######################################################################################
    #                                                                                     #
    # My approach is to cycle through all event types.                                    #
    # None get missed and each event type gets a break.                                   #
    # Each event type can have sub-events, small actions like admin or prep.              #
    #                                                                                     #
    #  http://www.webtechglobal.co.uk/hacking/event-types                                        #
    #                                                                                     #
    # DATA                                                                                #
    #      dataimport (no update)                                                         #
    #      dataupdate (new files)                                                         #
    #                                                                                     #
    # POSTS                                                                               #
    #      postcreation (unused records)                                                  #
    #      postupdate (updated records)                                                   #
    #                                                                                     #
    # TWITTER EVENTS (if activated)                                                       #
    #      twittersend (tweet new post)                                                   #
    #      twitterupdate (tweet that post has been updated)                               #
    #      twitterget (get a tweet reply if any, for displaying in widget)                #
    #                                                                                     #
    #######################################################################################
    if(!isset($wpecus_schedule_array['history']['lasteventtype'])){return 'postcreation';}
    
    // set the event type to be run
    $run_event_type = 'postcreation';
    $bypass = false;// change to true when the next event after the last is not active, then the first available in the list will be the event 
    
    // dataimport -> dataupdate  
    if($wpecus_schedule_array['history']['lasteventtype'] == 'dataimport'){
        if( isset($wpecus_schedule_array['eventtypes']['dataupdate']['switch']) && $wpecus_schedule_array['eventtypes']['dataupdate']['switch'] == true){
             return 'dataupdate';    
        }else{
            $bypass = true; 
        }            
    }
    
    // dataupdate -> postcreation
    if($wpecus_schedule_array['history']['lasteventtype'] == 'dataupdate' || $bypass == true){
        if( isset($wpecus_schedule_array['eventtypes']['postcreation']['switch']) && $wpecus_schedule_array['eventtypes']['postcreation']['switch'] == true){
             return 'postcreation';    
        }else{
            $bypass = true; 
        }            
    }    
    
    // postcreation -> postupdate
    if($wpecus_schedule_array['history']['lasteventtype'] == 'postcreation' || $bypass == true){
        if( isset($wpecus_schedule_array['eventtypes']['postupdate']['switch']) && $wpecus_schedule_array['eventtypes']['postupdate']['switch'] == true){
            return 'postupdate';    
        }else{
            $bypass = true; 
        }            
    }    

    // postupdate -> twittersend
    if($wpecus_schedule_array['history']['lasteventtype'] == 'postupdate' || $bypass == true){
        if( isset($wpecus_schedule_array['eventtypes']['twittersend']['switch']) && $wpecus_schedule_array['eventtypes']['twittersend']['switch'] == true){
             return 'twittersend';    
        }else{
            $bypass = true; 
        }            
    }    

    // twittersend -> twitterupdate
    if($wpecus_schedule_array['history']['lasteventtype'] == 'twittersend' || $bypass == true){
        if( isset($wpecus_schedule_array['eventtypes']['twitterupdate']['switch']) && $wpecus_schedule_array['eventtypes']['twitterupdate']['switch'] == true){
            return 'twitterupdate';    
        }else{
            $bypass = true; 
        }            
    }
    
    // twitterupdate -> twitterget
    if($wpecus_schedule_array['history']['lasteventtype'] == 'twitterupdate' || $bypass == true){
        if( isset($wpecus_schedule_array['eventtypes']['twitterget']['switch']) && $wpecus_schedule_array['eventtypes']['twitterget']['switch'] == true){
            return 'twitterget';    
        }else{
            $bypass = true; 
        }            
    }    
    
    // twitterget -> dataimport
    if($wpecus_schedule_array['history']['lasteventtype'] == 'twitterget' || $bypass == true){
        if( isset($wpecus_schedule_array['eventtypes']['dataimport']['switch']) && $wpecus_schedule_array['eventtypes']['dataimport']['switch'] == true){
            return 'dataimport';    
        }else{
            $bypass = true; 
        }            
    }    
        
    return $run_event_type;        
}

/**
* Determines if user wants the schedule to focus on one specific event type
* @todo MEDIUMPRIORITY, provide an expiry timer option 
* @todo MEDIUMPRIORITY, provide option to alternate between focused event type and all others, must not update lasteventtype value in schedule array or it will loop 
*/
function wpecus_event_decide_focus(){
    global $wpecus_schedule_array;
    if(isset($wpecus_schedule_array['focus']) && $wpecus_schedule_array['focus'] != false){
        return $wpecus_schedule_array['focus'];    
    }
}

/**
* This function calls functions that are not related to scheduling i.e. post creation, data update functions.
* 1. The event type determines what function is to be called. 
* 2. We can add arguments here to call different (custom) functions and more than one action.
* 3. Global settings effect the event type selected, it is always cycled to ensure good admin
* 
* @param mixed $run_event_type, see wpecus_event_decide() for list of event types 
*/
function wpecus_event_action($run_event_type){    
    global $wpecus_schedule_array;
     
    $wpecus_schedule_array['history']['lasteventaction'] = $run_event_type . ' Requested';
    
    // we can override the $run_event_type                          
    // run specific script for the giving action      
    switch ($run_event_type) {
        case "dataimport":  
            
            wpecus_log_schedule('The beginning of the scheduled data import procedure. Other log entries will indicate the result of the entire procedure.'
            ,'Start data import procedure',1,$run_event_type,__LINE__,__FILE__,__FUNCTION__);
                
            $event_complete = false;
            
            // first determine that there is an unfinished data import job - return the job id
            
            // using data import job id - initiate data import from CSV file
            
            // update the lasteventaction value
            
            wpecus_log_schedule('no data import was performed.','end of data import procedure',1,$run_event_type,__LINE__,__FILE__,__FUNCTION__);
                       
            break;  
        case "dataupdate":
            
            wpecus_log_schedule('The beginning of the scheduled data update procedure. Other log entries will indicate the result of the entire procedure.'
            ,'Start data update procedure',1,'updatedata',__LINE__,__FILE__,__FUNCTION__);
                        
            // check all active data import jobs for new version of csv files until one is found
            
            // using the file name            
            
            // update the lasteventaction value
            wpecus_log_schedule('no data update was performed.','end of data update procedure',1,$run_event_type,__LINE__,__FILE__,__FUNCTION__);
            
            break;
        case "postcreation":
        
            wpecus_log_schedule('beginning of scheduled post creation procedure. Further log entries will indicate the outcome.','post creation procedure started',1,$run_event_type,__LINE__,__FILE__,__FUNCTION__);
            
            // find an active post creation project + on drip-feeding + with unused records
            global $wpecus_projectslist_array,$wpecus_schedule_array;

            if(!is_array($wpecus_projectslist_array)){ 
                $event_action = 'No posts created during post creation schedule event as there are no projects.';     
                wpecus_log_schedule('no post creation projects at the moment, no changes were made','post creation procedure ended',1,$run_event_type,__LINE__,__FILE__,__FUNCTION__);               
            }else{
            
                $event_action = 'No posts created, either no active drip-feed projects or no unused records';
                $projectfound_dripfeeding = false;
                
                foreach($wpecus_projectslist_array as $project_code => $project_array){
                    if($project_code != 'arrayinfo'){
                        
                        if(isset($project_array['dripfeeding']) && $project_array['dripfeeding'] == 'on'){
                            
                            $projectfound_dripfeeding = true;
                            
                            // post creation function returns false if no records, keep trying all projects until a post is created
                            $create_post_result = wpecus_create_posts_advanced($project_code,1,'automatic');
                            if(isset($post_creation_array['ID']) && is_numeric($post_creation_array['ID'])){
                               
                                wpecus_log_schedule('the ID of the last post created is '.$post_creation_array['ID'].' and the project code for which this post belongs is '.$project_code.'','schedule indicates posts were created',1,$run_event_type,__LINE__,__FILE__,__FUNCTION__);                                                              
                                break;// foreach: stop looping through projects
                                
                            }else{
                                wpecus_log_schedule('no posts could be created for product with code '.$project_code.'. This log entry is within the schedule system and not the function that creates posts.','no posts were created',1,$run_event_type,__LINE__,__FILE__,__FUNCTION__);
                            } 
                               
                        }
                    }
                }
                
                if(!$projectfound_dripfeeding){
                    wpecus_log_schedule('none of your projects have been activated for drip-feeding posts, no posts were created during the schedule post creation procedure','no applicable projects found',1,$run_event_type,__LINE__,__FILE__,__FUNCTION__);
                }
            }

            // update the lasteventaction value - a very short sentence explaining the specific changes in blog
            $wpecus_schedule_array['history']['lasteventaction'] = $event_action;
                         
            wpecus_log_schedule('this is the last log entry for the schedule post creation procedure. See log entries before this one for the outcome.','procedure has finished',1,$run_event_type,__LINE__,__FILE__,__FUNCTION__);
            
            break;
        case "postupdate":

            /*
            * This has been designed to carry out one method of updating at a time.
            * 1st = Changed Records: search for records updated but not applied (post update function will update the wpecus_applied or at least the sql function called will)
            * 2nd = Meta Key: wpecus_outdated meta is searched, any meta value of 'yes' indicates posts needs to be updated 
            * 3rd = Changed Projected: project array includes a value ['updatecomplete'] which is set to true when no posts can be found with wpecus_updated dates older than ['modified']. This allows an update to all posts. 
            */
            
            global $wpecus_jobtable_array,$wpecus_projectslist_array;
            
            // break now if there are no data import jobs stored
            if(!is_array($wpecus_jobtable_array)){
                wpecus_log_schedule('Schedule post update procedure has begun but there are no data import jobs, no changes made','post update procedure cannot run',1,$run_event_type,__LINE__,__FILE__,__FUNCTION__);
                break;
            }else{
                wpecus_log_schedule('Schedule post update procedure has begun and data import jobs found. Log entries after this one will indicate the outcome.','post update procedure started',1,$run_event_type,__LINE__,__FILE__,__FUNCTION__);            
            }
            
            // due to having multiple methods, we set this variable to bypass methods if updating has been carried out
            // this is to reduce the amount of processing done
            $updating_carried_out = false;

            ############################################################################
            #                                                                          #
            #                     UPDATED JOB TABLE RECORD METHOD                      #
            #                                                                          #
            ############################################################################
            // pick a job table at random, this is to increase the chance of each table in use being used at some point
            $randarray = array_rand($wpecus_jobtable_array);
            
            // set project array value now, we will hold the project array and avoid requesting it multiple times
            $project_array = false;
            $last_project_code = false;// will compare it on each loop, we cannot assume records from the same table were used by the same project
                       
            // find a record updated but not applied since the update
            $record = wpecus_WP_SQL_query_updated_records($wpecus_jobtable_array[$randarray],1);
            
            // if we get a record that has been updated and needs to be applied, we apply it now. Otherwise we check for template changes.
            if($record){
     
                // ensure no further methods are processed
                $updating_carried_out = true; 
                
                foreach($record as $key => $r){
                    // must be numeric id
                    if(is_numeric($r['wpecus_postid'])){
                        // post must have project code as we will need project settings later
                        if(wpecus_WP_SQL_does_post_exist_byid($r['wpecus_postid'])){
                            
                            // get project code from posts meta else we do nothing, we need project settings to re-build the post
                            $project_code = get_post_meta($r['wpecus_postid'], 'wpecus_project_code', true);
                            
                            // if the last project code used does not equal the one we just retrieve get the new project array
                            if($project_code != $last_project_code || !is_array($project_array)){
                                unset($project_array);
                                $project_array = wpecus_get_project_array($last_project_code);
                            }// else we continue using the $project_array already set, this measure helps to avoid querying the project array many times                                   
                            
                            // ensure $meta_value is not false from failure and $project_array
                            if($project_code && is_array($project_array)){
                                
                                // set $last_project_code value which is used to avoid querying project array each iteration
                                $last_project_code = $project_code;        
                                
                                wpecus_post_updatepost($r['wpecus_postid'],$r['wpecus_id'],$project_code);

                                // update the lasteventaction value - a very short sentence explaining the specific changes in blog
                                $wpecus_schedule_array['history']['lasteventaction'] = 'Post with ID ' . $r['wpecus_postid'] . ' was updated using method one';        
                                
                            }       
                        }                                     
                    }
                }
            }
            
            ############################################################################
            #                                                                          #
            #             wpecus_outdated META VALUE SET TO "yes" METHOD             #
            #                                                                          #
            ############################################################################        
            // this method searches the meta with key wpecus_outdated 
            if(!$updating_carried_out){

                // locate a post that have been marked as wpecus_outdated
                $post = wpecus_WP_SQL_get_posts_join_meta('wpecus_outdated','yes',$limit = 1,$select = 'ID');
                if($post){
                    
                    // stop looking for a post to update
                    $updating_carried_out = true;
                    
                    foreach($post as $key => $p){

                        wpecus_post_updatepost($p->ID,get_post_meta($p->ID, 'wpecus_record_id', true),$project_code);
         
                        // update the lasteventaction value - a very short sentence explaining the specific changes in blog
                        $wpecus_schedule_array['history']['lasteventaction'] = 'Post with ID ' . $p->ID . ' was updated using wpecus_outdated method';        
                    }
                }                  
            }

            ############################################################################
            #                                                                          #
            #                  CHANGED PROJECT CONFIGURATION METHOD                    #
            #                                                                          #
            ############################################################################          
            // this method checks $wpecus_projectslist_array[PROJECTCODE]['updatecomplete']
            // if ['updatecomplete'] = false then it means we have yet to determine if some or no posts need updating
            // if we cannot locate just 1 post requiring updating, we change ['updatecomplete'] to true
            // we determine if a post requires updating by comparing $wpecus_projectslist_array[PROJECTCODE]['modified']
            // against the wpecus_updated meta value  
            if(!$updating_carried_out){
                                     
                // locate a project in the project list with ['updatecomplete'] set to false
                if(is_array($wpecus_projectslist_array)){
                    
                    // if we find an outdated project we will store the code
                    $outdated_project_code = false;
                    
                    // loop until we find a project that has a status of ['updatecomplete'] = false
                    foreach($wpecus_projectslist_array as $project_code => $parray){
                        if($project_code != 'arrayinfo'){
                            if(isset($parray['updatecomplete']) && $parray['updatecomplete'] == false){
                                
                                // get the date that the project was modified, posts need to be updated after that date
                                $project_modified = $parray['modified'];
                                
                                // set project code in $outdated_project_code which is used in argument that allows script to continue
                                $outdated_project_code = $project_code;
                                
                                // finally get the project array for the post we are about to udpated simply to ensure it still exists, else we cannot continue
                                $project_array = wpecus_get_project_array($project_code);
                                break;    
                            }   
                        }                     
                    }
                    
                    // only continue if we have a code (do not do return here, we may extend post updating script later)
                    if($outdated_project_code != false && $outdated_project_code != '' && is_array($project_array)){
                    
                        // stop looking for a post to update, we will either update a post or set project as uptodate
                        $updating_carried_out = true;
              
                        // now using the ['modified'] date not stored in $project_modified, we can search for a post not updated
                        // 1. if we find no posts then we change ['updatecomplete'] to true
                        // 2. if we find a post we will update just one for now 
      
                        // run query for a post that has an older wpecus_last_updated date than the projects modified date                        
                        $post = wpecus_SQL_get_posts_oudated($project_modified,$project_code,$limit = 1,$select = 'ID');
                        if($post){
        
                            foreach($post as $key => $p){

                                // get record ID used to create post and ensure the value is numeric (precaution against user editing the value)
                                $record_id = get_post_meta($p->ID, 'wpecus_record_id', true);
                                if($record_id && is_numeric($record_id)){
                                    
                                    // update the post, this function does not just use new records
                                    wpecus_post_updatepost($p->ID,$record_id,$project_code);
                 
                                    // update the lasteventaction value - a very short sentence explaining the specific changes in blog
                                    $wpecus_schedule_array['history']['lasteventaction'] = 'Post with ID ' . $p->ID . ' was updated because project configuration was changed';        
                                }
                            }                         
  
                        }else{
                                    
                            // update the lasteventaction value - a very short sentence explaining the specific changes in blog
                            $wpecus_schedule_array['history']['lasteventaction'] = 'Schedule determine posts for project with code '.$outdated_project_code.' are all updated';
                                                                
                            // set the ['updatecomplete'] value to true to indicate all posts have been updated since project settings changed                            
                            $wpecus_projectslist_array[$outdated_project_code]['arrayinfo'] = wpecus_ARRAY_arrayinfo_set(__LINE__,__FUNCTION__,__FILE__,false,false);
                            $wpecus_projectslist_array[$outdated_project_code]['updatecomplete'] = true;
                            wpecus_update_option_postcreationproject_list($wpecus_projectslist_array);    
                        }                       
                    }
                }                
            }
            
            wpecus_log_schedule('Schedule post update procedure has complete. See previous log entries to determine the outcome.','post upate procedure ended',1,$run_event_type,__LINE__,__FILE__,__FUNCTION__);
                 
            break;
        case "twittersend":
            wpecus_log_schedule('Scheduled procedure has started for sending tweets, see further log entries for the actual outcome.','sending tweets procedure started',1,$run_event_type,__LINE__,__FILE__,__FUNCTION__);
                             
            // confirm tweet ready
            
            // find project post not tweeted
            
            // create tweet string
            
            // send tweet            

            // update the lasteventaction value
            
            wpecus_log_schedule('Sending tweets procedure in the schedule system has complete. See log entries before this one for the outcome.','sending tweets has finished',1,$run_event_type,__LINE__,__FILE__,__FUNCTION__);         
            break;
    }
   
    wpecus_update_option_schedule_array($wpecus_schedule_array);    
}  
?>
