<?php
class WPeCus_core extends WPeCus_web_core {
    
} 

class WPeCustomers extends WPeCus_core {
    protected
        // Actions and filters: this approach was taking from WP United and found while we researched WP to phpBB bridges, it suits our sections approach nicely
        // Format: array( event | function in this class(in an array if optional arguments are needed) | loading circumstances)
        $plugin_actions = array(
            
            /*
                Examples - last value are the sections that the action apply to
                    array('plugins_loaded',                     'init_plugin',                                'all'), 
                    array('wp_head',                             'add_scripts',                                'forum'),
                    array('wp_footer',                             'add_footer_output',                        'affiliates'),
            */
            
            array('registration_errors',                     array('registration_submission', 10, 3),                                'all'),
        ),

        $plugin_filters = array(
        
            /*
                Examples - last value are the sections the filter apply to
                    array('plugin_row_meta',                     array('examplefunction1', 10, 2),         'all'),
                    array('page_link',                             array('examplefunction2', 10, 2),             'downloads'),
                    array('admin_footer_text',                     'examplefunction3',                         'monetization'),
                    
            */
        );
    
    /**
    * This class is being introduced gradually, we will move various lines and config functions from the main file to load here eventually
    */
    public function __construct() {
        $this->WPeCus_core;
    }
      
    /**
     * Initialises the plugin from WordPress
     * @return void
     */
    public function wp_init() {
        $this->add_actions();
        $this->add_filters();
        unset($this->plugin_actions, $this->plugin_filters);
    }
    
    protected function add_actions() {        
        foreach($this->plugin_actions as $actionArray) {     
            list($action, $details, $whenToLoad) = $actionArray;

            if(!$this->filteraction_should_beloaded($whenToLoad)) {      
                continue;
            }
                 
            switch(count((array)$details)) {         
                case 3:
                    add_action($action, array($this, $details[0]), $details[1], $details[2]);     wpecus_line_dump(__LINE__);
                break;
                case 2:
                    add_action($action, array($this, $details[0]), $details[1]);   wpecus_line_dump(__LINE__);
                break;
                case 1:
                default:
                    add_action($action, array($this, $details));
            }
        }    
    }
    
    protected function add_filters() {
        foreach($this->plugin_filters as $filterArray) {
            list($filter, $details, $whenToLoad) = $filterArray;
            
            if(!$this->filteraction_should_beloaded($whenToLoad)) {
                continue;
            }
            
            switch(sizeof((array)$details)) {
                case 3:
                    add_filter($filter, array($this, $details[0]), $details[1], $details[2]);
                break;
                case 2:
                    add_filter($filter, array($this, $details[0]), $details[1]);
                break;
                case 1:
                default:
                    add_filter($filter, array($this, $details));
            }
        }    
    }    
    
    /**
    * Should the giving action or filter be loaded?
    * 1. we can add security and check settings per case
    * 2. each case is a section and we use this approach to load action or filter for specific section
    * 3. In early development all sections are loaded, this function is prep for a modular plugin
    * 4. addons will require core functions like this to be updated rather than me writing dynamic functions for any possible addons
    *  
    * @param mixed $whenToLoad
    */
    private function filteraction_should_beloaded($whenToLoad) {
        global $wpecus_sections_array;

        if($whenToLoad != 'all' && !in_array($wpecus_sections_array) ) {
            return false;
        }
                
        switch($whenToLoad) {
            case 'all':
                return true;
            break;
            case 'affiliates':
                return true;    
            break;
            case 'clients':
                return true;
            break;
            case 'community':
                return true;
            break;
            case 'downloads':
                return true;
            break;
            case 'faq':
                return true;
            break;
            case 'finance':
                return true;
            break;
            case 'forum':
                return true;
            break;
            case 'lab':
                return true;
            break;
            case 'monetize':
                return true;
            break;
            case 'questions':
                return true;
            break;
            case 'rep':
                return true;
            break;     
            case 'tickets':
                return true;
            break;
            case 'users':
                return true;
            break;
        }

        return true;

    }    
    
    /**
    * comma separated domains
    */
    public function get_banned_domains() {
        return get_option('wpecus_banned_domains');
    }
    
    /**
    * Use add_banned_domains() first if you want to add domains to the list
    * 
    * @param mixed $domains_list
    * @return bool
    */
    public function update_banned_domains($domains_list) {
        return update_option('wpecus_banned_domains',$domains_list);
    }
    
    /**
    * Add new domains to the banned list
    * 1. can submit a single domain i.e. spammers.com
    * 2. can submit full address i.e. http://www.morespammmers.com (will strip this down)
    * 3. can submit an array i.e. array('toomanyspammers.ru','www.spammersrus.ch','http://www.spamcity.ru')
    * 
    * @param mixed $domains array or comma separated string, if one domain being added do not use commas
    */
    public function add_banned_domains($new_domains) {
        $domains_list = WPeCustomers::get_banned_domains();
        
        $domains_array = explode(',',$domains_list);
        
        // establish an array
        if(!is_array($new_domains)){
            $new_domains_array = explode(',',$new_domains);
        }else{
            $new_domains_array = $new_domains;
        }
        
        // use array to avoid duplicates but add new domians to the comma separated string
        foreach($new_domains_array as $dom){
            if(is_string($dom) && !in_array($dom,$domains_array)){
                $domains_list .= ',' . WPeCustomers::domain_only($dom);
            }
        }        
        
        return WPeCustomers::update_banned_domains($domains_list);
    }
    
    public function domain_only ( $url ) { 
        $url = trim($url);
        $url = preg_replace("/^(http:\/\/)*(www.)*/is", "", $url); 
        $url = preg_replace("/\/.*$/is" , "" ,$url); 
        return $url; 
    }
    
    // user_register action
    public function registration_submission($errors,$sanitized_user_login,$user_email) {

        // block those spammy domains
        $banned = WPeCus_core::is_email_domain_banned($user_email);  
                               
        if($banned === true){     
            wp_die('Your email addresses domain is banned on this website, no registration is permitted.');
        }
                    
        // block the trolls IP addresses
        
        // prevent use of banned usernames
        
        // set some values WP e-Customers usually needs very soon                        
        //WPeCus_core::initiateuser($user_id);            # move this to an after use created action
        
        return $errors;
    }  
}

class WPeCus_ui extends WPeCus_web_ui {
    
}  
?>
