<?php
/** 
 * Form processing functions for WP e-Customers plugin 
 * 
 * @link by http://www.webtechglobal.co.uk
 * 
 * @author Ryan Bayne | ryan@webtechglobal.co.uk
 *
 * @package WP e-Customers
 */

// include processing functions for all sections
require_once(WTG_WPECUS_PATH . 'functions/sections/clients/wpecus_clients_processing.php');
require_once(WTG_WPECUS_PATH . 'functions/sections/users/wpecus_users_processing.php');
require_once(WTG_WPECUS_PATH . 'functions/sections/downloads/wpecus_downloads_processing.php');
require_once(WTG_WPECUS_PATH . 'functions/sections/lab/wpecus_lab_processing.php');
require_once(WTG_WPECUS_PATH . 'functions/sections/affiliates/wpecus_affiliates_processing.php');
require_once(WTG_WPECUS_PATH . 'functions/sections/rep/wpecus_rep_processing.php');
require_once(WTG_WPECUS_PATH . 'functions/sections/tickets/wpecus_tickets_processing.php');
require_once(WTG_WPECUS_PATH . 'functions/sections/community/wpecus_community_processing.php');
require_once(WTG_WPECUS_PATH . 'functions/sections/monetize/wpecus_monetize_processing.php');
require_once(WTG_WPECUS_PATH . 'functions/sections/finance/wpecus_finance_processing.php');
require_once(WTG_WPECUS_PATH . 'functions/sections/forum/wpecus_forum_processing.php');
require_once(WTG_WPECUS_PATH . 'functions/sections/questions/wpecus_questions_processing.php');
require_once(WTG_WPECUS_PATH . 'functions/sections/web/wpecus_web_processing.php');

function wpecus_form_delete_persistentnotice(){
    if(isset($_POST['wpecus_post_deletenotice']) && $_POST['wpecus_post_deletenotice'] == true){
         
        global $wpecus_persistent_array;

        foreach($wpecus_persistent_array['notifications'] as $key => $notice){
            if($notice['id'] == $_POST['wpecus_post_deletenotice_id']){
                unset($wpecus_persistent_array['notifications'][$key]);
            }            
        }
        
        wpecus_option('wpecus_notifications','update',$wpecus_persistent_array);        
                  
        return false;
    }else{
        return true;
    }     
}                  

/**
* Deletes one or more database tables
*/
function wpecus_form_drop_database_tables(){
    if(isset( $_POST['wpecus_hidden_pageid'] ) && $_POST['wpecus_hidden_pageid'] == 'data' && isset($_POST['wpecus_hidden_panel_name']) && $_POST['wpecus_hidden_panel_name'] == 'createddatabasetableslist'){

        if(!isset($_POST["wpecus_table_array"])){
            wpecus_notice('You did not select any database tables. Check the boxes for the tables you want to delete.','warning','Large','No Tables Deleted','','echo');
            return false;
        }else{
            
            global $wpdb,$wpecus_jobtable_array,$wpecus_dataimportjobs_array;
   
            foreach($_POST["wpecus_table_array"] as $key => $table_name){
                
                // if table is in use by a data import job we do not delete it, the job must be deleted first
                $code = str_replace('wpecus_','',$table_name);   
                
                if(isset($wpecus_dataimportjobs_array[$code])){
                    wpecus_notice('Table named '.$table_name.' is still used by Data Import Job named '.$wpecus_dataimportjobs_array[$code]['name'].'. Please delete the job first then delete the database table.','warning','Large','Cannot Delete ' . $table_name,'','echo');
                }else{
                    
                    wpecus_SQL_drop_dataimportjob_table($table_name); 
                                            
                    wpecus_notice('','success','Small','Database Table Deleted: '.$table_name,'','echo');
                }  
            }
        }

        return false;
    }else{
        return true;
    }          
}          
                                                    
/**
* Creates the csv file folder in the wp-content path
*/
function wpecus_form_deletecontentfolder(){
    if(isset($_POST['wpecus_contentfolder_delete'])){ 
        // this function does the output when set to true for 2nd parameter
        wpecus_delete_contentfolder(WTG_WPECUS_CONTENTFOLDER_DIR,true);    
        return false; 
    }else{
        return true;
    }    
} 

/**
* Creates the csv file folder in the wp-content path
*/
function wpecus_form_createcontentfolder(){
    if(isset($_POST['wpecus_contentfolder_create'])){ 
        // this function does the output when set to true for 2nd parameter
        wpecus_install_contentfolder_wpecustomerscontent(WTG_WPECUS_CONTENTFOLDER_DIR,true);    
        return false;
    }else{
        return true;
    }    
}    

/**
* Activate or disable text spinning 
*/
function wpecus_form_textspinning_switch(){
    if(isset( $_POST[WTG_WPECUS_ABB.'hidden_pageid'] ) && $_POST[WTG_WPECUS_ABB.'hidden_pageid'] == 'projects' && isset($_POST[WTG_WPECUS_ABB.'hidden_panel_name']) && $_POST[WTG_WPECUS_ABB.'hidden_panel_name'] == 'textspinningswitch'){
        global $wpecus_textspin_array,$wpecus_adm_set;

        if(isset($wpecus_textspin_array['settings']['status']) && $wpecus_textspin_array['settings']['status'] == true){
            $wpecus_textspin_array['settings']['status'] = false;
            wpecus_notice_postresult('success','Text Spin Features ','You have disabled
            text spin features. This will reduce processing during post creation and hide options.
            However at does not disable shortcodes already in use.');       
        }else{
            $wpecus_textspin_array['settings']['status'] = true;
            wpecus_notice_postresult('success','Text Spin Features On','You have activated
            text spin features. It will put all post content through more processing during post creation
            and display more accordion panels.');                          
        }
        
        wpecus_update_option_textspin($wpecus_textspin_array);        
        
        return false;
    }else{
        return true;
    }     
}   

/**
* Saves easy configuration questions
*/
function wpecus_form_save_easyconfigurationquestions(){
    if(isset( $_POST['wpecus_hidden_pageid'] ) && $_POST['wpecus_hidden_pageid'] == 'main' && isset($_POST['wpecus_hidden_panel_name']) && $_POST['wpecus_hidden_panel_name'] == 'easyconfigurationquestions'){
        global $wpecus_adm_set,$wpecus_ecq_array;
        
        // save answers
        foreach($wpecus_ecq_array as $key => $q){
            // if $_POST value for this question
            if(isset($_POST['wpecus_'.$key])){
                $wpecus_adm_set['ecq'][$key] = $_POST['wpecus_'.$key];  
            }  
        }

        /**
        * Where there are existing settings we will update those settings to help make this
        * function central to the ECQ system. However if there is no existing system, consider
        * makign one. If one is not required required, then use the ['ecq'] value itself in the appropriate
        * script. 
        */
        
        ### TODO:HIGHPRIORITY, some question processing requires other systems to be complete, when ready please complete the processing below
        
        // 102 - Data Tables (all or wpecustomers)
        if($wpecus_adm_set['ecq'][102] == 'yes'){$wpecus_adm_set['dbtablesscope'] = 'all';}else{$wpecus_adm_set['dbtablesscope'] = 'wpecustomers';}
        // 103 - Schedule/Automation Trigger (admin also or public only)
        if($wpecus_adm_set['ecq'][103] == 'yes'){/* TODO:MEDIUMPRIORITY, was a setting but it was removed, we will apply this later */}
        // 104 - New version tweets
        if($wpecus_adm_set['ecq'][104] == 'yes'){/* return answer using User Experience Program when it is complete */}                                   
        // 105 - Use log system
        if($wpecus_adm_set['ecq'][105] == 'yes'){/* TODO:MEDIUMPRIORITY, complete when new log to database complete */}
        // 106 - RSS feed tutorials and videos
        if($wpecus_adm_set['ecq'][106] == 'yes'){/* return answer using User Experience Program when it is complete */}
        // 107 - Encoding
        if($wpecus_adm_set['ecq'][107] == 'yes'){/* process once encoding functions re-vised and complete */}
        // 108 - Hide jQuery theme and use Wordpress CSS
        if($wpecus_adm_set['ecq'][108] == 'yes'){update_option('wpecus_theme','wordpresscss'); }  
        // 109 - Only one user
        if($wpecus_adm_set['ecq'][109] == 'yes'){/* a) screen permissions panel hidden b) page permissions panel hidden */}
        // 110 - Do you need to merge two or more CSV files into one set of data?
        if($wpecus_adm_set['ecq'][110] == 'no'){/* no setting - answer will be used to hide multiple file related features */}
        // 111 - Table to table
        if($wpecus_adm_set['ecq'][111] == 'yes'){/* TODO:MEDIUMPRIORITY, apply this when we have controls for hiding and displaying screens, change those settings here. This will prevent the screen being displayed again if user re-installs menu array */}
        // 112 - Two or more database tables
        if($wpecus_adm_set['ecq'][112] == 'yes'){/* radios instead of checkboxes are set on table selection when creating new project */}
        // 113 - Content template rules
        if($wpecus_adm_set['ecq'][113] == 'yes'){/* hides panels related to contact template rules */}
        // 114 - Post dates
        if($wpecus_adm_set['ecq'][114] == 'yes'){/* TODO:MEDIUMPRIORITY,when screen display settings finished hide post dates tab screen on yes */}
        // 115 - Post updating 
        if($wpecus_adm_set['ecq'][115] == 'yes'){/* TODO:MEDIUMPRIORITY, once screen options complete hide updating screens */}  
        // 116 - Social networking
        if($wpecus_adm_set['ecq'][116] == 'yes'){/* TODO:MEDIUMPRIORITY, return answer using User Experience Program when it is complete */}
        // 117 - URL Cloaking
        if($wpecus_adm_set['ecq'][117] == 'yes'){/* TODO:MEDIUMPRIORITY, once screen options complete hide related screens */}
        // 118 - Text Spinning
        if($wpecus_adm_set['ecq'][118] == 'yes'){/* TODO:MEDIUMPRIORITY, once screen options complete hide related screens */}
        // 119 - Create authors
        if($wpecus_adm_set['ecq'][119] == 'yes'){/* TODO:MEDIUMPRIORITY, once screen options complete hide related screens */}
        // 120 - Display info or video buttons in panels?
        if($wpecus_adm_set['ecq'][120] == 'yes'){$wpecus_adm_set['interface']['panels']['supportbuttons']['status'] = 'display';}
        // 121 - One or more projects
        if($wpecus_adm_set['ecq'][121] == 'yes'){/* TODO:MEDIUMPRIORITY, return answer using User Experience Program when it is complete */}
        // 122 - User Experience Program
        if($wpecus_adm_set['ecq'][122] == 'yes'){/* TODO:HIGHPRIORITY, this is a system and all systems need to be 90% comlete asap, then activate it here */}
        // 123 - Register domain as user
        if($wpecus_adm_set['ecq'][123] == 'yes'){/* TODO:MEDIUMPRIORITY, return answer using User Experience Program when it is complete, also setup a way for user to manually submit domain with URL GET at our end, users should have incentives to do so once credits system in place as a service */}
        // 124 - Support account using wordpressplugin@wpecustomers.com
        if($wpecus_adm_set['ecq'][124] == 'yes'){/* TODO:MEDIUMPRIORITY, create support account and pass details to us using SOAP once web services complete */}

        wpecus_update_option_adminsettings($wpecus_adm_set);
   
        wpecus_notice('Your answers for the Easy Configuration Questions have been saved. Please remember
        that this may hide features, display new features or change the way a feature operates.',
        'success','Large','Answers Saved','','echo');
           
        return false;
    }else{
        return true;
    }       
}     
     
function wpecus_form_reinstall_databasetables(){
    if(isset($_POST[WTG_WPECUS_ABB.'hidden_pageid']) && $_POST[WTG_WPECUS_ABB.'hidden_pageid'] == 'main' && isset($_POST[WTG_WPECUS_ABB.'hidden_panel_name']) && $_POST[WTG_WPECUS_ABB.'hidden_panel_name'] == 'reinstalldatabasetables'){
        
        $tables = 0;
        $result = true;
        $result = wpecus_INSTALL_reinstall_databasetables();++$tables;
        if($result){
            wpecus_n_postresult('success','Tables Re-Installed Successfully','A total of '.$tables.' tables were deleted then created. All data in the original tables is lost.');
        }else{   
            wpecus_n_postresult('error','Tables Re-Installation Failed','A total of '.$tables.' tables were meant to be deleted then installed again but a problem was detected. Please
            try again then investigate the issue. It may be a single table causing this issue. Please report it and we will be happy to help.');
        }
        
        return false;// must go inside $_POST validation, not at end of function         
    }else{
        return true;
    }      
} 

/**
* Data panel one settings
*/
function wpecus_form_save_settings_datapanelone(){
         if(isset( $_POST['wpecus_hidden_pageid'] ) && $_POST['wpecus_hidden_pageid'] == 'main' && isset($_POST['wpecus_hidden_panel_name']) && $_POST['wpecus_hidden_panel_name'] == 'panelone'){
        
        global $wpecus_adm_set;

        $wpecus_adm_set['encoding']['type'] = $_POST['wpecus_radiogroup_encoding'];
        $wpecus_adm_set['admintriggers']['newcsvfiles']['status'] = $_POST['wpecus_radiogroup_detectnewcsvfiles'];
        $wpecus_adm_set['admintriggers']['newcsvfiles']['display'] = $_POST['wpecus_radiogroup_detectnewcsvfiles_display'];
        $wpecus_adm_set['postfilter']['status'] = $_POST['wpecus_radiogroup_postfilter'];          
        $wpecus_adm_set['postfilter']['tokenrespin']['status'] = $_POST['wpecus_radiogroup_spinnertokenrespin'];        
 
        wpecus_update_option_adminsettings($wpecus_adm_set);

        wpecus_n_postresult('success','Data Related Settings Saved','We
        recommend that you monitor the plugin for a short time and ensure
        your new settings are as expected.');
        
        return false;
    }else{
        return true;
    } 
}     

function wpecus_form_extensionsettings(){
    if(isset( $_POST['wpecus_hidden_pageid'] ) && $_POST['wpecus_hidden_pageid'] == 'main' && isset($_POST['wpecus_hidden_panel_name']) && $_POST['wpecus_hidden_panel_name'] == 'extensionsettings'){

        // save extension status
        update_option('wpecus_extensions',$_POST['wpecus_radiogroup_extensionstatus']);
        
        //wpecus_update_option_adminsettings($wpecus_adm_set);

        wpecus_n_postresult('success','Extension Settings Saved','Please ensure your blog operates as you expect with or without certain extensions.');        
        
        wp_redirect( get_bloginfo('url') . '/wp-admin/admin.php?page=wpecustomers' );
        exit;
                                 
        return false;
    }else{
        return true;
    }        
}  
 
function wpecus_form_save_screenpermissionssettings(){
    if(isset( $_POST['wpecus_hidden_pageid'] ) && $_POST['wpecus_hidden_pageid'] == 'main' && isset($_POST['wpecus_hidden_panel_name']) && $_POST['wpecus_hidden_panel_name'] == 'screenpermissions'){
        
        global $wpecus_mpt_arr;

        // loop through tabs, this is the same loop build as on the capabilities interface itself
        $menu_id = 0;
        foreach($wpecus_mpt_arr['menu'] as $page_name => $page_array){

            foreach($page_array['tabs'] as $key => $tab_array){
                
                if(isset($tab_array['display']) && $tab_array['display'] != false){
                                        
                    // is post value set for current tab
                    if(isset($_POST['wpecus_capabilitiesmenu_'.$page_name.'_'.$key.''])){
         
                        // update capability setting for screen
                        $wpecus_mpt_arr['menu'][$page_name]['tabs'][$key]['permissions']['customcapability'] = $_POST['wpecus_capabilitiesmenu_'.$page_name.'_'.$key.''];
                    }

                    ++$menu_id; 
                }        
            }
        }        

        wpecus_option('wpecus_tabmenu','update',$wpecus_mpt_arr);

        wpecus_notice_postresult('success','Screen Permissions Saved','Your saved screen permissions
        may hide or display screens for users. We recommend that you access your blog using applicable 
        user accounts to test your new permissions.');
        
        return false;
    }else{
        return true;
    }       
} 

function wpecus_form_save_pagepermissionssettings(){
    if(isset( $_POST['wpecus_hidden_pageid'] ) && $_POST['wpecus_hidden_pageid'] == 'main' && isset($_POST['wpecus_hidden_panel_name']) && $_POST['wpecus_hidden_panel_name'] == 'pagepermissions'){

        global $wpecus_mpt_arr;

        foreach($wpecus_mpt_arr as $page_name => $page_array){
            
            if(isset($_POST['wpecus_capabilitiesmenu_'.$page_name.'_99'])){
                $wpecus_mpt_arr['menu'][$page_name]['permissions']['customcapability'] = $_POST['wpecus_capabilitiesmenu_'.$page_name.'_99'];    
            }
 
        }        

        wpecus_option('wpecus_tabmenu','update',$wpecus_mpt_arr); 
        
        wpecus_notice_postresult('success','Page Permissions Saved','Your saved page permissions
        may hide or display the plugins pages for users. We recommend that you access your blog using applicable 
        user accounts to test your new permissions.');
                
        return false;
    }else{
        return true;
    }       
}  
     
/**
* Create a data rule for replacing specific values after import 
*/
function wpecus_form_save_eventtypes(){
    if(isset( $_POST['wpecus_hidden_pageid'] ) && $_POST['wpecus_hidden_pageid'] == 'main' && isset($_POST['wpecus_hidden_panel_name']) && $_POST['wpecus_hidden_panel_name'] == 'eventtypes'){
        global $wpecus_schedule_array;   

        $wpecus_schedule_array['eventtypes']["postcreation"]['switch'] = $_POST["wpecus_eventtype_postcreation"];
        $wpecus_schedule_array['eventtypes']["postupdate"]['switch'] = $_POST["wpecus_eventtype_postupdate"];
        $wpecus_schedule_array['eventtypes']["dataimport"]['switch'] = $_POST["wpecus_eventtype_dataimport"];
        $wpecus_schedule_array['eventtypes']["dataupdate"]['switch'] = $_POST["wpecus_eventtype_dataupdate"];
        $wpecus_schedule_array['eventtypes']["twittersend"]['switch'] = $_POST["wpecus_eventtype_twittersend"];
        $wpecus_schedule_array['eventtypes']["twitterupdate"]['switch'] = $_POST["wpecus_eventtype_twitterupdate"];
        $wpecus_schedule_array['eventtypes']["twitterget"]['switch'] = $_POST["wpecus_eventtypes_twitterget"];

        wpecus_update_option_schedule_array($wpecus_schedule_array);
        
        wpecus_notice('Schedule event types have been saved, the changes will have an effect on the types of events run, straight away.','success','Large','Schedule Event Types Saved','','echo');
        return false;
    }else{
        return true;
    }          
}       
?>