<?php
/** 
 * Function file for WP e-Customers plugin 
 * 
 * @link by WebTechGlobal.co.uk
 * 
 * @author Ryan Bayne | ryan@webtechglobal.co.uk
 *
 * @package WP e-Customers
 */

/**
* Loads CSS for plugin not core
* 
* @param string $side, admin, public
* @param mixed $wpecus_css_side_override, makes use of admin lines in front-end of blog
*/
function wpecus_css_core($side = 'admin',$wpecus_css_side_override = false){        
    include_once(WTG_WPECUS_PATH . 'wtg-core/wp/css/wtgcore_css_parent.php');
}                  

/**
* When request will display maximum php errors including Wordpress errors 
*/
function wpecus_debugmode(){
    global $wpecus_debug_mode;
    if($wpecus_debug_mode){
        global $wpdb;
        ini_set('display_errors',1);
        error_reporting(E_ALL);      
        if(!defined("WP_DEBUG_DISPLAY")){define("WP_DEBUG_DISPLAY",true);}
        if(!defined("WP_DEBUG_LOG")){define("WP_DEBUG_LOG",true);}
        //add_action( 'all', create_function( '', 'var_dump( current_filter() );' ) );
        //define( 'SAVEQUERIES', true );
        //define( 'SCRIPT_DEBUG', true );
        $wpdb->show_errors();
        $wpdb->print_error();
    }
}

/**
* Gets option value for wpecustomers _adminset or defaults to the file version of the array if option returns invalid.
* 1. Called in the main wpecustomers.php file.
* 2. Installs the admin settings option record if it is currently missing due to the settings being required by all screens, this is to begin applying and configuring settings straighta away for a per user experience 
*/
function wpecus_get_option_adminsettings(){
    $result = wpecus_option('wpecus_adminset','get');
    $result = maybe_unserialize($result); 
    if(is_array($result)){
        return $result; 
    }else{     
        return wpecus_INSTALL_admin_settings();
    }  
} 

/**
* Determine if post exists using any giving method
*/
function wpecus_does_post_exist($method,$value) {
    
    if($method == 'id'){
        return wpecus_does_post_exist_byid($value);
    }
    return false;
} 

/**
 * Standard HTTP forwarding
 * @param array $attributes_array
 * @property string status (standard http code i.e. 301 Moved Permanently)
 * @property url location (must begin with http)#
 * 
 * @todo add log recording and possibly statistical storing
 * @todo validate passed url before forwarding, provide option to not apply validation also
 * @todo ensure url validation includes forward slash at end
 * @todo validate status to expected values
 */
function wpecus_header_forward($atts){
    extract( shortcode_atts( array(
            'status' => '301 Moved Permanently',
            'location' => 'http://www.webtechglobal.co.uk',
    ), $atts ) );

    // clear cache then return result
    $wpdb->flush();
    
    // TODO: LOWPRIORITY, confirm this is the best way to do this in Wordpress (is there possibly a Wordpress method)
    header("HTTP/1.1 ".$status."");
    header("Status: ".$status."");
    header("Location: ".$location."");
    header("Connection: close");
    exit(0); // Optional, prevents any accidental output
}

/**
 * Starts A Timer - Used To Time Scripts
 * @return the current time in micro
 * 
 * @todo microtime function has caused problems with some users, research alternatives if any
 * @todo put error reporting in this to handle problems better
 */
function wpecus_microtimer_start(){
    list($utime, $time) = explode(" ", microtime());
    return ((float)$utime + (float)$time);
}

/**
* Checks if a domain exists by ensuring a site is online
* 
* @return boolean, false if domain does not exist or site is not online, true otherwise
* 
* @param string $domain
*/
function wpecus_domain_online($domain){
    // @todo look into these 2 lines, benefits they have when used in Wordpress if any
   //ini_set("default_socket_timeout","05");
   //set_time_limit(5);
   $f = @fopen($domain,"r");
   if($f == false){
       return false;
   }else{
       $r=fread($f,1000);
       fclose($f);
       if(strlen($r)>1) {
           return true;
       }
       else {
           return false;
       }      
   }  
}

/**
* Returns a value for Tab Number, if $_GET[WTG_##_ABB . 'tabnumber'] not set returns 0 
*/
function wpecus_get_tabnumber(){                      
    if(!isset($_GET['wpecustab'])){
        return 0;
    }else{
        return $_GET['wpecustab'];                   
    }                                                      
}  

/**
* Called when plugin is being activated in Wordpress.
* I am avoiding anything actually being installed during this process. * 
*/
function wpecus_register_activation_hook(){
    global $wpecus_isbeingactivated;
    $wpecus_isbeingactivated = true;// used to avoid loading files not required during activation (minimise errors during activation related to none activation related)
}

/**
* Update admin theme
* 
* @param mixed $theme_name
*/
function wpecus_update_theme($theme_name){
    update_option('wpecus_theme',$theme_name);  
}

/**
* Delets plugin main navigation 
*/
function wpecus_delete_tabmenu(){
    return delete_option('wpecus_tabmenu');
}

/**
 * Exit on error or use misconfiguration but offer help information, links etc
 * The information will be based on keywords related to the area of the plugin that the user is experiencing problems with
 * @param array $keywords (an array of keywords that will be used to determine extra help content)
 * @param url $url (link from a big button to the related page for the feature the exit is used in connection with)
 * 
 * @todo display notice box that will hold the help button
 * @todo design or have designed a suitable help button
 * @todo display further possible help options crawled on the plugins site
 * @todo display Google AdWords search lastly
 */
function wpecus_exit($url,$keywords){
    // call a function here that displays a notice box of helpful information including links and a humerous image
    // in relation to any sort of problem

    // we'll use the keywords to retrieve the information using RSS or API, not 100% sure at this time
    exit;
}

/**
* Determines if giving varaible is WP_Error or not
* Logs error if boolean true result determined
* 
* @uses class WP_Error($code, $message, $data)
* @uses is_wp_error(),$wpval->get_error_data(),$wpval->get_error_message() 
* @param mixed $v is the value being checked as possible Wordpress error
* @param boolean $returnv, default is false, true causes $v to be returned instead of boolean when value is NOT an error
* @returns boolean true if giving value is WP_Error and false if not
*/
function wpecus_is_WP_Error($wpval,$returnv = false){
    if(is_wp_error($wpval)){

        $atts = array();
        $atts['projectname'] = 'NA';// TODO:LOWPRIORITY, get current project name from global
        //$atts['date'] = wpecus_date();// wpecus_date()   
        $atts['logtype'] = 'error';
        $atts['dump'] = 'Data Value: ' . $wpval->get_error_data();
        $atts['comment'] = 'Created by wpecus_is_WP_Error, Wordpress message is: ' . $wpval->get_error_message();
        $atts['style'] = 'error';

        wpecus_log($atts);  
        
        $result = true;   
    }else{
        $result = false;
    }

    if($result == false && $returnv == true){
        // not an error and value to be returned
        return $wpval;
    } elseif($result == true){
        // is an error or request needs boolean returned
        return true;
    }elseif($result == false && $returnv == false){
        // not an error but request wants boolean returned, false to indicate not WP_Error
        return false;
    }
}

/**
* Uses error_log to record an error to servers main error log.
* Use when loggin to our own database table is not viable
*  
* @param string $m, the message to be recorded
*/
function wpecus_error_log($m){ 
   error_log($m);
}

/**
 * Checks if the cores minimum requirements are met and displays notices if not
 * Checks: Internet Connection (required for jQuery), PHP version, Soap Extension
 * 
 * @todo HIGH PRIORITY check the status of all external files mainly jquery then display warnings i.e.  http://hotlink.jquery.com/jqueryui/jquery-1.6.2.js?ver=3.2.1
 * @todo HIGH PRIORITY begin a system to deal with missing jquery if even possible so that the interface is not unusable
 */
function wpecus_check_requirements($display){
    // variable indicates message being displayed, we will only show 1 message at a time
    $requirement_missing = false;

    // php version
    if(defined(WTG_WPECUS_PHPVERSIONMINIMUM)){
        if(WTG_WPECUS_PHPVERSIONMINIMUM > phpversion()){
            $requirement_missing = true;
            if($display == true){
                global $wpecus_plugintitle;
                wpecus_notice('The plugin detected an older PHP version than the minimum requirement which 
                is ' . WTG_WPECUS_PHPVERSIONMINIMUM . '. You can requests an upgrade for free from your hosting, use .htaccess to switch
                between PHP versions per WP installation or sometimes hosting allows customers to switch using their control panel.',
                'warning','Large',$wpecus_plugintitle . ' Requires PHP ' . WTG_WPECUS_PHPVERSIONMINIMUM);                
            }
        }
    }
    
    return $requirement_missing;
}

/**
* Returns the admin theme 
*/
function wpecus_get_theme(){ 
    $theme = get_option('wpecus_theme');
    if(!$theme || $theme == NULL){return 'jquery';}
    return $theme;
}
?>