<?php 
/** 
 * WTG own Wordpress Plugin development function library for diagnostics|preventitive type functions in WP e-Customers plugin 
 * 
 * @link by WebTechGlobal.co.uk
 * 
 * @author Ryan Bayne | ryan@webtechglobal.co.uk
 *
 * @package WP e-Customers
 */
 
 
/**
* Used in admin page headers to constantly check the plugins status while administrator logged in 
*/
function wpecus_diagnostics_constant_adminside(){
    if(is_admin() && current_user_can( 'manage_options' )){
                                                          
        if(!wpecus_is_processing()){
                                
            ###########################################################################################
            #                              PEFORM EXTENSION DIAGNOSTICS                               #
            ###########################################################################################
            if(WTG_WPECUS_EXTENSIONS != 'disable' && file_exists(WP_CONTENT_DIR . '/wpecusextensions')){
                $current_extension_name = 'df1'; 
                if(wpecus_extension_activation_status($current_extension_name) == 3){
                    if(file_exists(WP_CONTENT_DIR . '/eciextensions/'.$current_extension_name.'/functions/diagnostics.php')){
                        
                        require_once(WP_CONTENT_DIR . '/eciextensions/'.$current_extension_name.'/functions/diagnostics.php');            
                    }
                }
            }
                                 
            ###########################################################################################
            #                                PEFORM CORE DIAGNOSTICS                                  #
            ###########################################################################################                    
            // core database tables
            global $wpecus_tables_array;# core tables
            wpecus_diagnostics_databasecomparison_alertonly($wpecus_tables_array);  
        }   
    }
}

/**
* Uses database table arrays to check if any installed tables have not been updated yet 
*/
function wpecus_diagnostics_databasecomparison_alertonly($expected_tables_array){
    if(is_array($expected_tables_array)){  
        
        // loop through plugins tables
        foreach($expected_tables_array['tables'] as $key => $table){
                       
            $table_installed = wpecus_WP_SQL_does_table_exist($table['name']);
            
            // is table required and exist?
            if($table['required'] == true && $table_installed == false){  
                
                wpecus_n('Database Table Missing','WP e-Customers diagnostic
                indicates a required database table is missing. This must be dealt with before the
                plugin is used. The table name is <strong>' . $table['name'] .'</strong> and you can install it manually
                on the plugins installation screens. WebTechGlobal is happy to help you.','error','Small',array());
                return false;   
                 
            }elseif($table_installed == true){
                       
                // whatever the requirements, the table is installed so we need to ensure all columns are as required            
                $installed_table_array = wpecus_WP_SQL_get_tablecolumns($table['name'],true,true);
                     
                foreach($table['columns'] as $something => $somethingelse){
                    $column_found = false;
     
                    foreach($installed_table_array as $installedCol){
                        if($installedCol == $something){
                            $column_found = true;
                        }
                    }
                    
                    if(!$column_found){
                        wpecus_n('Database Update Required','The database table named <strong>'.$table['name'].'</strong> needs to be updated. A column
                        named <strong>'.$something.'</strong> must be added to it. This is a requirement in the latest version of the plugin. You can
                        perform this update on the installation screens.','error','Small');
                        return false;
                    }
                }
                    
                ###############################################
                #               PRECISE COMPARISON            #
                ###############################################   
                $installed_table_array = wpecus_WP_SQL_get_tablecolumns($table['name'],true);
                         
                // loop through the current tables array of columns
                foreach($installed_table_array as $key => $column){
                    
                    $columnName = $column[0];

                    // Type
                    // remove "unsigned" first,we are not checking for that right now
                    $installed_column = str_replace(' unsigned','',$column[1]);# example: int(10) unsigned
                    if($installed_column != $table['columns'][$columnName]['type']){
                        wpecus_n('Database Table Update Required','The plugins database table named <strong>'.$table['name'].'</strong>
                        needs to be updated. The column type for column named <strong>' . $columnName .'</strong> does not
                        match the current version of files installed. Please use the installation tools provided
                        to update the table manually. Seek support from WebTechGlobal if unsure.','error','Small');
                        return false;      
                    }
                   
                    // Null (mysql returns NO or YES but the CREATE query requires NOT NULL or no value at all)
                    if($column[2] == 'NO' && $table['columns'][$columnName]['null'] != 'NOT NULL'
                    || $column[2] == 'YES' && $table['columns'][$columnName]['null'] != ''){ 
                        wpecus_n('Database Table Update Required','The plugins database table named <strong>'.$table['name'].'</strong>
                        needs to be updated. The "null" setting for column named <strong>' . $columnName .'</strong> does not
                        match the current version of files installed. Please use the installation tools provided
                        to update the table manually. Seek support from WebTechGlobal if unsure.','error','Small');
                        return false;  
                    }

                    // Key - $column[3] - this wont change after database design so no check required
     
                    // Default
                    if(!$column[4] && $table['columns'][$columnName]['default'] != 'NULL' && $table['columns'][$columnName]['default'] != '' && $column[3] != 'PRI'){ 
                        wpecus_n('Database Table Update Required','The plugins database table named <strong>'.$table['name'].'</strong>
                        needs to be updated. The "default" value setting for column named <strong>' . $columnName .'</strong> does not
                        match the current version of files installed. Please use the installation tools provided
                        to update the table manually. Seek support from WebTechGlobal if unsure.','error','Small'); 
                        return false; 
                    }
                    
                    // Extra - installed returns lowercase while CREATE table query has uppercase
                    $installed = strtolower($column[5]);
                    $expected = strtolower($table['columns'][$columnName]['extra']);
                    if($installed != $expected){ 
                        wpecus_n('Database Table Update Required','The plugins database table named <strong>'.$table['name'].'</strong>
                        needs to be updated. Configuration for the column named <strong>' . $columnName .'</strong> does not
                        match the current version of files installed. Please use the installation tools provided
                        to update the table manually. Seek support from WebTechGlobal if unsure.','error','Small');
                        return false;  
                    }             
                }
            }
        }
    }     
}
?>
