<?php
/** 
 * WTG own Wordpress Plugin development function library for installation of WP e-Customers plugin 
 * 
 * @link by WebTechGlobal.co.uk
 * 
 * @author Ryan Bayne | ryan@webtechglobal.co.uk
 *
 * @package WP e-Customers
 */
 
/**
 * Installs the main elements for all packages
 */
function wpecus_install_core(){
    
    // settings arrays are includes by 
    global $wpecus_extension_loaded,$wpecus_pub_set,$wpecus_mpt_arr,$wpecus_currentversion,$wpecus_is_free;
            
    $minor_fails = 0;// count minor number of failures, if 3 or more then we'll call it a failed install
    $overall_install_result = true;// used to indicate overall result

    #################################################
    #         INSTALL DATABASE TABLES FIRST         #
    #################################################    
    wpecus_INSTALL_table_log();
    wpecus_INSTALL_table_downloads();
        
    #################################################
    #       INSTALL SCHEDULE ARRAY NOTICE ARRAY     #
    #################################################
    if(!$wpecus_is_free){    
        require(WTG_WPECUS_PAID_PATH . 'arrays/wpecus_schedule_array.php');
        if(!wpecus_option('wpecus_schedule','add',serialize($wpecus_schedule_array)) ){
             
            // should never happen - _uninstall() used at the beginning of _install_core()
            wpecus_notice('Schedule settings are already installed, no changes were made to those settings.','warning','Tiny',false,'','echo');
            wpecus_n('Schedule Settings Already Installed','Schedule settings are already installed, no changes were made to those settings.','warning','Tiny');
            $overall_install_result = false;          
       
        }else{
            wpecus_notice('Installed the schedule settings','success','Tiny',false,'','echo');
            wpecus_n('Schedule Settings Already Installed','Schedule settings are already installed, no changes were made to those settings.','warning','Tiny');
        }
    }
    wpecus_n('Schedule Settings Already Installed','Schedule settings are already installed, no changes were made to those settings.','warning','Tiny');
    
    #################################################
    #                                               #
    #         INSTALL PERSISTENT NOTICE ARRAY       #
    #                                               #
    #################################################
    $wpecus_persistent_array = array();
    if( !wpecus_option('wpecus_notifications','add',serialize($wpecus_persistent_array)) ){
        // should never happen - _uninstall() used at the beginning of _install_core()
        wpecus_notice('Notification settings are already installed, no changes were made to those settings.','warning','Tiny',false,'','echo');
        $overall_install_result = false;          
    }else{
        wpecus_notice('Installed the notification settings','success','Tiny',false,'','echo');
    }    

    // theme - only change the theme value when it is not set  
    if(!wpecus_option('wpecus_theme','get')){
        wpecus_option('wpecus_theme','add','jquery');# jquery or wordpresscss
    }         
                             
    // extension switch option record
    if(!wpecus_option('wpecus_extensions','get')){
        wpecus_option('wpecus_extensions','add','disable');    
    }
         
    // installation state values
    update_option('wpecus_is_installed',true);
    update_option('wpecus_was_installed',true); 
    update_option('wpecus_installedversion',$wpecus_currentversion);# will only be updated when user prompted to upgrade rather than activation
    update_option('wpecus_installeddate',time());# update the installed date, this includes the installed date of new versions

    return $overall_install_result;
}

/**
* Creates wpecus_log
* @link http://www.wpecus.com/hacking/log-table
*/
function wpecus_INSTALL_table_log(){
    $table_name = $wpdb->prefix . 'wpecuslog';

    if(wpecus_does_table_exist($table_name)){
        wpecus_notice('Database table named '.$table_name.' already exists.','warning','Tiny','','','echo');    
    }else{ 
        global $wpdb;
        
        // table name 
        $create = 'CREATE TABLE `'.$table_name.'` (';

        // columns - please update http://www.wpecustomers.com/hacking/log-table   
        $create .= '`row_id` bigint(20) unsigned NOT NULL AUTO_INCREMENT,
        `outcome` tinyint(1) unsigned NOT NULL DEFAULT 1,
        `timestamp` timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP,
        `line` int(11) unsigned DEFAULT NULL,
        `file` varchar(250) DEFAULT NULL,
        `function` varchar(250) DEFAULT NULL,
        `sqlresult` blob,
        `sqlquery` varchar(45) DEFAULT NULL,
        `sqlerror` mediumtext,
        `wordpresserror` mediumtext,
        `screenshoturl` varchar(500) DEFAULT NULL,
        `userscomment` mediumtext,
        `page` varchar(45) DEFAULT NULL,
        `version` varchar(45) DEFAULT NULL,
        `panelid` varchar(45) DEFAULT NULL,
        `panelname` varchar(45) DEFAULT NULL,
        `tabscreenid` varchar(45) DEFAULT NULL,
        `tabscreenname` varchar(45) DEFAULT NULL,
        `dump` longblob,
        `ipaddress` varchar(45) DEFAULT NULL,
        `userid` int(11) unsigned DEFAULT NULL,
        `comment` mediumtext,
        `type` varchar(45) DEFAULT NULL,
        `category` varchar(45) DEFAULT NULL,
        `action` varchar(45) DEFAULT NULL,
        `priority` varchar(45) DEFAULT NULL,
        `trigger` varchar(45) DEFAULT NULL,        
        PRIMARY KEY (`rowid`),
        UNIQUE KEY `rowid` (`rowid`)';

        // table config  
        $create .= ') ENGINE=MyISAM AUTO_INCREMENT=0 DEFAULT CHARSET=utf8';
        
        $result = $wpdb->query( $create ); 
         
        if( $result ){
            wpecus_notice('Database table named '.$table_name.' has been created.','success','Tiny','','','echo');
            return true;
        }else{
            wpecus_notice('Database table named '.$table_name.' could not be created. This must be investigated before using the plugin','error','Tiny','','','echo');
            return false;
        }
    }  
}

/**
* Creates $wpdb->downloads table which stores users status for a download (download count, price paid, donations paid)
*
* Currently uses $result = $wpdb->query( $create ); rather than dbDelta()
*
* @package WP e-Customers
* @since 0.0.1  
*/
function wpecus_INSTALL_table_downloads(){
    global $wpdb;
    
    $table_name = $wpdb->prefix . 'wpecusdownloads';

    if(wpecus_does_table_exist($table_name)){
        wpecus_notice('Database table named '.$table_name.' already exists.','warning','Tiny','','','echo');    
    }else{ 
        global $wpdb;
        $create = 'CREATE TABLE `'.$table_name.'` (';
        $create .= '`row_id` bigint(20) unsigned NOT NULL AUTO_INCREMENT,
        `user_id` bigint(20) unsigned NOT NULL default "0",
        `download_id` bigint(20) unsigned NOT NULL default "0",
        `stakeholder` tinyint(1) NOT NULL default "0",
        `paidpermission` tinyint(1) NOT NULL default "0",
        `freepermission` tinyint(1) NOT NULL default "0",
        `subscribed` tinyint(1) NOT NULL default "0",     
        `downloaded` bigint(20) unsigned NOT NULL default "0",
        `timestamp` timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP,
        `eventdate` datetime NOT NULL default "0000-00-00 00:00:00",
        PRIMARY KEY (`row_id`),
        INDEX combo(`user_id`,`download_id`)';

        // table config  
        $create .= ') ENGINE=MyISAM AUTO_INCREMENT=0 DEFAULT CHARSET=utf8';
        
        $result = $wpdb->query( $create ); 
         
        if( $result ){
            wpecus_notice('Database table named '.$table_name.' has been created.','success','Tiny','','','echo');
            return true;
        }else{
            wpecus_notice('Database table named '.$table_name.' could not be created. This must be investigated before using the plugin','error','Tiny','','','echo');
            return false;
        }
    }  
}

/**
* Quick way to delete then install all core database tables                             
*/
function wpecus_INSTALL_reinstall_databasetables(){
    global $wpecus_tables_array,$wpdb;
    $result_array = wpecus_ARRAY_init('result array for database table re-installation',__LINE__,__FUNCTION__,__FILE__,'result array',true);
    $result_array['droppedtables'] = array();
    
    if(is_array($wpecus_tables_array)){
        
        // delete all tables
        foreach($wpecus_tables_array['tables'] as $key => $table){
           
            if(wpecus_does_table_exist($table['name'])){         
                $wpdb->query( 'DROP TABLE '. $table['name'] );
                $result_array['droppedtables'][] = $table['name'];
            }                                                             
        }
        
        // now install tables
        wpecus_INSTALL_table_log();
        wpecus_INSTALL_table_downloads();
        
    }else{
        $result_array['outcome'] = false;
        $result_array['failreason'] = 'tables array is not an array';
    }   
    
    return $result_array;
}

/**
* DO NOT CALL DURING FULL PLUGIN INSTALL
* This function uses update. Do not call it during full install because user may be re-installing but
* wishing to keep some existing option records.
* 
* Use this function when installing admin settings during use of the plugin. 
*/
function wpecus_INSTALL_admin_settings(){
    require_once(WTG_WPECUS_PATH . 'arrays/wpecus_variables_adminset_array.php');
    return wpecus_option('wpecus_adminset','update',$wpecus_adm_set);# update creates record if it does not exist   
}

/**
* DO NOT CALL DURING FULL PLUGIN INSTALL
* This function uses update. Users may want their installation to retain old values, we cannot assume the
* installation is 100% fresh.
* 
* Use this function when the tab menu option array is missing or invalid or when user actions a re-install of everything 
*/
function wpecus_INSTALL_tabmenu_settings(){
    require_once(WTG_WPECUS_PATH . 'views/wpecus_menu_array.php');    
    $result = wpecus_option('wpecus_tabmenu','update',$wpecus_mpt_arr);   
} 

/**
* Installs the Quick Start by using update.
* This function should not be used during plugin installation because it
* would destroy values that the user may be trying to retain for a new
* installation. 
*/
function wpecus_INSTALL_ecisession(){
    $wpecus_qs_array = array();
    $wpecus_qs_array['arrayupdated'] = time();
    $wpecus_qs_array['nextstep'] = 1;  
    wpecus_option('wpecus_ecisession','update',serialize($wpecus_qs_array));
}

/**
* Checks if plugin IS FULLY INSTALLED (all required options arrays,folders,path files etc)
* Another function and variable establishes if the plugin WAS installed in the past by finding a trace
* 
* @see wpecus_variables_admin.php
* @todo MEDIUMPRIORITY, return a result array so we can make use of negative installation state reason
*/
function wpecus_is_installed(){
    
    global $wpecus_options_array;
       
    if(!isset($wpecus_options_array) || !is_array($wpecus_options_array)){
        return false;
    }
             
    // currently this value is returned, if changed to false
    $returnresult = true;
    $failcause = 'Not Known';// only used for debugging to determine what causes indication of not fully installed
    
    // function only returns boolean but if required we will add results array to the log
    $is_installed_result = array();
    $is_installed_result['finalresult'] = false;
    $is_installed_result['options'] = null;
                
    foreach($wpecus_options_array as $id => $optionrecord){
            
        if($optionrecord['required'] == true){
                    
            $currentresult = get_option($id);    
            
            $is_installed_result['options'][$id]['result'] = $currentresult;
                        
            // change return switch to false if option not found
            if($currentresult == false || $currentresult == null){ 
              
                $returnresult = false;
                $failcause = 'Option RecordMissing:'.$id;    
            }
        } 
    }                       
      
    return $returnresult;
}   

/**
* NOT YET IN USE - WILL CHECK IF PLUGIN HAS BEEN ACTIVATED WITH SECURITY VIA SERVER
* Determines if the plugin is activated and validates credentials to ensure ongoing use.
* Once done, less SOAP calls will be required and the aim is to reduce traffic.
*/
function wpecus_is_activated(){ 
   global $wpecus_activationcode;
   return false;
}

/**
 * Removes the plugins main wp-content folder, used in wpecus_install_core() for failed install
 * @todo function to be complete
 */
function wpecus_remove_contentfolder(){

}

/**
 * Checks if plugin has been installed before (not security and not indication of full existing installation)
 */
function wpecus_was_installed(){
    global $wpecus_options_array;
    
    $result = false;
    
    if(isset($wpecus_options_array) && is_array($wpecus_options_array)){
        // check possible option records
        foreach($wpecus_options_array as $id => $optionrecord){
     
            // avoid checking tab menu as that is added to blow on Wordpress plugin activation, we do not want to use it as an indication of install
            if($id != 'wpecus_tabmenu' 
            && $id != 'wpecus_installedversion' 
            && $id != 'wpecus_installeddate' ){
                                        
                $currentresult = get_option($id);    

                // change return switch to false if option not found
                if(isset($currentresult) && $currentresult != null){

                    // we return on first detection of previous installation to avoid going through entire loop
                    return true;    
                }
            }
        } 
        
        return $result; 
    }
     
    return $result;
}  

/**
* Deletes the plugins main content folder
* 
* @param mixed $pathdir (the path to be deleted)
* @param mixed $output (boolean true means that the file was found and deleted)
*/
function wpecus_delete_contentfolder($pathdir,$output = false){
    if(!is_dir($pathdir)){
        global $wpecus_plugintitle;
        wpecus_notice($wpecus_plugintitle . ' could not find the main content folder, it
        may have already been deleted or moved.', 'warning', 'Tiny','Content Folder Not Found');
        return false;
    }else{
    
        if (wpecus_dir_is_empty($pathdir)) {
            rmdir($pathdir);
            wpecus_notice('Content folder has been deleted after confirming it did not contain any files.', 'success', 'Tiny','Content Folder Removed');                
            return true; 
        }else{
            wpecus_notice('Content folder cannot be deleted as it contains files.', 'warning', 'Tiny','Content Folder Not Removed');                      
        }
    }
}

/**
 * Install main content folder in wp-content directory for holding uploaded items
 * Called from install function in install.php if constant is not equal to false "WTG_WPECUS_CONTENTFOLDER_DIR
 *
 * @return boolean true if folder installed or already exists false if failed
 */
function wpecus_install_contentfolder_wpecustomerscontent($pathdir,$output = false){
    $contentfolder_outcome = true;

    // does folder already exist
    if(!is_dir($pathdir)){
        $contentfolder_outcome = wpecus_createfolder($pathdir);
    }else{
        return true;
    }

    if(!$contentfolder_outcome){
        $contentfolder_outcome = false;
        if($output){
            wpecus_notice('Plugins content folder could be not created:'.$pathdir, 'error', 'Tiny');
        }
    }elseif($contentfolder_outcome){
         if($output){
            wpecus_notice('Plugin content folder has been created here: '.$pathdir, 'success', 'Tiny');
         }
    }

    return $contentfolder_outcome;
}
?>
