<?php
/** 
 * WTG own Wordpress Plugin development function library for interface features in WP e-Customers plugin 
 * 
 * @link by WebTechGlobal.co.uk
 * 
 * @author Ryan Bayne | ryan@webtechglobal.co.uk
 *
 * @package WP e-Customers
 */
 
/**
 * Add hidden form fields, to help with processing and debugging
 * Adds the _form_processing_required value, required to call the form validation file
 *
 * @param integer $pageid (the id used in page menu array)
 * @param slug $panel_name (panel name form is in)
 * @param string $panel_title (panel title form is in)
 * @param integer $panel_number (the panel number form is in),(tab number passed instead when this function called for support button row)
 * @param integer $step (1 = confirm form, 2 = process request, 3+ alternative processing)
 */
function wpecus_hidden_form_values($tabnumber,$page_name,$panel_name,$panel_title){

    wp_nonce_field($panel_name); 

    echo '<input type="hidden" name="wpecus_hidden_tabnumber" value="'.$tabnumber.'">';
    echo '<input type="hidden" name="wpecus_hidden_pagename" value="'.$page_name.'">';
    echo '<input type="hidden" name="wpecus_hidden_panel_name" value="'.$panel_name.'">';
    echo '<input type="hidden" name="wpecus_hidden_panel_title" value="'.$panel_title.'">';
}

/**
* Builds text link, also validates it to ensure it still exists else reports it as broken
* 
* The idea of this function is to ensure links used throughout the plugins interface
* are not broken. Over time links may no longer point to a page that exists, we want to 
* know about this quickly then replace the url.
* 
* @return $link, return or echo using $response parameter
* 
* @param mixed $text
* @param mixed $url
* @param mixed $htmlentities, optional (string of url passed variables)
* @param string $target, _blank _self etc
* @param string $class, css class name (common: button)
* @param strong $response [echo][return]
* 
* @todo prepare function to accept the htmlentities string
* @todo check url type, ensure http etc in place
* @todo capture statistcs on links being clicked to establish how often they are actually used
* @todo can we do a check to ensure the html entities as a valid join to the domain? even just ensuring ? exists in url or apply it using $middle
*/
function wpecus_link($text,$url,$htmlentities = '',$target = '_blank',$class = '',$response = 'echo',$title = ''){
    
    // add ? to $middle if there is no proper join after the domain
    $middle = '';
                             
    // decide class
    if($class != ''){$class = 'class="'.$class.'"';}
    
    // build final url
    $finalurl = $url.$middle.htmlentities($htmlentities);
    
    // check the final result is valid else use a default fault page
    $valid_result = wpecus_validate_url($finalurl);
    
    if($valid_result){

        $link = '<a href="'.$finalurl.'" '.$class.' target="'.$target.'" title="'.$title.'">'.$text.'</a>';

    }else{
        ### @todo log this event
        ### @todo send variables for WTG to log and raise the issue in the admin
        $link = '<a href="http://www.webtechglobal.co.uk/blog/help/invalid-application-link" target="_blank">Invalid Link Please Click Here To Report This</a>';        
    }
    
    if($response == 'echo'){
        echo $link;
    }else{
        return $link;
    }     
} 

/**
* Builds link to google search or WTG AdSense search - Used in help content
* 
* @param mixed $text
* @param mixed $subscription
* @param mixed $string
* 
* @todo make use of adsense for none gold users
*/
function wpecus_google_searchlink($text,$package,$string){
                                            
    if($package == 'paid'){
        // standard google
        $url = 'http://www.google.co.uk/search?q='.$text;    
    }else{
        // adsense google
        ### @todo LOWPRIORITY, consider a method that puts the search through our own search page instead
        $url = 'http://www.google.co.uk/search?q='.$text;   
    }
    
    return wpecus_link($text,$url,'_blank',$string);   
}

/**
* Returns array with common values required for forms that need jQuery dialog etc.
* The default values can be overridden by populating the $jsform_set_override array. 
* 
* @param mixed $pageid
* @param mixed $wpecus_tab_number
* @param mixed $panel_number
* @param mixed $panel_name
* @param mixed $panel_title
* @param array $jsform_set_override, (not yet in use) use to customise the return value, not required in most uses
*/
function wpecus_jqueryform_commonarrayvalues($pageid,$wpecus_tab_number,$panel_number,$panel_name,$panel_title,$jsform_set_override = ''){
    ### TODO:MEDIUMPRIORITY  extract the override values after callign global values
    // $jsform_set_override
    // this is so we can pass the override array for custom settings rather than the default
    $jsform_set = array();
    // http://www.webtechglobal.co.uk/blog/wordpress/wtg-plugin-template/wtg-pt-jquery-dialogue-form 
    $jsform_set['pageid'] = $pageid;
    $jsform_set['tab_number'] = $wpecus_tab_number; 
    $jsform_set['panel_number'] = $panel_number;
    $jsform_set['panel_name'] = $panel_name;
    $jsform_set['panel_title'] = $panel_title;                
    // dialog box, javascript
    $jsform_set['dialogbox_id'] = $panel_name.$panel_number;
    $jsform_set['dialogbox_height'] = 300;// false will remove the height entry from the script
    $jsform_set['dialogbox_width'] = 800;// false will remove the width entry from the script
    $jsform_set['dialogbox_autoresize'] = false;// true or false, overrides width and height 
    // form related
    $jsform_set['form_id'] = wpecus_create_formid($panel_name);
    $jsform_set['form_name'] = wpecus_create_formname($panel_name);                                   
    return $jsform_set;
}

/**
* Adds the opening divs for panels and help button
* 
* @param array $form_array
*/
function wpecus_panel_header( $form_array, $boxintro_div = true ){
    global $wpecus_adm_set,$wpecus_guitheme;
                         
    // establish global panel state for all panels in plugin, done prior 
    $panel_state = ''; 
    if($panel_state){
        $panel_state = 'closed';    
    }    
          
    // override panel state if $form_array includes specific state
    if(isset($form_array['panel_state']) && ($form_array['panel_state'] == 1 || $form_array['panel_state'] == 'open')){
        $panel_state = 'open';    
    }elseif(isset($form_array['panel_state']) && ($form_array['panel_state'] == 0 || $form_array['panel_state'] == 'closed')){
        $panel_state = 'closed';
    }?>

    <div id="titles" class="postbox <?php echo $panel_state;?>">
        <div class="handlediv" title="Click to toggle"><br /></div>
        <h3 class="hndle"><span><?php echo $form_array['panel_title'];?></span></h3>
        <div class="inside">

            <?php // row of panel support buttons
            if($wpecus_guitheme == 'jquery'){
                if(!isset($wpecus_adm_set['interface']['panels']['supportbuttons']['status']) || $wpecus_adm_set['interface']['panels']['supportbuttons']['status'] == 'display'){?>

                <?php
                // two column accordion 
                if($boxintro_div && $wpecus_guitheme == 'jquery'){      
                    echo '<div class="wpecus_panelcolumn_Top_A_Right">';
                }
                ?>
                       
                <?php wpecus_panel_support_buttons($form_array);?>

                <?php 
                // two column accordion
                if($wpecus_guitheme == 'jquery'){        
                    echo '</div>';
                }?>
                    
                <?php 
                }
            }?>
            
            <?php
            // display persistent notices for the current panel
            wpecus_persistentnotice_output('panel',$form_array['panel_id']);
}

/**
* Adds closing divs for panels 
*/
function wpecus_panel_footer(){
    echo '</div></div>';
}

/**
* Includes screen as requested through CSS menu
* 1. Calls all globals for screen file to access, saves us putting them on all screens
* 
* @param mixed $pageid
* @param mixed $panel_number
* @param mixed $wpecus_tab_number
*/     
function wpecus_GUI_wordpresscss_screen_include($pageid,$panel_number,$wpecus_tab_number){
    global $wpecus_textspin_array,$wpecus_jobtable_array,$wpecus_job_array,$wpecus_dataimportjobs_array,$wpecus_project_array,$wpecus_currentproject_code,$wpecus_is_dev,$wpecus_guitheme,$wpecus_extension_loaded,$wpecus_adm_set,$wpecus_is_installed,$wpecus_currentversion,$wpecus_file_profiles,$wpecus_mpt_arr,$wpdb,$wtgtp_pluginforum,$wtgtp_pluginblog,$wpecus_options_array,$wpecus_is_free,$wpecus_projectslist_array,$wpecus_schedule_array;
    $wpecus_form_action = '';         
    include($wpecus_mpt_arr['menu'][$pageid]['tabs'][$wpecus_tab_number]['path']);        
}

/**
* Lists all DISTINCT custom field keys (it actually queries meta-keys).
* There is not a currently any measures that hide none custom field keys.  
*/
function wpecus_list_customfields(){
    $result = wpecus_get_customfield_keys_distinct();
    foreach ($result as $customfield) {
        echo $customfield->meta_key .'<br />';
    }    
}

/**
* Uses wpecus_get_customfield_keys_distinct() to get DISTINCT meta-keys then loops and prints them.
* Can be used for custom field related features however there is wpecus_list_customfields() for that also. 
*/
function wpecus_list_metakeys(){
    $result = wpecus_get_customfield_keys_distinct();
    foreach ($result as $customfield) {
        echo $customfield->meta_key .'<br />';
    }   
}

/**
* Used to build post type form. Echos option items, use function inside select object.
*/
function wpecus_display_posttypes_menu_options( $current = false ){

    $post_types = get_post_types('','names');
    
    foreach ( $post_types as $post_type ) {
        
        // do not display post types that wont be used
        if( $post_type != "revision" && $post_type != "nav_menu_item" ){
            
            $selected = '';
            
            if($current == $post_type){
                $selected = 'selected="selected"';            
            }
            
            echo '<option value="'.$post_type.'" '.$selected.'>'.$post_type.'</option>';
        }
    }
}

/**
* Echos options for a form menu.
* 
* @param string $current_value, pass the current saved value or the form setting or pass null
* @todo LOWPRIORITY, change this function to display heirarchy properly 
*/
function wpecus_display_categories_options($current_value = 'NOTSELECTED2333617351'){

    $cats = get_categories('hide_empty=0&echo=0&show_option_none=&style=none&title_li=');

    foreach( $cats as $c ){ 
        
        // apply selected value to current save
        $selected = '';
        if( $current_value == $c->term_id ) {
            $selected = 'selected="selected"';
        }
        
        echo '<option value="'.$c->term_id.'" '.$selected.'>'. $c->term_id . ' - ' . $c->name .'</option>'; 
    }            
}

function wpecus_display_users_options($current_value = 'NOTSELECTED09731717236'){

    $blogusers = get_users('blog_id=1&orderby=nicename');

    foreach ($blogusers as $user){ 
        
        // apply selected value to current save
        $selected = '';
        if( $current_value == $user->ID ) {
            $selected = 'selected="selected"';
        }
        
        echo '<option value="'.$user->ID.'" '.$selected.'>'. $user->ID . ' - ' . $user->display_name .'</option>'; 
    }            
    
}

/**
* Outputs a form menu of the giving database tables columns for single selection
* 
* @param string $table_name
* @param mixed $id used to make ID attribute unique, recommend that it not be a number only if the menu is to be used many times
*/
function wpecus_menu_tablecolumns($table_name,$id = ''){?>  
    <select name="wpecus_table_columns_<?php echo $table_name;?><?php echo $id;?>" id="wpecus_table_columns_<?php echo $table_name;?><?php echo $id;?>_id" >
        <?php wpecus_options_tablecolumns($table_name);?>                                                                                                                     
    </select><?php    
}

/**
* Creates url to an admin page
*  
* @param mixed $page, registered page slug i.e. wpecus_install which results in wp-admin/admin.php?page=wpecus_install   
* @param mixed $values, pass a string beginning with & followed by url values
*/
function wpecus_link_toadmin($page,$values = ''){
    return get_admin_url() . 'admin.php?page=' . $page . $values;
}              

/**
 * Adds <button> with jquerybutton class and </form>, for using after a function that outputs a form
 * Add all parameteres or add none for defaults
 * @param string $buttontitle
 * @param string $buttonid
 */
function wpecus_formend_standard($buttontitle = 'Submit',$buttonid = 'notrequired'){
    global $wpecus_guitheme;
        
        if($buttonid == 'notrequired'){
            $buttonid = 'wpecus_notrequired'.rand(1000,1000000);# added during debug
        }else{
            $buttonid = $buttonid.'_formbutton';
        }?>

        <p class="submit">
            <input type="submit" name="wpecus_wpsubmit" id="<?php echo $buttonid;?>" class="button button-primary" value="<?php echo $buttontitle;?>">
        </p>
        
        <?php /*  change to Wordpress styling however there may be or were scripts that require <button i.e. dialog content that updates
        <div class="jquerybutton"> 
            <button id="<?php echo $buttonid;?>"><?php echo $buttontitle;?></button>
        </div>  */ 
        ?>

    </form><?php
} 

/**
 * Echos the html beginning of a form and beginning of widefat post fixed table
 * 
 * @param string $name (a unique value to identify the form)
 * @param string $method (optional, default is post, post or get)
 * @param string $action (optional, default is null for self submission - can give url)
 * @param string $enctype (pass enctype="multipart/form-data" to create a file upload form)
 */
function wpecus_formstart_standard($name,$id = 'none', $method = 'post',$class,$action = '',$enctype = ''){
    if($class){
        $class = 'class="'.$class.'"';
    }else{
        $class = '';         
    }
    echo '<form '.$class.' '.$enctype.' id="'.$id.'" method="'.$method.'" name="'.$name.'" action="'.$action.'">
    <input type="hidden" id="wpecus_post_processing_required" name="wpecus_post_processing_required" value="true">';
}

/**
 * Calls wtgtp_jquery_opendialog (with close button)
 * 
 * Intended use is to display information, tutorial video etc
 *
 * @param array
 * 
 * @deprecated 16th February 2013 use wpecus_display_accordianpanel_buttons()
 */
function wpecus_panel_support_buttons($form_array){
    wpecus_display_accordianpanel_buttons($form_array);  
}

/**
* Displays the status of the content folder with buttons to delete or create the folder
* 
* @param mixed $logtype
*/
function wpecus_contentfolder_display_status(){
    
    $contentfolder_exists = wpecus_contentfolder_exist();
    
    if($contentfolder_exists){

        echo wpecus_notice('Content folder exists'.
        wpecus_formstart_standard('wpecus_deletecontentfolder_form','none','post','').'
            <button class="button" name="wpecus_contentfolder_delete">Delete</button>                        
        </form>', 'success', 'Small', false,'','return');

    }elseif(!$contentfolder_exists){
        echo wpecus_notice('Content folder does not exist please create it'.
        wpecus_formstart_standard('wpecus_createcontentfolder_form','none','post','').'
            <button class="button" name="wpecus_contentfolder_create">Create</button>        
        </form>', 'error', 'Small', false,'','return');
    }
}

/**
 * Adds Script Start and Stylesheets to the beginning of pages
 */
function wpecus_header_page($pagetitle,$layout){
    global $current_user,$wpecus_mpt_arr,$wpecus_adm_set,$wpecus_pub_set,$wpecus_currentproject_code,$wpecus_is_free;

    $wpecus_adm_set = wpecus_get_option_adminsettings(); 

    get_currentuserinfo();
    
    wpecus_users_get_current_user();?>
                
    <div class="wrap">
        <?php        
        // run diagnostics
        wpecus_diagnostics_constant_adminside();
             
        // display persistent notices for all pages
        wpecus_persistentnotice_output('global');
        ?>
    
        <div id="icon-options-general" class="icon32"><br /></div>
        <h2>Beta <?php echo $pagetitle;?></h2>
        <p>This is a beta plugin. Updates coming soon will deliver many features for logged in subscribers and for visitors not logged in. We are not
        recommending the plugin for live websites for that reason. Please install WP e-Commerce on a test blog and monitor our development. If you feel
        all is working perfectly we would appreciate it if you let us know.</p>
        
        <?php 
        // run specific admin triggered automation tasks, this way an output can be created for admin to see
        wpecus_admin_triggered_automation();  

        // check existing plugins and give advice or warnings
        wpecus_plugin_conflict_prevention();
                 
        // display form submission result notices
        wpecus_notice_output();
        
        // on first visit or after suspension users must request authorization
        wpecus_users_request_authorization();        
        
        // display persistent notices for current page
        wpecus_persistentnotice_output('page',$_GET['page']);
      
        // process global security and any other types of checks here such such check systems requirements, also checks installation status
        $wpecus_requirements_missing = wpecus_check_requirements(true);?>

        <div class="postbox-container" style="width:99%">
            <div class="metabox-holder">
                <div class="meta-box-sortables"><?php
}

function wpecus_GUI_br(){
    echo '<br />';
}

function wpecus_GUI_nbsp(){
    echo '&nbsp;';
}

/**
 * Displays author and adds some required scripts
 */
function wpecus_footer(){?>

                </div><!-- end of post boxes -->
            </div><!-- end of post boxes -->
        </div><!-- end of post boxes -->
    </div><!-- end of wrap - started in header -->

    <script type="text/javascript">
        // <![CDATA[
        jQuery('.postbox div.handlediv').click( function() { jQuery(jQuery(this).parent().get(0)).toggleClass('closed'); } );
        jQuery('.postbox h3').click( function() { jQuery(jQuery(this).parent().get(0)).toggleClass('closed'); } );
        jQuery('.postbox.close-me').each(function(){
        jQuery(this).addClass("closed");
        });
        //-->
     </script><?php
}

/**
* Gets tab menu array
* 
* @todo LOWPRIORITY, rename function, it gets array from file not just option record. Possible wpecus_WP_SETTINGS_tabmenu()
*   
*/
function wpecus_get_option_tabmenu(){
    global $wpecus_adm_set,$wpecus_menu_source;# this is coming from the loaded array file
    // if load method not set and global is an array return the global
    if(isset($wpecus_adm_set['tabmenu']['loadmethod']) && $wpecus_adm_set['tabmenu']['loadmethod'] == 'file'
    || $wpecus_menu_source == 'file'){     
        require_once(WTG_WPECUS_PATH . 'views/wpecus_menu_array.php');
        return $wpecus_mpt_arr;
                      
    }else{

        // load from option array by default ($wpecus_menu_source over-rides this to load from file)
        // but only return value if its a valid array else we install the admin settings array now        
        $result = wpecus_option('wpecus_tabmenu','get');
        if(is_array($result) && isset($result['menu'])){# if the new ['menu'] is not in array we re-install
            return $result;
        }else{
            // users wants menu to load from stored option value but it returned an invald value
            return wpecus_INSTALL_tabmenu_settings();# returns the tabmenu array
        }
    }        
} 

/**
 * Display a notice box per option record. Where possible defaults are used. So a failure
 * in these checks does not mean a failure in the plugin, it could indicate that the user wants to
 * use the file version of an array and not store it locally.
 * 
 * @todo HIGHPRIORITY, generate this list based on options array. Make outcome more complex. Indicate if option record or not.
 */
function wpecus_install_optionstatus_list(){

    $admin_settings_array = get_option('wpecus_adminset');
    if(is_array($admin_settings_array)){
        echo wpecus_notice('wpecus_adminset is installed', 'success', 'Small', false,'','return');
    }else{
        echo wpecus_notice('wpecus_adminset not installed properly <a class="button" href="'.wpecus_currenturl().'&test=test">Install Now</a>', 'error', 'Small', false,'','return');
    }

    $public_settings_array = get_option('wpecus_publicset');
    if(is_array($public_settings_array)){
        echo wpecus_notice('wpecus_publicset is installed', 'success', 'Small', false,'','return');
    }else{
        echo wpecus_notice('wpecus_publicset not installed properly <a class="button" href="'.wpecus_currenturl().'&test=test">Install Now</a>', 'error', 'Small', false,'','return');
    } 
}

/**
 * Echos file size formatted to suit size and ensures file exists before using formatting function
 * @uses eci_csvfileexists
 * @param path $file_path
 *
 * @todo add function to check file_path exists
 */
function wpecus_displayfilesize($file_path){
   //$exists = eci_csvfileexists(basename($filepath), $pro);
   $exists = true;
   if($exists == true){
      echo wpecus_format_file_size(filesize($file_path));
   }else{
      echo 0;
   }
}

/**
* Returns standard formatted form name
* 
* @param string $panel_name
* @param string $specificpurpose, used to append a value, important when multiple forms used in a single panel
*/
function wpecus_create_formname($panel_name,$specificpurpose = ''){
    return 'wpecus_form_name_' . $panel_name . $specificpurpose;    
} 

/**
* Returns a standard formatted form ID
* 
* @param string $panel_name
* @param string $specificpurpose, used to append a value, important when multiple forms used in a single panel 
*/
function wpecus_create_formid($panel_name,$specificpurpose = ''){
    return 'wpecus_form_id_' . $panel_name . $specificpurpose;
}

/**
* Displays various notice boxes to help users get a quick idea of the current schedule and auto events setup 
*/
function wpecus_schedulescreen_notices(){
    global $wpecus_schedule_array,$wpecus_projectslist_array;

    // if not allowed today display
    $day = strtolower( date('l') );
    if(!isset($wpecus_schedule_array['days'][$day])){
        echo wpecus_notice('Scheduled events have not been permitted for ' . date('l'),'info','Tiny','','','return');     
    }
    
    // if not allowed this hour display
    $hour = strtolower(date('G'));
    if(!isset($wpecus_schedule_array['hours'][$hour])){
        echo wpecus_notice('Scheduled events have not been permitted for the current hour: ' . date('G'),'info','Tiny','','','return');    
    }
    
    // if no hours array set OR if no boolean true exists for any hours
    if( !isset($wpecus_schedule_array['hours']) ){
        echo wpecus_notice('Schedule is not ready as no hours have been permitted','info','Tiny','','','return');    
    }else{
        // if no hours are boolean true
        $hour_permitted = false;
        foreach( $wpecus_schedule_array['hours'] as $key => $hour ){
            if($hour == true){
                $hour_permitted = true;
                break;    
            }    
        }    
        
        if(!$hour_permitted){
            echo wpecus_notice('Schedule paused because no hours are permitted','info','Tiny','','','return');    
        }
    }
    
    // if no days array set OR if no boolean true exists for any days
    if( !isset($wpecus_schedule_array['days']) ){
        echo wpecus_notice('Schedule is not ready as no days have been permitted','info','Tiny','','','return');    
    }else{
        // if no hours are boolean true
        $days_permitted = false;
        foreach( $wpecus_schedule_array['days'] as $key => $day ){
            if($day == true){
                $days_permitted = true;
                break;    
            }    
        }    
        
        if(!$days_permitted){
            echo wpecus_notice('Schedule has been disabled because no days are permitted','info','Tiny','','','return');    
        }
    }
    
    // if no event types set display this
    if(!isset($wpecus_schedule_array['eventtypes'])){
        echo wpecus_notice('Schedule is not setup, no event types have been activated yet','info','Tiny','','','return');    
    }else{
        // have event types been disabled
        $event_type_set = false;
        foreach($wpecus_schedule_array['eventtypes'] as $event_type){
            if($event_type['switch'] == true){
                $event_type_set = true;
                break;    
            }    
        }
        
        if(!$event_type_set){
            echo wpecus_notice('Schedule has been stopped because all event types are disabled','info','Tiny','','','return');    
        }
    }

    // if current 24 hour period limit reached display this
    if(isset($wpecus_schedule_array['history']['daycounter']) && isset($wpecus_schedule_array['limits']['day']) && $wpecus_schedule_array['history']['daycounter'] >= $wpecus_schedule_array['limits']['day']){
        echo wpecus_notice('The maximum events number for the current 24 hour period has been reached','info','Tiny','','','return');        
    }
    
    // if current 60 minute period limit reached display this
    if(isset($wpecus_schedule_array['history']['hourcounter']) && isset($wpecus_schedule_array['limits']['hour']) && $wpecus_schedule_array['history']['hourcounter'] >= $wpecus_schedule_array['limits']['hour']){
        echo wpecus_notice('The maximum events number for the current 60 minute period has been reached','info','Tiny','','','return');        
    }

    // if no projects are on drip feeding display
    $project_dripfeeding = false;
    if(isset($wpecus_projectslist_array) && is_array($wpecus_projectslist_array)){
        foreach($wpecus_projectslist_array as $project_code => $project_array){
            if($project_code != 'arrayinfo'){
                if(isset($project_array['dripfeeding']) && $project_array['dripfeeding'] == 'on'){
                    $project_dripfeeding = true;
                    break;
                }   
            }     
        }
    }
    
    if(!$project_dripfeeding){
        echo wpecus_notice('You do not have any Post Creation Projects activated for drip-feeding through the plugins schedule','info','Tiny','','','return');    
    } 
} 

/**
* <table class="widefat">
* Allows control over all table
* 
*/
function wpecus_GUI_tablestart($class = false){
    if(!$class){$class = 'widefat';}
    echo '<table class="'.$class.'">';    
}


/**
 * Adds a jquery effect submit button, for using in form
 * 
 * @param string $panel_name (original use for in panels,panel name acts as an identifier)
 * @uses csvip_helpbutton function uses jquery script required by this button to have any jquery effect
 */
function wpecus_formsubmitbutton_jquery($form_name){?>
    <p class="submit">
        <input type="submit" name="<?php echo WTG_WPECUS_ABB;?><?php echo $form_name;?>_submit" value="Submit" />
    </p><?php
} 

/**
* List of notification boxes displaying folders created by WP e-Customers.
*/
function wpecus_list_folders(){
    global $wpecus_dataimportjobs_array,$wpecus_jobtable_array;

    $contentfolder_exists = wpecus_contentfolder_exist();
    
    if($contentfolder_exists){?>
        <script language="JavaScript">
        function wpecus_deletefolders_checkboxtoggle(source) {
          checkboxes = document.getElementsByName('wpecus_deletefolders_array[]');
          for(var i in checkboxes)
            checkboxes[i].checked = source.checked;
        }
        </script>
        <input type="checkbox" onClick="wpecus_deletefolders_checkboxtoggle(this)" /> Select All Folders<br/>
        <?php echo wpecus_notice('<input type="checkbox" name="wpecus_deletefolders_array[]" value="wpecustomerscontent" /> 1. wpecustomerscontent','success','Tiny','','','return');                    
    }
}

/**
* List of notification boxes displaying core plugin tables
*/
function wpecus_list_plugintables(){
    global $wpecus_tables_array;?>

    <script language="JavaScript">
    function wpecus_deletecoretables_checkboxtoggle(source) {
      checkboxes = document.getElementsByName('wpecus_deletecoretables_array[]');
      for(var i in checkboxes)
        checkboxes[i].checked = source.checked;
    }
    </script>

    <input type="checkbox" onClick="wpecus_deletecoretables_checkboxtoggle(this)" /> Select All Tables<br/>

    <?php 
    $count = 0;
    foreach($wpecus_tables_array['tables'] as $key => $table){
        if(wpecus_does_table_exist($table['name'])){
            ++$count;
            echo wpecus_notice('<input type="checkbox" name="wpecus_deletecoretables_array[]" value="'.$table['name'].'" /> ' . $count . '. ' . $table['name'],
            'success','Tiny','','','return');               
        }
    }
    
    if($count == 0){echo '<p>There are no core tables installed right now</p>';}
}    

/**
* Tool Tip
* 1. the javascript for this is added to the footer
* 
* @todo the tooltips script conflicts with WP Add Media button
*/
function wpecus_tt($label ='?',$title = 'Please visit www.wpecustomers.com for help'){
    echo $label;
    //echo '<span class="wpecustomerstooltip" title="'.$title.'">'.$label.'</span>';
}     

/**
* Displays a table of wpecus_option records with ability to view their value or delete them

* @todo LOWPRIORITY, allow output of specific types of options then make use of this on the Un-Install screen
* @param boolean $form true adds checkbox object to each option record (currently used on uninstall panel) 
*/
function wpecus_list_optionrecordtrace($form = false,$size = 'Small',$optiontype = 'all'){
    
    // first get all records that begin with wpecus_
    $wpecustomersrecords_result = wpecus_options_beginning_with('wpecus_');
    $counter = 1;?>
        
        <script language="JavaScript">
        function wpecus_deleteoptions_checkboxtoggle(source) {
          checkboxes = document.getElementsByName('wpecus_deleteoptions_array[]');
          for(var i in checkboxes)
            checkboxes[i].checked = source.checked;
        }
        </script>

        <input type="checkbox" onClick="wpecus_deleteoptions_checkboxtoggle(this)" /> Select All Options<br/>
    
    <?php       
    foreach($wpecustomersrecords_result as $key => $option ){
        
        if($form){
            $form = '<input type="checkbox" name="wpecus_deleteoptions_array[]" value="'.$option.'" />';
        }
        
        echo wpecus_notice($form . ' ' . $counter . '. ' . $option,'success',$size,'','','return');
        
        ++$counter;
    }
} 

function wpecus_GUI_plugin_screens_menuoptions($current){
    global $wpecus_mpt_arr;
    foreach($wpecus_mpt_arr['menu'] as $page_slug => $page_array){ 
        foreach($wpecus_mpt_arr['menu'][$page_slug]['tabs'] as $whichvalue => $screen_array){
            $selected = '';
            if($screen_array['slug'] == $current){
                $selected = 'selected="selected"';    
            }             
            echo '<option value="'.$screen_array['slug'].'" '.$selected.'>'.$screen_array['label'].'</option>'; 
        }
    }    
}

function wpecus_GUI_plugin_page_menuoptions($current){
    global $wpecus_mpt_arr;
    foreach($wpecus_mpt_arr['menu'] as $page_slug => $page_array){ 
        $selected = '';
        if($page_slug == $current){
            $selected = 'selected="selected"';    
        } 
        echo '<option value="'.$page_slug.'" '.$selected.'>'.$wpecus_mpt_arr['menu'][$page_slug]['title'].'</option>';
    }    
}           
?>
