<?php
/** 
 * WTG own Wordpress Plugin development function library for navigation in WP e-Customers plugin 
 * 
 * @link by WebTechGlobal.co.uk
 * 
 * @author Ryan Bayne | ryan@webtechglobal.co.uk
 *
 * @package WP e-Customers
 */
 
/**
 * Tabs menu loader - calls function for css only menu or jquery tabs menu
 * 
 * @param string $thepagekey this is the screen being visited
 */
function wpecus_createmenu($page_name){           
    global $wpecus_mpt_arr;
    
    echo '<h2 class="nav-tab-wrapper">';
        
    foreach($wpecus_mpt_arr['menu'][$page_name]['tabs'] as $tab=>$values){

        $tabslug = $wpecus_mpt_arr['menu'][$page_name]['tabs'][$tab]['slug'];
        $tablabel = $wpecus_mpt_arr['menu'][$page_name]['tabs'][$tab]['label'];  

        if(wpecus_menu_should_tab_be_displayed($page_name,$tab)){
        
            if(!isset($_GET['wpecustab']) && $tab == 0){
                $activeclass = 'class="nav-tab nav-tab-active"';
            }else{
                $activeclass = 'class="nav-tab"';
            }
            
            if(isset($_GET['wpecustab']) && $_GET['wpecustab'] == $tab){$activeclass = 'class="nav-tab nav-tab-active"';}
            echo '<a href="'.wpecus_create_adminurl($wpecus_mpt_arr['menu'][$page_name]['slug']).'&wpecustab='.$tab.'" '.$activeclass.'>' . $tablabel . '</a>';       

        }
    }      
    
    echo '</h2>';
} 

/**
* Used to determine if a screen is meant to be displayed or not, based on package and settings 
*/

/**
* Determines if giving tab for the giving page should be displayed or not based on current user.
* 
* Checks for reasons not to display and returns false. If no reason found to hide the tab then true is default.
* 
* @param mixed $page
* @param mixed $tab
* 
* @return boolean
*/
function wpecus_menu_should_tab_be_displayed($page,$tab){
    global $wpecus_mpt_arr,$wpecus_is_free,$wpecus_beta_mode;

    if(isset($wpecus_mpt_arr['menu'][$page]['tabs'][$tab]['permissions']['capability'])){
        $boolean = current_user_can( $wpecus_mpt_arr['menu'][$page]['tabs'][$tab]['permissions']['capability'] );
        if($boolean ==  false){
            return false;
        }
    }

    // if screen not active
    if(isset($wpecus_mpt_arr['menu'][$page]['tabs'][$tab]['active']) && $wpecus_mpt_arr['menu'][$page]['tabs'][$tab]['active'] == false){
        return false;
    }    
    
    // if screen is not active at all (used to disable a screen in all packages and configurations)
    if(isset($wpecus_mpt_arr['menu'][$page]['tabs'][$tab]['active']) && $wpecus_mpt_arr['menu'][$page]['tabs'][$tab]['active'] == false){
        return false;
    }
               
    // display screen if the package not set, just to be safe as the package value should also be set if menu array installed properly
    if(isset($wpecus_mpt_arr['menu'][$page]['tabs'][$tab]['package'])){      
      
        // dont allow beta mode screens to be displayed if plugin build is not in beta mode
        if($wpecus_mpt_arr['menu'][$page]['tabs'][$tab]['package'] == 'beta' && $wpecus_beta_mode == false){  
            return false;    
        } 
        
        if($wpecus_is_free && $wpecus_mpt_arr['menu'][$page]['tabs'][$tab]['package'] == 'paid'){   
            return false;
        } 
    } 
                 
    return true;      
}

/**
* includes the default screen file for the current user, either admin or subscriber/user
*/
function wpecus_include_default_screen($page_name){
    global $wpecus_mpt_arr;
    if(current_user_can( 'activate_plugins' )){ 
        
        foreach($wpecus_mpt_arr['menu'][$page_name]['tabs'] as $tab_number => $tab_array){       
            if(isset($tab_array['admindefault']) && $tab_array['admindefault'] == true){   
                include($wpecus_mpt_arr['menu'][$page_name]['tabs'][$tab_number]['path']);    
            }    
        }
            
    }else{
        
        foreach($wpecus_mpt_arr['menu'][$page_name]['tabs'] as $tab_number => $tab_array){
            if(isset($tab_array['userdefault']) && $tab_array['userdefault'] == true){
                include($wpecus_mpt_arr['menu'][$page_name]['tabs'][$tab_number]['path']);    
            }    
        }
                
    }
}

/**
* Returns value for displaying or hiding a page based on edition (free or full).
* These is no point bypassing this. The pages hidden require PHP that is only provided with
* the full edition. You may be able to use the forms, but the data saved won't do anything or might
* cause problems.
* 
* @param mixed $package_allowed, 0=free 1=full/paid 2=dont ever display
* @returns boolean true if screen is to be shown else false
* 
* @deprecated
*/
function wpecus_page_show_hide($package_allowed = 0){
    global $wpecus_is_free;
    
    if($package_allowed == 2){
        return false;// do not display in any package   
    }elseif($wpecus_is_free && $package_allowed == 0){
        return true;     
    }elseif($wpecus_is_free && $package_allowed == 1){
        return false;// paid edition page only, not be displayed in free edition
    }
    return true;
} 
?>
