<?php
/** 
 * WTG own Wordpress Plugin development function library for reports|notifications in WP e-Customers plugin 
 * 
 * @link by WebTechGlobal.co.uk
 * 
 * @author Ryan Bayne | ryan@webtechglobal.co.uk
 *
 * @package WP e-Customers
 */
 
/**                                              
* Outputs the contents of $wpecus_notice_array, used in wpecus_header_page.
* Will hold new and none persistent notifications. May also hold persistent. 
*/
function wpecus_notice_output(){
    $wpecus_notice_array = wpecus_get_option_persistentnotifications_array();
    if(isset($wpecus_notice_array['notifications'])){
 
        foreach($wpecus_notice_array['notifications'] as $key => $notice){
                            
            // persistent notices are handled by another function as the output is different
            if(!isset($notice['persistent']) || $notice['persistent'] != false){
        
                // set default values where any requires are null
                if(!isset($notice['type'])){$notice['type'] = 'info';}
                if(!isset($notice['helpurl'])){$notice['helpurl'] = false;} 
                if(!isset($notice['size'])){$notice['size'] = 'Large';} 
                if(!isset($notice['title'])){$notice['title'] = 'Sorry No Title Was Provided';}
                if(!isset($notice['message'])){$notice['message'] = 'Sorry No Message Was Provided';}
                if(!isset($notice['clickable'])){$notice['clickable'] = false;} 
                if(!isset($notice['persistent'])){$notice['persistent'] = false;}
                if(!isset($notice['id'])){$notice['id'] = false;} 
                  
                echo wpecus_notice_display($notice['type'],$notice['helpurl'],$notice['size'],$notice['title'],$notice['message'],$notice['clickable'],$notice['persistent'],$notice['id']);                                               
            
                // notice has been displayed so we now removed it
                unset($wpecus_notice_array['notifications'][$key]);
            }
        }
                           
        wpecus_update_option_persistentnotifications_array($wpecus_notice_array);
    }  
}

/**
* Creates a new notification with a long list of style options available.
* Can return the message for echo or add it to an array, which can also be stored for persistent messages.
* Requires visitor to be logged in and on an admin page, dont need to do prevalidation before calling function
*     
* Dont include a title attribute if you want to use defaults and stick to a standard format
*  
* @param string $message, main message
* @param mixed $type, determines colour styling (question,info,success,warning,error,processing,stop,step)
* @param mixed $size, determines box size (Tiny,Small,Large,Extra)
* @param mixed $title, a simple header
* @param mixed $helpurl, when required can offer link to help content (will be added closer to 2013)
* @param mixed $output_type (echo,return,public) 

* Output Types
* 1. echo - admin side only, adds notification to an array which is output in one place, allowing many notifications to be printed in a list
* 2. return - returns the html for doing as we want, usually we use echo to display notification in specific place
* 3. public - returns the html for use on front-end, bypasses is_admin() which exists as a safety measure to prevent admin information being displayed on front-end in error

* @deprecated, use wpecus_n()
*/
function wpecus_notice($message,$type = 'success',$size = 'Extra',$title = false, $helpurl = 'www.wpecustomers.com/support', $output_type = 'echo',$persistent = false,$clickable = false,$user_type = false){
    
    if(is_admin() || $output_type == 'public'){
        
        // change unexpected values into expected values (for flexability and to help avoid fault)
        if($type == 'accepted'){$type == 'success';}
        if($type == 'fault'){$type == 'error';}
        if($type == 'next'){$type == 'step';}
        
        // prevent div being clickable if help url giving (we need to more than one link in the message)
        if($helpurl != false && $helpurl != '' && $helpurl != 'http://www.wpecustomers.com/support'){$clickable = false;}
                                 
        if($output_type == 'return' || $output_type == 'public'){   
            return wpecus_notice_display($type,$helpurl,$size,$title,$message,$clickable,$persistent);
        }else{
            global $wpecus_notice_array;

            // establish next array key
            $next_key = 0;
            if(isset($wpecus_notice_array['notifications'])){
                $next_key = wpecus_get_array_nextkey($wpecus_notice_array['notifications']);
            }
            
            // add new message to the notifications array
            $wpecus_notice_array['notifications'][$next_key]['message'] = $message;
            $wpecus_notice_array['notifications'][$next_key]['type'] = $type;
            $wpecus_notice_array['notifications'][$next_key]['size'] = $size;
            $wpecus_notice_array['notifications'][$next_key]['title'] = $title;
            $wpecus_notice_array['notifications'][$next_key]['helpurl'] = $helpurl; 
            $wpecus_notice_array['notifications'][$next_key]['clickable'] = $clickable; 
                      
            wpecus_update_option_persistentnotifications_array($wpecus_notice_array);       
        }
    } 
}

/**
* Updates notifications array in Wordpress options table
* 
* @param array $notifications_array
* @return bool
*/
function wpecus_update_option_persistentnotifications_array($notifications_array){
    return update_option('wpecus_notifications',maybe_serialize($notifications_array));    
}

/**
* New Notice Function (replacing wpecus_notice())
* 1. None clickable (using a different function for that)
* 2. Not a Step Notice (also using a different function for that)
* 3. Does add link for help url though
*/
function wpecus_n($title,$mes,$style,$size,$atts = array()){
                
    extract( shortcode_atts( array( 
        'url' => false,
        'output' => 'norm',// default normal echos html, return will return html, public also returns it but bypasses is_admin() etc
        'audience' => 'administrator',// admin or user (use to display a different message to visitors than to staff)
        'user_mes' => 'No user message giving',// only used when audience is set to user, user_mes replaces $mes
        'side' => 'private',// private,public (use to apply themes styles if customised, do not use for security)      
        'clickable' => false,// boolean
    ), $atts ) );
              
    // do not allow a notice box if $output not stated as public and current visitor is not logged in
    // this forces backend messages which may be more private i.e. account info or key admin details
    if(!is_admin() && $output != 'public'){
        // visitor is on front-end, but the $output set is not for public
        return false; 
    }
                   
    // if return wanted or $side == public (used to bypass is_admin() check)
    // this allows the notice to be printed within content where the function is called rather than within the $wpecus_notice_array loop
    if($output == 'return' || $output == 'public'){
        return wpecus_notice_display($style,$url,$size,$title,$mes,$clickable,$persistent = false);
    }
                   
    // if user is not an administrator and notice is for administrators only do not display
    if ( $audience == 'administrator' && !current_user_can( 'manage_options' ) ) {
        return false;
    }
            
    // arriving here means normal, most common output to the backend of Wordpress
    $wpecus_notice_array = wpecus_get_option_persistentnotifications_array();
    
    // set next array key value
    $next_key = 0;

    // determine next array key
    if(isset($wpecus_notice_array['notifications'])){    
        $next_key = wpecus_get_array_nextkey($wpecus_notice_array['notifications']);
    }    
                   
    // add new message to the notifications array
    // this will be output during the current page loading. The notification will show once unless persistent is set to true
    $wpecus_notice_array['notifications'][$next_key]['message'] = $mes;
    $wpecus_notice_array['notifications'][$next_key]['type'] = $style;
    $wpecus_notice_array['notifications'][$next_key]['size'] = $size;
    $wpecus_notice_array['notifications'][$next_key]['title'] = $title;
    $wpecus_notice_array['notifications'][$next_key]['helpurl'] = $url; 
    $wpecus_notice_array['notifications'][$next_key]['output'] = $output;
    $wpecus_notice_array['notifications'][$next_key]['audience'] = $audience;
    $wpecus_notice_array['notifications'][$next_key]['user_mes'] = $user_mes;
    $wpecus_notice_array['notifications'][$next_key]['side'] = $side;
    $wpecus_notice_array['notifications'][$next_key]['clickable'] = $clickable;
                      
    wpecus_update_option_persistentnotifications_array($wpecus_notice_array);
}


/**
* Displays persistent notices, the ones users need to delete manually.
* The parameters allow filtering to display notices in their intended parts of the interface.
* 
* @param mixed $placement_type global OR page OR screen OR panel
* @param mixed $placement_specific page slug OR screens tab number OR panel id
* @param string $pageid used with screen number, $placement_specific is the screen number
* 
* When $placement_type is global, all other parameteres are false
*/
function wpecus_persistentnotice_output($placement_type,$placement_specific = false,$pageid = false){
    $wpecus_notice_array = wpecus_get_option_persistentnotifications_array();

    if(!is_array($wpecus_notice_array) || !isset($wpecus_notice_array['notifications'])){
        return false;
    }   

    foreach($wpecus_notice_array['notifications'] as $key => $n){                                                                               
        if(isset($n['persistent']) && $n['persistent'] == true){
            if($placement_type == $n['placement_type'] && $placement_specific == $n['placement_specific'] && $pageid == $n['pageid']){
                
                // if user is not an administrator and notice is for administrators only do not display
                $display_this_notice = true;
                
                if ( $n['audience'] == 'administrator' && !current_user_can( 'manage_options' ) ) {
                    $display_this_notice == false;# customers and visitors wont see administration information
                }
                    
                if($display_this_notice){
                    echo wpecus_persistentnotice_display($n['type'],$n['helpurl'],$n['size'],$n['title'],$n['message'],true,$n['id']);    
                }
            }
        }
    }
}

/**
* Creates a persistent notice that the use must delete manually
* 
* @param mixed $title
* @param mixed $message
* @param mixed $type
* @param mixed $size
* @param mixed $atts
*/
function wpecus_notice_persistent($title,$message,$type,$size,$atts){

    extract( shortcode_atts( array( 
        'url' => false,
        'output' => 'norm',// default normal echos html, return will return html, public also returns it but bypasses is_admin() etc
        'audience' => 'admin',// admin or user (use to display a different message to visitors than to staff)
        'user_mes' => 'No user message giving',// only used when audience is set to user, user_mes replaces $mes
        'side' => 'private',// private,public (use to apply themes styles if customised, do not use for security)      
        'clickable' => false,// boolean
        'placement_type' => 'global',// global, page, screen, panel
        'placement_specific' => false,// page slug, screen number, panel id
        'pageid' => false,// used when placement_type is screen
        'id' => rand(100,10000) . time(),// instance id             
    ), $atts ) );

    // get persistent notice array
    $wpecus_notice_array = wpecus_get_option_persistentnotifications_array();

    // avoid 
    // set next array key value
    $next_key = 0;

    // determine next array key
    if(isset($per['notifications'])){
        $next_key = wpecus_get_array_nextkey($per['notifications']);
    }   
     
    $wpecus_notice_array['notifications'][$next_key]['message'] = $message;
    $wpecus_notice_array['notifications'][$next_key]['type'] = $type;
    $wpecus_notice_array['notifications'][$next_key]['size'] = $size;
    $wpecus_notice_array['notifications'][$next_key]['title'] = $title;
    $wpecus_notice_array['notifications'][$next_key]['helpurl'] = $url; 
    $wpecus_notice_array['notifications'][$next_key]['output'] = $output;
    $wpecus_notice_array['notifications'][$next_key]['audience'] = $audience;
    $wpecus_notice_array['notifications'][$next_key]['user_mes'] = $user_mes;
    $wpecus_notice_array['notifications'][$next_key]['side'] = $side;
    $wpecus_notice_array['notifications'][$next_key]['clickable'] = $clickable;
    $wpecus_notice_array['notifications'][$next_key]['placement_type'] = $placement_type;      
    $wpecus_notice_array['notifications'][$next_key]['placement_specific'] = $placement_specific;  
    $wpecus_notice_array['notifications'][$next_key]['pageid'] = $pageid;  
    $wpecus_notice_array['notifications'][$next_key]['persistent'] = true;
                        
    // generate a unique notice ID, no need to validate it if we use time()
    $wpecus_notice_array['notifications'][$next_key]['id'] = $id;
                       
    wpecus_update_option_persistentnotifications_array($wpecus_notice_array);
}

function wpecus_persistentnotice_display($type,$helpurl,$size,$title,$message,$persistent = false,$id = false){
    // ID is required to prevent conflict with other persistent notices and for deleting them
    if($id == false){
        return;
    }
    
    global $wpecus_form_action; 

    // start div
    $output = '<div class="'.$type.$size.'">';
        
    // if is not clickable (entire div) and help url is not null then display a clickable icon
    $thelink = '';
    if($helpurl != '' && $helpurl != false){
        ### TODO:HIGHPRIORITY, finish ability to add a help link
        //$thelink = '<a href="'.$helpurl.'" target="_blank">Get Help</a>';
    }   
   
    // set h4 when required
    if($size != 'Tiny'){$output .= '<h4>'.$title.'</h4>';}

    // if persistent message, add link for deleting the message
    $output .= '
    <p>' . $message . '</p>

    <form id="wpecus_notice_delete" method="post" name="wpecus_notice_delete" action="'.$wpecus_form_action.'">
        
        <input type="hidden" id="wpecus_post_processing_required" name="wpecus_post_processing_required" value="true">       
        <input type="hidden" id="wpecus_post_deletenotice_id" name="wpecus_post_deletenotice_id" value="'.$id.'">       
        <input type="hidden" id="wpecus_post_deletenotice" name="wpecus_post_deletenotice" value="true">
        
        <div class="jquerybutton">
        
            <button id="wpecus_notice_delete_'.$id.'">Delete Notice</button>

            '.$thelink.'

        </div>
    </form>';    
    
    // complete notice with closing div
    $output .= '</div>';

    return $output;    
}

/**
* Returns notification HTML.
* This function has the html and css to make all notifications standard.

* @param mixed $type
* @param string $helpurl
* @param string $size
* @param string $title
* @param string $message
* @param bool $clickable
* @param mixed $persistent
* @param mixed $id, used for persistent messages which use jQuery UI button, the ID should be the notice ID
*/
function wpecus_notice_display($type,$helpurl,$size,$title,$message,$clickable,$persistent = false,$id = false){
    // begin building output
    $output = '';
                    
    // if clickable (only allowed when no other links being used) - $helpurl will actually be a local url to another plugin or Wordpress page
    if($clickable){
        $output .= '<div class="stepLargeTest"><a href="'.$helpurl.'">';
    }

    // start div
    $output .= '<div class="'.$type.$size.'">';     
   
    // set h4 when required
    if($size != 'Tiny'){$output .= '<h4>'.$title.'</h4>';}

    $output .= '<p>' . $message . '</p>';

    // if is not clickable (entire div) and help url is not null then display a clickable ico
    $thelink = '';
    if($helpurl != '' && $helpurl != false){
        //$output .= '<a class="jquerybutton" href="'.$helpurl.'" target="_blank">Get Help</a>';
    }   
        
    // complete notice with closing div
    $output .= '</div>';
    
    // end wrapping with link and styled div for making div clickable when required
    if($clickable){$output .= '</a></div>';}

    return $output;    
}

/**
* Standard notice format to be displayed on submission of a form (Large).
* Change the size if a plugin using core is to have a different format.
* 
* @deprecated use wpecus_n_postresult as of 13th February 2013
*/
function wpecus_notice_postresult($type,$title,$message,$helpurl = false,$user = 'admin'){
    wpecus_notice($message,$type,'Large',$title, $helpurl, 'echo');    
}
                        
/**
* Display a standard notification after form submission.
* This function simply helps to quickly apply a notice to the outcome while using the same
* configuration for all form submissions.
* 
* @param mixed $type
* @param mixed $title
* @param mixed $message
* @param mixed $helpurl
* @param mixed $user
*/
function wpecus_n_postresult($type,$title,$message,$helpurl = false,$user = 'admin'){  
    wpecus_n($title,$message,$type,'Large');  
}
     
/**
* Standard notice format to be displayed at the top of a screen (Tiny)
*/
function wpecus_n_screeninfo($title,$message,$helpurl = false,$user = 'admin'){
    echo wpecus_notice($message,'info','Tiny',$title, $helpurl, 'return');    
}

/**
* Standard notice format to be displayed inside a panel as information
*/
function wpecus_n_panelinfo($title,$message,$helpurl = false,$user = 'admin'){
    echo wpecus_notice($message,'info','Small',$title, $helpurl, 'return');    
}

/**
* Standard notice format for displaying notice on public side
*/
function wpecus_n_public($title,$message,$helpurl = false){
    echo wpecus_n($title,$message,'info','Small',array('output' => 'public','url' => $helpurl));    
}

/**
* Notice html is printed where this function is used 
*/
function wpecus_n_incontent($message,$type = 'info',$size = 'Small',$title = '',$helpurl = ''){
    echo wpecus_notice($message,$type,$size,$title, $helpurl, 'return');    
}

/**
* Returns the html for a notice box that can be clicked on and styled as the "Next Step".
* This is easier to use than wtgcsv_notice() for when the next step box is required.
* 
* 1. Do not add links inside the message content as the entire div is linked
* 2. This is only be used to suggest the next step users should take
* 3. This function always returns, it cannot be used to update the notifications array
* 
* @param mixed $type
* @param mixed $helpurl
* @param mixed $size
* @param mixed $title
* @param mixed $message
* @todo MEDIUMPRIORITY, do a check to determine if user is already on the clickable url, if they are then we suggest it on the message
*/
function wpecus_notice_display_step($helpurl,$title,$message){
    $output = '<div class="stepLargeTest"><a href="'.$helpurl.'">    
    <div class="stepLarge"><h4>'.$title.'</h4>'. $message .'</div></a></div>';
    return $output;    
}

/**
* Displays notice if combined CSV file size is more than recommended for the interface to easily handle
* 104857600 = 100 MB 
*/
function wpecus_notice_filesizetotal($mb){
    if($mb > 104857600){
        echo wpecus_notice('Your CSV files combined size is larger than 100MB, it is recommended that you delete any files
        not in use to reduce memory usage and increase plugin interface performance','notice','Tiny','','','return');
    }      
}

/**
* Performs a var_dump with <pre> using the giving value, but only if debug mode active.
* This is good for debugging and not accidently leaving the dump in the code on release 
*/
function wpecus_var_dump($v,$h = false,$append = '<br>'){
    global $wpecus_debug_mode;
    if($wpecus_debug_mode){
        
        // if header wanted
        if($h){
            echo '<h1>'.$h.'</h1>';
        }
        
        echo '<pre>';
        var_dump($v);
        echo '</pre>';
    }
}

/**
* WP e-Customers function for logging project changes.
* The main intention is to aid support. We will use this log to check what the user is doing with their project.
* 
* @param mixed $post_id
* @param mixed $post_title
* @param mixed $action (created,updated,deleted,checked) (check: for updating etc but nothing done, just the check done)
* @param mixed $message
* @param mixed $project_name
*/
function wpecus_log_posts($post_id,$post_title,$action,$message,$project_name){
    $atts = array();   
    $atts['logged'] = wpecus_date();
    $atts['postid'] = $post_id;
    $atts['posttitle'] = $post_title;
    $atts['action'] = $action;
    $atts['message'] = $message; 
    $atts['projectname'] = $project_name;
    // set log type so the log entry is made to the required log file
    $atts['type'] = 'posts';
    wpecus_log($atts);    
}

/**
* Use this to log automated events and track progress in automated scripts.
* Mainly used in schedule function but can be used in any functions called by add_action() or
* other processing that is triggered by user events but not specifically related to what the user is doing.
* 
* @param mixed $outcome
* @param mixed $trigger schedule, hook (action hooks such as text spinning could be considered automation), cron, url, user (i.e. user does something that triggers background processing)
* @param mixed $line
* @param mixed $file
* @param mixed $function
*/
function wpecus_log_schedule($comment,$action,$outcome,$category = 'scheduledeventaction',$trigger = 'autoschedule',$line = 'NA',$file = 'NA',$function = 'NA'){
    $atts = array();   
    $atts['logged'] = wpecus_date();
    $atts['comment'] = $comment;
    $atts['action'] = $action;
    $atts['outcome'] = $outcome;
    $atts['category'] = $category;
    $atts['line'] = $line;
    $atts['file'] = $file;
    $atts['function'] = $function;
    $atts['trigger'] = $function;
    // set log type so the log entry is made to the required log file
    $atts['type'] = 'automation';
    wpecus_log($atts);    
}

/**
* Make log entries into the data log file, a log file specific to WP e-Customers. 
* This function is not to be moved to the wtgcore. 
*/
function wpecus_log_data($comment,$line,$file,$function,$dump = 'NA',$sqlresult = 'NA',$sqlquery = 'NA',$lastrecordedid = 'NA',$jobid = 'NA'){
    $atts = array();   
    $atts['lastrecordedid'] = $lastrecordedid;
    $atts['jobid'] = $jobid;
    $atts['comment'] = $comment;
    $atts['line'] = $line;
    $atts['file'] = $file;
    $atts['function'] = $function;
    $atts['dump'] = $dump;
    $atts['sqlresult'] = $sqlresult;
    $atts['sqlquery'] = $sqlquery;
    // set log type so the log entry is made to the required log file
    $atts['type'] = 'data';    
    wpecus_log($atts);       
}

/**
* Gets notifications array if it exists in Wordpress options table else returns empty array
*/
function wpecus_get_option_persistentnotifications_array(){
    $a = get_option('wpecus_notifications');
    $v = maybe_unserialize($a);
    if(!is_array($v)){
        return array();    
    }
    return $v;    
}

/**
* Used to build history, flag items and schedule actions to be performed.
* 1. it all falls under log as we would probably need to log flags and scheduled actions anyway
*
* @global $wpdb
* @uses extract, shortcode_atts
*
* @todo create other constants like the one setup for sql log entries
* @todo create option to add entries to server error log file
* @todo create a function for each 'type'
* 
* @link http://www.wpecustomers.com/hacking/log-table
*/
function wpecus_log($atts){     
    global $wpecus_adm_set,$wpdb,$wpecus_currentversion;

    // if ALL logging is off - if ['uselog'] not set then logging for all files is on by default
    if(isset($wpecus_adm_set['reporting']['uselog']) && $wpecus_adm_set['reporting']['uselog'] == 0){return false;}
    
    // if log table does not exist return false
    if(!wpecus_does_table_exist('wpecus_log')){return false;}
    
    // if a value is false, it will not be added to the insert query, we want the database default to kick in, NULL mainly
    extract( shortcode_atts( array(  
        'outcome' => 1,# 0|1 (overall outcome in boolean) 
        'line' => false,# __LINE__ 
        'function' => false,# __FUNCTION__
        'file' => false,# __FILE__ 
        'sqlresult' => false,# dump of sql query result 
        'sqlquery' => false,# dump of sql query 
        'sqlerror' => false,# dump of sql error if any 
        'wordpresserror' => false,# dump of a wp error 
        'screenshoturl' => false,# screenshot URL to aid debugging 
        'userscomment' => false,# beta testers comment to aid debugging (may double as other types of comments if log for other purposes) 
        'page' => false,# related page 
        'version' => $wpecus_currentversion, 
        'panelid' => false,# id of submitted panel
        'panelname' => false,# name of submitted panel 
        'tabscreenid' => false,# id of the menu tab  
        'tabscreenname' => false,# name of the menu tab 
        'dump' => false,# dump anything here 
        'ipaddress' => false,# users ip 
        'userid' => false,# user id if any    
        'noticemessage' => false,# when using log to create a notice OR if logging a notice already displayed      
        'comment' => false,# dev comment to help with troubleshooting
        'type' => false,# general|error|trace 
        'category' => false,# createposts|importdata|uploadfile|deleteuser|edituser 
        'action' => false,# 3 posts created|22 posts updated (the actuall action performed)
        'priority' => false,# low|normal|high (use high for errors or things that should be investigated, use low for logs created mid procedure for tracing progress)                        
        'trigger' => false# autoschedule|cronschedule|wpload|manualrequest
    ), $atts ) );
    
    // start query
    $query = "INSERT INTO `wpecus_log`";
    
    // add columns and values
    $query_columns = '(outcome';
    $query_values = '(1';
    
    if($line){$query_columns .= ',line';$query_values .= ',"'.$line.'"';}
    if($file){$query_columns .= ',file';$query_values .= ',"'.$file.'"';}                                                                           
    if($function){$query_columns .= ',function';$query_values .= ',"'.$function.'"';}  
    if($sqlresult){$query_columns .= ',sqlresult';$query_values .= ',"'.$sqlresult.'"';}     
    if($sqlquery){$query_columns .= ',sqlquery';$query_values .= ',"'.$sqlquery.'"';}     
    if($sqlerror){$query_columns .= ',sqlerror';$query_values .= ',"'.$sqlerror.'"';}    
    if($wordpresserror){$query_columns .= ',wordpresserror';$query_values .= ',"'.$wordpresserror.'"';}     
    if($screenshoturl){$query_columns .= ',screenshoturl';$query_values .= ',"'.$screenshoturl.'"' ;}     
    if($userscomment){$query_columns .= ',userscomment';$query_values .= ',"'.$userscomment.'"';}     
    if($page){$query_columns .= ',page';$query_values .= ',"'.$page.'"';}     
    if($version){$query_columns .= ',version';$query_values .= ',"'.$version.'"';}     
    if($panelid){$query_columns .= ',panelid';$query_values .= ',"'.$panelid.'"';}     
    if($panelname){$query_columns .= ',panelname';$query_values .= ',"'.$panelname.'"';}     
    if($tabscreenid){$query_columns .= ',tabscreenid';$query_values .= ',"'.$tabscreenid.'"';}     
    if($tabscreenname){$query_columns .= ',tabscreenname';$query_values .= ',"'.$tabscreenname.'"';}     
    if($dump){$query_columns .= ',dump';$query_values .= ',"'.$dump.'"';}     
    if($ipaddress){$query_columns .= ',ipaddress';$query_values .= ',"'.$ipaddress.'"';}     
    if($userid){$query_columns .= ',userid';$query_values .= ',"'.$userid.'"';}     
    if($noticemessage){$query_columns .= ',noticemessage';$query_values .= ',"'.$noticemessage.'"';}     
    if($comment){$query_columns .= ',comment';$query_values .= ',"'.$comment.'"';}     
    if($type){$query_columns .= ',type';$query_values .= ',"'.$type.'"';}     
    if($category){$query_columns .= ',category';$query_values .= ',"'.$category.'"';}     
    if($action){$query_columns .= ',action';$query_values .= ',"'.$action.'"';}     
    if($priority){$query_columns .= ',priority';$query_values .= ',"'.$priority.'"';}     
    if($trigger){$query_columns .= ',trigger';$query_values .= ',"'.$trigger.'"';}
    
    $query_columns .= ')';
    $query_values .= ')';
    $query .= $query_columns .' VALUES '. $query_values;  
    $wpdb->query( $query );     
}

/**
* Log form submissions on admin side
*/
function wpecus_log_adminform($function,$outcome,$action,$comment = 'unknown',$type = 'general',$priority = 'normal',$dump = 'unknown',$wordpresserror = 'none'){    
                           
    $atts = array();             
    $atts['outcome'] = $outcome;
    $atts['function'] = $function; 
    $atts['wordpresserror'] = $wordpresserror;;
    //$atts['tabscreenname'] = $tabscreenname; # not in use 
    $atts['dump'] = $dump;
    $atts['comment'] = $comment;
    $atts['type'] = $type;
    $atts['action'] = $action;
    $atts['priority'] = $priority;          
    
    // default values
    $atts['userid'] = get_current_user_id();       
    $atts['category'] = 'adminform'; 
    $atts['page'] = $_POST['wpecus_hidden_pageid'];
    $atts['panelname'] = $_POST['wpecus_hidden_panel_name'];
    $atts['paneltitle'] = $_POST['wpecus_hidden_panel_title'];
    $atts['tabscreenid'] = $_POST['wpecus_hidden_tabnumber'];                              
            
    wpecus_log($atts);     
}

function wpecus_log_urlaction($function,$outcome,$action,$comment = 'unknown',$type = 'general',$priority = 'normal',$dump = 'unknown',$wordpresserror = 'none'){    
                           
    $atts = array();             
    $atts['outcome'] = $outcome;
    $atts['function'] = $function; 
    $atts['wordpresserror'] = $wordpresserror;;
    //$atts['tabscreenname'] = $tabscreenname; # not in use 
    $atts['dump'] = $dump;
    $atts['comment'] = $comment;
    $atts['type'] = $type;
    $atts['action'] = $action;
    $atts['priority'] = $priority;          
    
    // default values
    $atts['userid'] = get_current_user_id();       
    $atts['category'] = 'adminform';                               
            
    wpecus_log($atts);     
}


/**
* This will log sql queries and details of any failures in a way that helps debug. 
*/
function wpecus_log_sql($comment,$function,$file,$line,$recordid = 'NA',$dump = 'NA',$sqlresult = 'NA',$sqlquery = 'NA',$outcome = 'success'){
    $atts = array();             
    $atts['outcome'] = $outcome;// success,failed
    $atts['comment'] = $comment;
    $atts['line'] = $line;
    $atts['file'] = $file;
    $atts['function'] = $function;
    $atts['dump'] = $dump;
    $atts['sqlresult'] = $sqlresult;
    $atts['sqlquery'] = $sqlquery;
    $atts['recordid'] = $recordid;
    // set log type so the log entry is made to the required log file
    $atts['type'] = 'sql';    
    wpecus_log($atts);    
}        

/**
* Admin log is about tracing admin users actions, tracing staffs use.
* Place the wpecus_log_user() function in form processing.
*/
function wpecus_log_users($message,$ipaddress = 'NA',$userid = 'NA',$panelname = 'NA',$paneltitle = 'NA',$page = 'NA'){
    $atts = array();              
    $atts['userid'] = $userid;
    $atts['message'] = $message;
    $atts['panelname'] = $panelname;
    $atts['paneltitle'] = $paneltitle;
    $atts['page'] = $page;
    $atts['ipaddress'] = $ipaddress;
    // set log type so the log entry is made to the required log file
    $atts['type'] = 'users';    
    wpecus_log($atts);  
}

/**
* Log all errors (these are plugin faults where we reach a place in code we should not and need to know about it)
* Do not log users own mistakes here (error is a term used in notices but it is not the same type of error)  
*/
function wpecus_log_error($message,$line,$function,$file,$dump = 'NA'){
    $atts = array();             
    $atts['line'] = $line;
    $atts['function'] = $function;
    $atts['file'] = $file;
    $atts['message'] = $message;
    $atts['dump'] = $dump;
    // set log type so the log entry is made to the required log file
    $atts['type'] = 'error';    
    wpecus_log($atts);     
}

/**
* Make log entry in install log
*/
function wpecus_log_install($action,$message,$userid = 'NA'){
    $atts = array();             
    $atts['action'] = $action;
    $atts['userid'] = $userid;
    $atts['message'] = $message;
    // set log type so the log entry is made to the required log file
    $atts['type'] = 'install';    
    wpecus_log($atts);    
}

/**
* Cleanup log table - currently keeps 2 days of logs
*/
function wpecus_log_cleanup(){     
    if(wpecus_database_table_exist('wpecus_log')){
        global $wpdb;
        
        $twodays_time = strtotime('2 days ago midnight');

        $twodays = date("Y-m-d H:i:s",$twodays_time);
        
        $wpdb->query( 
            "
                DELETE FROM wpecus_log
                WHERE timestamp < '".$twodays."'
            "
        );
    }
}


/**
* Default array for log screen
*
* @package WP e-Customers
* @since 7.0.4
*
* @return array $search_headers_array 
*/
function wpecus_logscreen_defaultarray(){
    // search headers (as used in log table)
    $search_headers_array = array();
    //rowid (treated different from others)
    $search_headers_array['rowid']['title'] = 'Row ID';
    $search_headers_array['rowid']['select'] = true;// add to SELECT part of query
    $search_headers_array['rowid']['display'] = true;// display in table
    // outcome
    $search_headers_array['outcome']['title'] = 'Outcome';
    $search_headers_array['outcome']['select'] = false;
    $search_headers_array['outcome']['display'] = false;
    $search_headers_array['outcome']['searchoptions'] = false;// 0|1
    // type
    $search_headers_array['type']['title'] = 'Type';
    $search_headers_array['type']['select'] = false;
    $search_headers_array['type']['display'] = false;
    $search_headers_array['type']['searchoptions'] = false;// general|error|trace
    // priority
    $search_headers_array['priority']['title'] = 'Priority';
    $search_headers_array['priority']['select'] = false;
    $search_headers_array['priority']['display'] = false;
    $search_headers_array['priority']['searchoptions'] = false;// low|high|medium
    // timestamp
    $search_headers_array['timestamp']['title'] = 'Date + Time';
    $search_headers_array['timestamp']['select'] = false;
    $search_headers_array['timestamp']['display'] = false;
    $search_headers_array['timestamp']['searchoptions'] = false;// onehour|oneday|oneweek 
    // line
    $search_headers_array['line']['title'] = 'Line';
    $search_headers_array['line']['select'] = false;
    $search_headers_array['line']['display'] = false;
    $search_headers_array['line']['searchoptions'] = false;
    // file
    $search_headers_array['file']['title'] = 'File';
    $search_headers_array['file']['select'] = false;
    $search_headers_array['file']['display'] = false;
    $search_headers_array['file']['searchoptions'] = false;    
    // function
    $search_headers_array['function']['title'] = 'Function';
    $search_headers_array['function']['select'] = false;
    $search_headers_array['function']['display'] = false;
    $search_headers_array['function']['searchoptions'] = false;
    // sqlresult
    $search_headers_array['sqlresult']['title'] = 'SQL Result';
    $search_headers_array['sqlresult']['select'] = false;
    $search_headers_array['sqlresult']['display'] = false;
    $search_headers_array['sqlresult']['searchoptions'] = false;
    // sqlquery
    $search_headers_array['sqlquery']['title'] = 'SQL Query';
    $search_headers_array['sqlquery']['select'] = false;
    $search_headers_array['sqlquery']['display'] = false;
    $search_headers_array['sqlquery']['searchoptions'] = false;
    // sqlerror
    $search_headers_array['sqlerror']['title'] = 'SQL Error';
    $search_headers_array['sqlerror']['select'] = false;
    $search_headers_array['sqlerror']['display'] = false;
    $search_headers_array['sqlerror']['searchoptions'] = false;
    // wordpresserror
    $search_headers_array['wordpresserror']['title'] = 'WP Error';
    $search_headers_array['wordpresserror']['select'] = false;
    $search_headers_array['wordpresserror']['display'] = false;
    $search_headers_array['wordpresserror']['searchoptions'] = false;
    // screenshoturl
    $search_headers_array['screenshoturl']['title'] = 'Screenshot URL';
    $search_headers_array['screenshoturl']['select'] = false;
    $search_headers_array['screenshoturl']['display'] = false;
    $search_headers_array['screenshoturl']['searchoptions'] = false;
    // userscomment
    $search_headers_array['userscomment']['title'] = 'Users Comment';
    $search_headers_array['userscomment']['select'] = false;
    $search_headers_array['userscomment']['display'] = false;
    $search_headers_array['userscomment']['searchoptions'] = false;
    // page
    $search_headers_array['page']['title'] = 'Page';
    $search_headers_array['page']['select'] = false;
    $search_headers_array['page']['display'] = false;
    $search_headers_array['page']['searchoptions'] = false;
    // version
    $search_headers_array['version']['title'] = 'Plugin Version';
    $search_headers_array['version']['select'] = false;
    $search_headers_array['version']['display'] = false;
    $search_headers_array['version']['searchoptions'] = false;
    // panelname
    $search_headers_array['panelname']['title'] = 'Panel Name';
    $search_headers_array['panelname']['select'] = false;
    $search_headers_array['panelname']['display'] = false;
    $search_headers_array['panelname']['searchoptions'] = false;
    // tabscreenid
    $search_headers_array['tabscreenid']['title'] = 'Screen ID';
    $search_headers_array['tabscreenid']['select'] = false;
    $search_headers_array['tabscreenid']['display'] = false;
    $search_headers_array['tabscreenid']['searchoptions'] = false;
    // tabscreenname
    $search_headers_array['tabscreenname']['title'] = 'Screen Name';
    $search_headers_array['tabscreenname']['select'] = false;
    $search_headers_array['tabscreenname']['display'] = false;
    $search_headers_array['tabscreenname']['searchoptions'] = false;
    // dump
    $search_headers_array['dump']['title'] = 'Dump';
    $search_headers_array['dump']['select'] = false;
    $search_headers_array['dump']['display'] = false;
    $search_headers_array['dump']['searchoptions'] = false;
    // ipaddress
    $search_headers_array['ipaddress']['title'] = 'IP Address';
    $search_headers_array['ipaddress']['select'] = false;
    $search_headers_array['ipaddress']['display'] = false;
    $search_headers_array['ipaddress']['searchoptions'] = false;
    // userid
    $search_headers_array['userid']['title'] = 'User ID';
    $search_headers_array['userid']['select'] = false;
    $search_headers_array['userid']['display'] = false;
    $search_headers_array['userid']['searchoptions'] = false;
    // comment
    $search_headers_array['comment']['title'] = 'Comment';
    $search_headers_array['comment']['select'] = false;
    $search_headers_array['comment']['display'] = false;
    $search_headers_array['comment']['searchoptions'] = false;       
    // category
    $search_headers_array['category']['title'] = 'Category';
    $search_headers_array['category']['select'] = false;
    $search_headers_array['category']['display'] = false;
    $search_headers_array['category']['searchoptions'] = false;
    // action
    $search_headers_array['action']['title'] = 'Action';
    $search_headers_array['action']['select'] = false;
    $search_headers_array['action']['display'] = false;
    $search_headers_array['action']['searchoptions'] = false;         
    // thetrigger
    $search_headers_array['thetrigger']['title'] = 'Trigger';
    $search_headers_array['thetrigger']['select'] = false;
    $search_headers_array['thetrigger']['display'] = false; 
    $search_headers_array['thetrigger']['searchoptions'] = false;
    
    return $search_headers_array;              
}

/**
* Updates the default headers array with stored criteria selections, $_GET criteria and columns to be displayed.
* 
* 1. The columns displayed are updated with $_GET criteria automatically.
* 2. The interface reflects $_GET search but the criteria is not stored, user must submit form to save the criteria they see
*
* @package WP e-Customers
* @since 7.0.4
*
* @return array $search_headers_array 
*/
function wpecus_logscreen_setcriteria($search_headers_array){
    global $wpecus_adm_set;

    if(isset($_GET['wpecuslogsearch'])){
        foreach($search_headers_array as $key => $header){
            if(isset($_GET[$key . 'criteria'])){

                $search_headers_array[$key]['searchoptions'] = $_GET[$key . 'criteria'];// changes from false to indicate we want to use the preset criteria value
                $search_headers_array[$key]['select'] = true;
                $search_headers_array[$key]['display'] = true;

            }            
        } 
    }elseif(!isset($_GET['wpecuslogsearch'])){
        foreach($search_headers_array as $key => $header){
            if(isset($wpecus_adm_set['log']['logscreen'][$key . 'criteria'])){

                $search_headers_array[$key]['searchoptions'] = $wpecus_adm_set['log']['logscreen'][$key . 'criteria'];// changes from false to indicate we want to use the preset criteria value
                $search_headers_array[$key]['select'] = true;
                $search_headers_array[$key]['display'] = true;

            }               
        }
    }  
    
    return $search_headers_array;      
}

/**
* Updates log screen array with column display values from admin settings

* @package WP e-Customers
* @since 7.0.4
*
* @return array $search_headers_array 
*/
function wpecus_logscreen_setcolumns($search_headers_array) {
    global $wpecus_adm_set;                                 
    foreach($search_headers_array as $key => $header){
        if(isset($wpecus_adm_set['log']['logscreen']['displayedcolumns'][$key]) && $wpecus_adm_set['log']['logscreen']['displayedcolumns'][$key] == true){
            $search_headers_array[$key]['select'] = true;
            $search_headers_array[$key]['display'] = true;
        }            
    } 
            
    return $search_headers_array;
}

/**
* Builds query and executes for log screen based on stored criteria or $_GET values
*
* @package WP e-Customers
* @since 7.0.4
*
* @return $query 
*/
function wpecus_logscreen_query($search_headers_array) {
    global $wpdb;
           
    $type = 'all';

    $select = 'rowid,timestamp';
    foreach($search_headers_array as $column => $column_array){
        if($search_headers_array[$column]['select'] == true){
            $select .= ',' . $column;    
        }
    }    

    // where
    $where = 'rowid IS NOT NULL ';
    foreach($search_headers_array as $column => $column_array){
        
        if(isset($search_headers_array[$column]['searchoptions']) && !empty($search_headers_array[$column]['searchoptions'])){
           
            $where .= '
            AND (';
            
            $array = explode(',',$search_headers_array[$column]['searchoptions']);
            
            $total_strings = count($array);
            
            $added = 0;
            foreach($array as $key => $string){
                
                ++$added;
                
                $where .= $column . ' = "' . $string .'"';
                
                if($total_strings > 0 && $added != $total_strings){
                    $where .= ' OR ';
                } 
            }  
            
            $where .= ')';
        }
        
    } 
                          
    $query = "SELECT ".$select." 
    FROM wpecus_log 
    WHERE ".$where."
    ORDER BY timestamp DESC
    LIMIT 200";
                         
    // get_results
    return $wpdb->get_results($query,ARRAY_A);
}
?>