<?php
/** 
 * WTG own WP Core functions in WP e-Customers 
 * 
 * @link by WebTechGlobal.co.uk
 * 
 * @author Ryan Bayne | ryan@webtechglobal.co.uk
 *
 * @package WP e-Customers
 */
 
/**
* Decides a tab screens required capability in order for dashboard visitor to view it
* 
* @param mixed $page_name the array key for pages
* @param mixed $tab_key the array key for tabs within a page
*/
function wpecus_WP_SETTINGS_get_tab_capability($page_name,$tab_key,$default = false){
    global $wpecus_mpt_arr,$wpecus_demo_mode;
    $codedefault = 'activate_plugins';
    if($wpecus_demo_mode){
        
        if(isset($wpecus_mpt_arr['menu'][$page_name]['tabs'][$tab_key]['permissions']['demoallowed']) && $wpecus_mpt_arr['menu'][$page_name]['tabs'][$tab_key]['permissions']['demoallowed'] == false){
            // the page is not permitted to be used during demo mode
        }elseif(!isset($wpecus_mpt_arr['menu'][$page_name]['tabs'][$tab_key]['permissions']['demoallowed']) || $wpecus_mpt_arr['menu'][$page_name]['tabs'][$tab_key]['permissions']['demoallowed'] == true){
            // page permits demo use in demo mode and we are in demo mode (otherwise all arguments passed and $thisdefault)
            return 'publish_pages';# publish_pages is an Editor capability
        }        
   
    }else{ 
        
        if(isset($wpecus_mpt_arr['menu'][$page_name]['tabs'][$tab_key]['permissions']['capability'])){
            return $wpecus_mpt_arr['menu'][$page_name]['tabs'][$tab_key]['permissions']['capability'];    
        }else{
            return $codedefault;    
        }    
                        
    }
    return $codedefault;   
}

function wpecus_WP_SETTINGS_get_page_capability($page_name,$default = false){
    global $wpecus_mpt_arr,$wpecus_demo_mode;
    $thisdefault = 'update_core';// script default for all outcomes
    
    if($wpecus_demo_mode){
             
        if(isset($wpecus_mpt_arr['menu'][$page_name]['permissions']['demoallowed']) && $wpecus_mpt_arr['menu'][$page_name]['permissions']['demoallowed'] == false){
            // the page is not permitted to be used during demo mode
        }elseif(!isset($wpecus_mpt_arr['menu'][$page_name]['permissions']['demoallowed']) || $wpecus_mpt_arr['menu'][$page_name]['permissions']['demoallowed'] == true){
            // page permits demo use in demo mode and we are in demo mode (otherwise all arguments passed and $thisdefault)
            return 'publish_pages';# publish_pages is an Editor capability
        }
        
    }elseif(!isset($wpecus_mpt_arr['menu'][$page_name]['permissions']['customcapability']) || $default == true){
        
        // there is no capability (setup by users settings), so we check for the defaultcapability we have already hard coded as most suitable
        if(!isset($wpecus_mpt_arr['menu'][$page_name]['permissions']['capability'])){
            return $thisdefault;    
        }else{
            return $wpecus_mpt_arr['menu'][$page_name]['permissions']['capability'];// our decided default    
        }
        
    }
    
    return $thisdefault;   
}

function wpecus_WP_SETTINGS_get_version(){
    return get_option('wpecus_installedversion');    
}
         
/**
* Initiates panel array values, some of which are also values changed by settings within this function and later in panel script
*/
function wpecus_WP_SETTINGS_panel_array($pageid,$panel_number,$wpecus_tab_number){
    $form_array = array();                             
    $form_array['panel_name'] = 'defaultpanelnamepleasechange';// slug to act as a name and part of the panel ID 
    $form_array['panel_number'] = $panel_number;// number of panels counted on page, used to create object ID
    $form_array['panel_title'] = __('Default Panel Title Please Change');// user seen panel header text 
    $form_array['pageid'] = $pageid;// must be set on panels lines
    $form_array['tabnumber'] = $wpecus_tab_number; 
    $form_array['panel_id'] = $form_array['panel_name'].$panel_number;// creates a unique id, may change from version to version but within a version it should be unique
    $form_array['panel_help'] = false;// default of false causes Info button to be hidden
    $form_array['form_button'] = 'Submit';
    $form_array['dialogdisplay'] = false; // yes or no - this value when used over-rides the global setting for hiding all dialogue
    return $form_array;   
}

/**
* Updates locally stored copy of tab menu array
* 
* @uses serialize 
* @param mixed $wpecus_mpt_arr
* @return bool
*/
function wpecus_update_tabmenu(){
    global $wpecus_mpt_arr;
    return update_option('wpecus_tabmenu',serialize($wpecus_mpt_arr)); 
}

/**
* Use to start a new array.
* 1. Adds our standard ['arrayinfo'] which helps us to determine how much an array has changed since being created
* 2. Adds description also
* 
* @uses wpecus_ARRAY_arrayinfo_set()
* @param mixed $description use to explain what array is used for
* @param mixed $line __LINE__
* @param mixed $function __FUNCTION__
* @param mixed $file __FILE__
* @param mixed $reason use to explain why the array was updated (rather than what the array is used for)
* @return string
*/
function wpecus_ARRAY_init($description,$line,$function,$file,$reason,$result_array = false){
    global $wpecus_currentversion;
    $array = array();
    $array['arrayinfo'] = wpecus_ARRAY_arrayinfo_set($line,$function,$file,$reason,$wpecus_currentversion);
    $array['arrayinfo']['description'] = $description;
    
    if($result_array){
        // array is being used for a result_array meaning it has a short life, is not stored unless for debugging and the returned array values are used to build a report
        $array['outcome'] = true;// boolean
        $array['failreason'] = false;// string - our own typed reason for the failure
        $array['error'] = false;// string - add php mysql wordpress error 
        $array['parameters'] = array();// an array of the parameters passed to the function using result_array, really only required if there is a fault
        $array['result'] = array();// the result values, if result is too large not needed do not use
    }
    return $array;
}

/**
* Get arrays next key (only works with numeric key)
*/
function wpecus_get_array_nextkey($array){
    if(!is_array($array)){
        return 1;   
    }
    
    ksort($array);
    end($array);
    return key($array) + 1;
}

/**
* Gets the schedule array from wordpress option table.
* Array [times] holds permitted days and hours.
* Array [limits] holds the maximum post creation numbers 
*/
function wpecus_get_option_schedule_array(){
    $wpecus_schedule_array = get_option( 'wpecus_schedule');
    return maybe_unserialize($wpecus_schedule_array);    
}

/**
* Updates the schedule array from wordpress option table.
* Array [times] holds permitted days and hours.
* Array [limits] holds the maximum post creation numbers 
*/
function wpecus_update_option_schedule_array($schedule_array){
    $schedule_array_serialized = maybe_serialize($schedule_array);
    return update_option('wpecus_schedule',$schedule_array_serialized);    
}

function wpecus_update_option_adminsettings($wpecus_adm_set){
    $admin_settings_array_serialized = maybe_serialize($wpecus_adm_set);
    return update_option('wpecus_adminset',$admin_settings_array_serialized);    
}

/**
* Returns Wordpress version in short
* 1. Default returned example by get_bloginfo('version') is 3.6-beta1-24041
* 2. We remove everything after the first hyphen
*/
function wpecus_get_wp_version(){
    $longversion = get_bloginfo('version');
    return strstr( $longversion , '-', true );
}
?>